<?php
/**
 * CodeIgniter Controller
 *
 * @category CodeIgniter
 * @license /_licenses/untermStrich/agb_normal.pdf
 * @author Christian Koller
 * @copyright 2009 untermStrich
 * @package controllers
 * @version $Id: server_office_pdf.php 19094 2019-07-01 13:50:38Z christian.w $
*/

/**
 * The process server functions controller of untermStrich
 *
 * Place everything into this controller, which runs on the process server
 */
class Process_server extends My_Controller
{
    
    protected $folder;
    protected $process_server_mode;
    
    /**
     * Constructor
     */
    public function __construct()
    {
        $this->in_simple_mode = true;
        $this->in_no_db_mode = true;
        $this->non_persistent_session = true;
        parent::__construct();
        $this->folder = $this->config->item('us_servertempdir').'/';
        $this->process_server_mode = $this->config->item('us_process_server_mode');
    }
    
    /**
     * Server process
     *
     * @return void
     */
    public function _remap($uri, $data)
    {
        if (php_sapi_name()!=='cli-server') {
            echo "Ungültiger Aufruf! \n\n Bitte rufen Sie diese Funktion mittels <tt>php -S 0.0.0.0:17340 process_server.php</tt> auf. \n\n";
            die(1);
        }
        
        switch ($uri) {
            case 'pdf_done':
                $this->pdf_done($data[0]);
                break;
            case 'pdf':
                $this->pdf();
            default:
                break;
            case 'index':
                $this->index();
                break;
        }
    }
    
    public function index()
    {
        $info = array(
            'process_server_mode' => $this->process_server_mode,
            'php_os' => strtoupper(substr(PHP_OS, 0, 3)),
            'libreoffice_path' => $this->config->item('us_libreoffice_path'),
            'libreoffice_set_temp' => $this->config->item('us_libreoffice_set_temp'),
            'temp_file_dir' => $this->config->item('us_tempdir'),
            'extension_loaded com_dotnet' => extension_loaded('com_dotnet'),
            'memory_limit' => ini_get('memory_limit'),
            'max_input_time' => ini_get('max_input_time'),
            'post_max_size' => ini_get('post_max_size'),
            'upload_max_filesize' => ini_get('upload_max_filesize')
        );
        
        $reply = new stdClass();
        $reply->msg = 'Willkommen!';
        if ($this->config->item('us_process_server_debug')===true) {
            $reply->info = $info;
        }
        $this->_write_reply($reply);
    }
    
    /**
     * Run process
     */
    public function pdf()
    {
        $reply = new stdClass();
        $reply->rnd = bin2hex(random_bytes(32));
        
        if (!@is_dir($this->folder)) {
            @mkdir($this->folder, DIR_WRITE_MODE, true);
        }
        
        $config['upload_path'] = $this->folder;
        $config['allowed_types'] = 'docx|docm|xlsx|xlsm|odt|ods|';
        $config['file_ext_tolower'] = true;
        $config['file_name'] = $reply->rnd;

        $this->load->library('upload', $config);

        if (!$this->upload->do_upload('file')) {
            $reply->error = $this->upload->display_errors();
            $reply->error_code = 400;
            $this->_write_reply($reply);
            return;
        }

        $data = $this->upload->data();
        $upload_full_path = $data['full_path'];
        $upload_file_ext = trim($data['file_ext'], ' .');
        
        switch ($upload_file_ext) {
            case 'docx':
            case 'docm':
            case 'xlsx':
            case 'xlsm':
                if (strtoupper(substr(PHP_OS, 0, 3))==='WIN') {
                    switch ($this->process_server_mode) {
                        case 'libreoffice':
                            list($converter_info, $pdf_convert) = $this->load->pdf_converter('libreoffice');
                            $reply->converter = 'libreoffice';
                            break;
                        case 'auto':
                        default:
                            list($converter_info, $pdf_convert) = $this->load->pdf_converter('msoffice_com');
                            $reply->converter = 'msoffice';
                    }
                } else {
                    list($converter_info, $pdf_convert) = $this->load->pdf_converter('libreoffice');
                    $reply->converter = 'libreoffice';
                }
                break;
            case 'odt':
            case 'ods':
                if (strtoupper(substr(PHP_OS, 0, 3))==='WIN') {
                    switch ($this->process_server_mode) {
                        case 'msoffice':
                            list($converter_info, $pdf_convert) = $this->load->pdf_converter('msoffice_com');
                            $reply->converter = 'msoffice';
                            break;
                        case 'auto':
                        default:
                            list($converter_info, $pdf_convert) = $this->load->pdf_converter('libreoffice');
                            $reply->converter = 'libreoffice';
                    }
                } else {
                    list($converter_info, $pdf_convert) = $this->load->pdf_converter('libreoffice');
                    $reply->converter = 'libreoffice';
                }
                break;
            default:
                $reply->error = 'Invalid file extension. Only use '. str_replace('|', ',', $config['allowed_types']);
                $reply->error_code = 500;
                $this->_write_reply($reply);
                return;
        }
        
        try {
            $error = '';
            $temp_file_path = $pdf_convert($upload_full_path, $upload_file_ext, '', '', $error);
            if ($temp_file_path===false || $error!=='') {
                $reply->error = $error;
                $reply->error_code = 500;
                $this->_write_reply($reply);
                return;
            }
            
            rename($temp_file_path, $this->folder.$reply->rnd.'.pdf');

            if (!unlink($upload_full_path)) {
                log_message('error', 'Cannot delete '.$upload_full_path);
            }

            $this->load->helper('url');
            $reply->url = site_url('pdf_done/'.$reply->rnd);
            $this->_write_reply($reply);
        } catch (Exception $exc) {
            $reply->error = 'Error processing file: '.$exc->getMessage();
            $reply->error_code = 500;
            $this->_write_reply($reply);
        }
    }
    
    /**
     * Run done
     * 
     * @param   string      $uri        URI part
     */
    public function pdf_done($uri)
    {
        $filename = preg_replace("/[^A-Za-z0-9]/", '', $uri);
        $pdf_full_path = $this->folder.$filename.'.pdf';
        
        if (!file_exists($pdf_full_path)) {
            $reply = new stdClass();
            $reply->error = 'Invalid or old token';
            $reply->error_code = 404;
            $this->_write_reply($reply);
            return;
        }
        
        header('Content-Type: application/pdf');
        header('Content-Disposition: attachment; filename="downloaded.pdf"');
        readfile($pdf_full_path);
        
        if (!unlink($pdf_full_path)) {
            log_message('error', 'Cannot delete '.$pdf_full_path);
        }
    }
    
    
    /**
     * Write reply
     *
     * @param  object       $reply      Reply to write
     * @return void
     */
    private function _write_reply($reply)
    {
        $msg = json_encode($reply, JSON_HEX_TAG|JSON_HEX_APOS|JSON_HEX_AMP);
        if (isset($reply->error_code) && !headers_sent()) {
            header('Error', true, $reply->error_code);
            log_message('error', $msg);
        }
        header('Content-Type: application/json');
        echo $msg;
    }
    
} // END class Server_office_pdf