<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet
    version="1.0"
    xmlns:xhtml="http://www.w3.org/1999/xhtml"
    xmlns="http://www.w3.org/1999/xhtml"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    exclude-result-prefixes="xhtml xsl"
    >
    <xsl:import href="_html-style.xslt" />
    <xsl:import href="_html-basicinfo.xslt" />
    <xsl:import href="_html-footer.xslt" />
    
    <xsl:param name="background_color_subheading" />
    <xsl:param name="border_color_table" />
    <xsl:param name="font_global_html" />
    <xsl:param name="font_size_global" />
    <xsl:param name="font_weight_heading" />
    <xsl:param name="page_header" />
    <xsl:param name="page_footer" />
    <xsl:param name="company_currency" />
    <xsl:param name="us_version" />

    <xsl:output
        method="html"
        version="1.0"
        encoding="UTF-8"
        doctype-public="-//W3C//DTD XHTML 1.0 Strict//EN"
        doctype-system="http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd"
        indent="yes"
        />

    <xsl:template match="/">
        <html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">
            <head>
                <title><xsl:value-of select="/report/head/title"/> - <xsl:value-of select="$page_owner" disable-output-escaping="yes"/></title>
                <meta charset="utf-8" />
                <meta http-equiv="content-type" content="text/html; charset=utf-8"/>
                <meta http-equiv="content-script-type" content="text/javascript" />
                <meta http-equiv="content-style-type" content="text/css" />
                <meta http-equiv="content-language" content="de-AT" />
                <meta name="author">
                    <xsl:attribute name="content">
                        <xsl:text>untermStrich </xsl:text>
                        <xsl:value-of select="$us_version" />
                    </xsl:attribute>
                </meta>
                <base>
                    <xsl:attribute name="href">
                        <xsl:value-of select="$base_url" />
                    </xsl:attribute>
                </base>
                <xsl:apply-imports/> <!-- Add the format information -->
            </head>
            <body>
                <pd4ml-page-header>
                    <xsl:value-of select="$page_header" disable-output-escaping="yes"/>
                </pd4ml-page-header>
                <span class="only_pdf">
                    <pd4ml-page-footer>
                        <xsl:value-of select="$page_footer" disable-output-escaping="yes"/>
                    </pd4ml-page-footer>
                </span>
                <xsl:apply-templates select="/report"/>
                <pd4ml-page-footer>
                    <xsl:value-of select="$page_footer" disable-output-escaping="yes"/>
                </pd4ml-page-footer>
                <xsl:call-template name="footer"/> <!-- Add the footer -->
            </body>
        </html>
    </xsl:template>

    <xsl:template match="/report">
        <xsl:apply-templates select="head"/> <!-- Add the head information -->
        <table>
            <xsl:attribute name="style">
                <xsl:text>border-color:</xsl:text>
                <xsl:value-of select="$border_color_table"/>
                <xsl:text>;</xsl:text>
            </xsl:attribute>
            
            <xsl:apply-templates select="head/columns"/> <!-- Heading -->
            <tbody>
                <xsl:apply-templates select="data/row"/> <!-- Values -->
            </tbody>
        </table>
        <xsl:apply-templates select="data/rowtotals"/> <!-- Totals -->
    </xsl:template>

    <xsl:template match="head/columns"> <!-- Heading -->
        <!-- Width -->
        <colgroup>
            <xsl:for-each select="*[local-name()='field']">
                <col>
                    <xsl:attribute name="style">
                        <xsl:text>width:</xsl:text>
                        <xsl:value-of select="@width"/>
                        <xsl:text>mm;</xsl:text>
                    </xsl:attribute>
                    <xsl:attribute name="class">
                        <xsl:value-of select="@class"/>
                    </xsl:attribute>
                </col>
            </xsl:for-each>
        </colgroup>
        
        <!-- Data -->
        <thead>
            <tr>
                <xsl:for-each select="*[local-name()='field']">
                    <th>
                        <xsl:attribute name="style">
                            <xsl:text>background-color:</xsl:text>
                            <xsl:value-of select="$background_color_subheading"/>
                            <xsl:text>;font-weight:</xsl:text>
                            <xsl:value-of select="$font_weight_heading"/>
                            <xsl:text>;text-align:center;</xsl:text>
                        </xsl:attribute>
                        <xsl:attribute name="class">
                            <xsl:value-of select="@class"/>
                        </xsl:attribute>
                        <xsl:value-of select="."/>
                    </th>
                </xsl:for-each>
            </tr>
        </thead>
    </xsl:template>

    <xsl:template match="data/row"> <!-- Values -->
        <tr>
            <xsl:for-each select="*">
                <xsl:variable name="current" select="@name"/>
                <xsl:variable name="fieldinfo" select="/report/head/columns/*[@name=$current]"/>
                <xsl:variable name="type" select="$fieldinfo/@type"/>
                <xsl:variable name="action" select="$fieldinfo/@action"/>
                <xsl:variable name="button_id" select="."/>
                <td>
                    <xsl:attribute name="class">
                        <xsl:value-of select="$fieldinfo/@class"/>
                    </xsl:attribute>
                    <xsl:choose>
                        <xsl:when test="$type='number'">
                            <xsl:attribute name="style">text-align:right;</xsl:attribute>
                            <xsl:value-of select="@usnumber"/>
                        </xsl:when>
                        <xsl:when test="$type='number2'">
                            <xsl:attribute name="style">text-align:right;</xsl:attribute>
                            <xsl:value-of select="@usnumber"/>
                        </xsl:when>
                        <xsl:when test="$type='currency'">
                            <xsl:attribute name="style">text-align:right;</xsl:attribute>
                            <xsl:value-of select="@usnumber"/>
                            <xsl:text> </xsl:text>
                            <xsl:value-of select="$company_currency"/>
                        </xsl:when>
                        <xsl:when test="$type='bool'">
                            <xsl:attribute name="style">text-align:center;</xsl:attribute>
                            <xsl:if test=". = 'true'">
                                <img alt="Yes">
                                    <xsl:attribute name="style">
                                        width:<xsl:value-of select="$font_size_global"/>pt;
                                        height:<xsl:value-of select="$font_size_global"/>pt;
                                    </xsl:attribute>
                                    <xsl:attribute name="src">
                                        <xsl:value-of select="$bool_yes_img"/>
                                    </xsl:attribute>
                                </img>
                            </xsl:if>
                            <xsl:if test=". = 'false'">
                                <img alt="No">
                                    <xsl:attribute name="style">
                                        width:<xsl:value-of select="$font_size_global"/>pt;
                                        height:<xsl:value-of select="$font_size_global"/>pt;
                                    </xsl:attribute>
                                    <xsl:attribute name="src">
                                        <xsl:value-of select="$bool_no_img"/>
                                    </xsl:attribute>
                                </img>
                            </xsl:if>
                        </xsl:when>
                        <xsl:when test="$type='date'">
                            <xsl:attribute name="style">text-align:right;</xsl:attribute>
                            <xsl:value-of select="@usdate"/>
                        </xsl:when>
                        <xsl:when test="$type='time'">
                            <xsl:attribute name="style">text-align:right;</xsl:attribute>
                            <xsl:value-of select="@ustime"/>
                        </xsl:when>
                        <xsl:when test="$type='timespan'">
                            <xsl:attribute name="style">text-align:right;</xsl:attribute>
                            <xsl:value-of select="@ustimespan"/>
                        </xsl:when>
                        <xsl:when test="$type='button' and $button_id!=''">
                            <button>
                                <xsl:attribute name="onclick">
                                    <xsl:choose>
                                        <xsl:when test="$action='delete'">
                                        if(!parent.basicForm.requestBigConfirm('')) return false;
                                        </xsl:when>
                                    </xsl:choose>   
                                    parent.openx('reporting', 'process_button', '<xsl:value-of select="/report/head/title/@id"/>', '<xsl:value-of select="$action"/>', '<xsl:value-of select="."/>')
                                </xsl:attribute>
                                <xsl:attribute name="title">
                                    <xsl:value-of select="."/>
                                </xsl:attribute>
                                <xsl:value-of select="$fieldinfo"/>
                            </button>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:attribute name="style">text-align:left;</xsl:attribute>
                            <xsl:value-of select="."/>
                        </xsl:otherwise>
                    </xsl:choose>
                </td>
            </xsl:for-each>
        </tr>
    </xsl:template>

    <xsl:template match="data/rowtotals"> <!-- Totals -->
        <table>
            <xsl:attribute name="style">
                <xsl:text>border-color:</xsl:text>
                <xsl:value-of select="$border_color_table"/>
                <xsl:text>;</xsl:text>
            </xsl:attribute>
            <tr>
                <xsl:for-each select="*">
                    <xsl:variable name="current" select="@name"/>
                    <xsl:variable name="fieldinfo" select="/report/head/columns/total/field[@name=$current]"/>
                    <td>
                        <xsl:attribute name="style">
                            <xsl:text>font-weight:</xsl:text>
                            <xsl:value-of select="$font_weight_heading"/>
                            <xsl:text>;width:30mm;</xsl:text>
                        </xsl:attribute>
                        <xsl:attribute name="class">
                            <xsl:value-of select="$fieldinfo/@class"/>
                        </xsl:attribute>
                        <p>
                            <xsl:value-of select="$fieldinfo/."/><xsl:text>:</xsl:text>
                        </p>
                    </td>
                    <td>
                        <xsl:attribute name="style">
                            <xsl:text>font-weight:</xsl:text>
                            <xsl:value-of select="$font_weight_heading"/>
                            <xsl:text>;</xsl:text>
                        </xsl:attribute>
                        <p>
                            <xsl:attribute name="class">
                                <xsl:value-of select="$fieldinfo/@class"/>
                            </xsl:attribute>
                            <xsl:choose>
                                <xsl:when test="@type='number'">
                                    <xsl:attribute name="style">text-align:right;width:30mm;</xsl:attribute>
                                    <xsl:value-of select="@usnumber"/>
                                </xsl:when>
                                <xsl:when test="@type='number2'">
                                    <xsl:attribute name="style">text-align:right;width:30mm;</xsl:attribute>
                                    <xsl:value-of select="@usnumber"/>
                                </xsl:when>
                                <xsl:when test="@type='currency'">
                                    <xsl:attribute name="style">text-align:right;width:30mm;</xsl:attribute>
                                    <xsl:value-of select="@usnumber"/>
                                    <xsl:text> </xsl:text>
                                    <xsl:value-of select="$company_currency"/>
                                </xsl:when>
                                <xsl:when test="@type='date'">
                                    <xsl:attribute name="style">text-align:right;width:30mm;</xsl:attribute>
                                    <xsl:value-of select="@usdate"/>
                                </xsl:when>
                                <xsl:when test="@type='time'">
                                    <xsl:attribute name="style">text-align:right;width:30mm;</xsl:attribute>
                                    <xsl:value-of select="@ustime"/>
                                </xsl:when>
                                <xsl:when test="@type='timespan'">
                                    <xsl:attribute name="style">text-align:right;width:30mm;</xsl:attribute>
                                    <xsl:value-of select="@ustimespan"/>
                                </xsl:when>
                                <xsl:otherwise>
                                    <xsl:attribute name="style">text-align:left;width:30mm;</xsl:attribute>
                                    <xsl:value-of select="."/>
                                </xsl:otherwise>
                            </xsl:choose>
                        </p>
                    </td>
                </xsl:for-each>
            </tr>
        </table>
    </xsl:template>

</xsl:stylesheet>