<xsl:stylesheet version="1.0"
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:office="urn:oasis:names:tc:opendocument:xmlns:office:1.0"
                xmlns:style="urn:oasis:names:tc:opendocument:xmlns:style:1.0"
                xmlns:text="urn:oasis:names:tc:opendocument:xmlns:text:1.0"
                xmlns:table="urn:oasis:names:tc:opendocument:xmlns:table:1.0"
                xmlns:draw="urn:oasis:names:tc:opendocument:xmlns:drawing:1.0"
                xmlns:fo="urn:oasis:names:tc:opendocument:xmlns:xsl-fo-compatible:1.0"
                xmlns:xlink="http://www.w3.org/1999/xlink"
                xmlns:dc="http://purl.org/dc/elements/1.1/"
                xmlns:meta="urn:oasis:names:tc:opendocument:xmlns:meta:1.0"
                xmlns:number="urn:oasis:names:tc:opendocument:xmlns:datastyle:1.0"
                xmlns:svg="urn:oasis:names:tc:opendocument:xmlns:svg-compatible:1.0"
                xmlns:chart="urn:oasis:names:tc:opendocument:xmlns:chart:1.0"
                xmlns:dr3d="urn:oasis:names:tc:opendocument:xmlns:dr3d:1.0"
                xmlns:math="http://www.w3.org/1998/Math/MathML"
                xmlns:form="urn:oasis:names:tc:opendocument:xmlns:form:1.0"
                xmlns:script="urn:oasis:names:tc:opendocument:xmlns:script:1.0"
                xmlns:ooo="http://openoffice.org/2004/office"
                xmlns:ooow="http://openoffice.org/2004/writer"
                xmlns:oooc="http://openoffice.org/2004/calc"
                xmlns:dom="http://www.w3.org/2001/xml-events"
                xmlns:xforms="http://www.w3.org/2002/xforms"
                xmlns:xsd="http://www.w3.org/2001/XMLSchema"
                xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
                >
    <xsl:import href="_ods-style.xslt" />
    <xsl:param name="company_currency" />

    <xsl:output
        method="xml"
        encoding="UTF-8"
        indent="yes"
        />

    <xsl:template match="*">
        <office:document-content office:version="1.0">
            <office:scripts/>
            <office:font-face-decls>
                <style:font-face style:name="Arial" svg:font-family="Arial" style:font-family-generic="swiss" style:font-pitch="variable"/>
            </office:font-face-decls>
            <office:automatic-styles>
                <xsl:call-template name="columnswidthformat"/> <!-- Width of columns format -->
                <xsl:apply-imports/> <!-- Add the format information -->
            </office:automatic-styles>
            <office:body>
                <office:spreadsheet>
                    <table:table table:name="Report" table:print="false">
                        <xsl:call-template name="columnswidthassign"/> <!-- Width of columns assign -->
                        <table:table-row>
                            <xsl:apply-templates select="/report/head/columns/*[local-name()='field']"/> <!-- Heading -->
                        </table:table-row>
                        <xsl:apply-templates select="/report/data"/> <!-- Values -->
                        <xsl:apply-templates select="data/rowtotals"/> <!-- Totals -->
                    </table:table>
                </office:spreadsheet>
            </office:body>
        </office:document-content>
    </xsl:template>

    <xsl:template name="columnswidthformat"> <!-- Width of columns format -->
        <xsl:for-each select="/report/head/columns/*[local-name()='field']">
            <style:style style:family="table-column">
                <xsl:attribute name="style:name">column_<xsl:value-of select="@name"/></xsl:attribute>
                <style:table-column-properties fo:break-before="auto">
                    <xsl:choose>
                        <xsl:when test="@width > 0">
                            <xsl:attribute name="style:column-width"><xsl:value-of select="@width div 7"/>cm</xsl:attribute>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:attribute name="style:column-width">2cm</xsl:attribute>
                        </xsl:otherwise>
                    </xsl:choose>
                </style:table-column-properties>
            </style:style>
        </xsl:for-each>
    </xsl:template>

    <xsl:template name="columnswidthassign"> <!-- Width of columns assign -->
        <xsl:for-each select="/report/head/columns/*[local-name()='field']">
            <table:table-column table:default-cell-style-name="Default">
                <xsl:attribute name="table:style-name">column_<xsl:value-of select="@name"/></xsl:attribute>
            </table:table-column>
        </xsl:for-each>
    </xsl:template>


    <xsl:template match="/report/head/columns/*[local-name()='field']"> <!-- Heading -->
        <table:table-cell office:value-type="string" table:number-rows-spanned="1" table:style-name="heading">
            <text:p><xsl:value-of select="."/></text:p>
        </table:table-cell>
    </xsl:template>

    <xsl:template match="/report/data"> <!-- Calls values -->
        <xsl:for-each select="row">
            <xsl:call-template name="values" />
        </xsl:for-each>
    </xsl:template>

    <xsl:template name="values"> <!-- Values -->
        <table:table-row>
            <xsl:apply-templates select="*[local-name()='cell']"/>
        </table:table-row>
    </xsl:template>

    <xsl:template match="data/rowtotals"> <!-- Totals -->
        <table:table-row>
            <table:table-cell office:value-type="string" table:number-rows-spanned="1" table:style-name="total0">
                <xsl:attribute name="table:number-columns-spanned"><xsl:value-of select="count(/report/head/columns/*[local-name()='field'])"/></xsl:attribute>
                <text:p>
                    <xsl:for-each select="cell">
                        <xsl:variable name="current" select="@name"/>
                        <xsl:call-template name="tablecell">
                            <xsl:with-param name="fieldinfo" select="/report/head/columns/total/field[@name=$current]"/>
                        </xsl:call-template>
                    </xsl:for-each>
                </text:p>
            </table:table-cell>
        </table:table-row>
    </xsl:template>

    <xsl:include href="_ods-value.xslt" />

</xsl:stylesheet>