<xsl:stylesheet version="1.0"
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns="urn:schemas-microsoft-com:office:spreadsheet"
                xmlns:o="urn:schemas-microsoft-com:office:office"
                xmlns:x="urn:schemas-microsoft-com:office:excel"
                xmlns:ss="urn:schemas-microsoft-com:office:spreadsheet"
                xmlns:c="urn:schemas-microsoft-com:office:component:spreadsheet"
                xmlns:html="http://www.w3.org/TR/REC-html40">

    <xsl:param name="background_color_subheading" />
    <xsl:param name="border_color_table" />
    <xsl:param name="font_global" />
    <xsl:param name="font_size_global" />
    <xsl:param name="font_weight_heading" />
    <xsl:param name="company_currency" />

    <xsl:output
        method="xml"
        version="1.0"
        encoding="UTF-8"
        indent="yes"
        media-type="vnd.openxmlformats-officedocument.spreadsheetml.sheet"
        />

    <xsl:template match="*">

        <xsl:processing-instruction name="mso-application">
            <xsl:text>progid="Excel.Sheet"</xsl:text>
        </xsl:processing-instruction>

        <Workbook>
            <!-- Some formating styles -->
            <Styles>
                <Style ss:ID="Default" ss:Name="Normal">
                    <Alignment ss:Horizontal="Left" ss:Vertical="Top"/>
                    <Font>
                        <xsl:attribute name="ss:FontName"><xsl:value-of select="$font_global"/></xsl:attribute>
                        <xsl:attribute name="ss:Size"><xsl:value-of select="$font_size_global"/></xsl:attribute>
                    </Font>
                </Style>
                <Style ss:ID="head">
                    <Alignment ss:Horizontal="Center" ss:Vertical="Top"/>
                    <Font>
                        <xsl:attribute name="ss:FontName"><xsl:value-of select="$font_global"/></xsl:attribute>
                        <xsl:attribute name="ss:Size"><xsl:value-of select="$font_size_global"/></xsl:attribute>
                        <xsl:if test="$font_weight_heading='bold'">
                            <xsl:attribute name="ss:Bold">1</xsl:attribute>
                        </xsl:if>
                    </Font>
                    <Interior ss:Pattern="Solid">
                        <xsl:attribute name="ss:Color"><xsl:value-of select="$background_color_heading"/></xsl:attribute>
                    </Interior>
                    <Borders>
                        <Border ss:Position="Top" ss:LineStyle="Continuous" ss:Weight="1">
                            <xsl:attribute name="ss:Color"><xsl:value-of select="$border_color_table"/></xsl:attribute>
                        </Border>
                        <Border ss:Position="Right" ss:LineStyle="Continuous" ss:Weight="1">
                            <xsl:attribute name="ss:Color"><xsl:value-of select="$border_color_table"/></xsl:attribute>
                        </Border>
                        <Border ss:Position="Bottom" ss:LineStyle="Continuous" ss:Weight="1">
                            <xsl:attribute name="ss:Color"><xsl:value-of select="$border_color_table"/></xsl:attribute>
                        </Border>
                        <Border ss:Position="Left" ss:LineStyle="Continuous" ss:Weight="1">
                            <xsl:attribute name="ss:Color"><xsl:value-of select="$border_color_table"/></xsl:attribute>
                        </Border>
                    </Borders>
                </Style>

                <Style ss:ID="content">
                    <Alignment ss:Horizontal="Left" ss:Vertical="Top"/>
                    <Borders>
                        <Border ss:Position="Right" ss:LineStyle="Continuous" ss:Weight="1">
                            <xsl:attribute name="ss:Color"><xsl:value-of select="$border_color_table"/></xsl:attribute>
                        </Border>
                        <Border ss:Position="Bottom" ss:LineStyle="Continuous" ss:Weight="1">
                            <xsl:attribute name="ss:Color"><xsl:value-of select="$border_color_table"/></xsl:attribute>
                        </Border>
                        <Border ss:Position="Left" ss:LineStyle="Continuous" ss:Weight="1">
                            <xsl:attribute name="ss:Color"><xsl:value-of select="$border_color_table"/></xsl:attribute>
                        </Border>
                    </Borders>
                </Style>
                <Style ss:ID="usnumber">
                    <Alignment ss:Horizontal="Right" ss:Vertical="Top"/>
                    <NumberFormat ss:Format="Fixed"/>
                    <Borders>
                        <Border ss:Position="Right" ss:LineStyle="Continuous" ss:Weight="1">
                            <xsl:attribute name="ss:Color"><xsl:value-of select="$border_color_table"/></xsl:attribute>
                        </Border>
                        <Border ss:Position="Bottom" ss:LineStyle="Continuous" ss:Weight="1">
                            <xsl:attribute name="ss:Color"><xsl:value-of select="$border_color_table"/></xsl:attribute>
                        </Border>
                        <Border ss:Position="Left" ss:LineStyle="Continuous" ss:Weight="1">
                            <xsl:attribute name="ss:Color"><xsl:value-of select="$border_color_table"/></xsl:attribute>
                        </Border>
                    </Borders>
                </Style>
                <Style ss:ID="uscurrency">
                    <Alignment ss:Horizontal="Right" ss:Vertical="Top"/>
                    <NumberFormat ss:Format="Fixed"/>
                    <Borders>
                        <Border ss:Position="Right" ss:LineStyle="Continuous" ss:Weight="1">
                            <xsl:attribute name="ss:Color"><xsl:value-of select="$border_color_table"/></xsl:attribute>
                        </Border>
                        <Border ss:Position="Bottom" ss:LineStyle="Continuous" ss:Weight="1">
                            <xsl:attribute name="ss:Color"><xsl:value-of select="$border_color_table"/></xsl:attribute>
                        </Border>
                        <Border ss:Position="Left" ss:LineStyle="Continuous" ss:Weight="1">
                            <xsl:attribute name="ss:Color"><xsl:value-of select="$border_color_table"/></xsl:attribute>
                        </Border>
                    </Borders>
                </Style>
                <Style ss:ID="usdate">
                    <Alignment ss:Horizontal="Right" ss:Vertical="Top"/>
                    <NumberFormat ss:Format="Short Date"/>
                    <Borders>
                        <Border ss:Position="Right" ss:LineStyle="Continuous" ss:Weight="1">
                            <xsl:attribute name="ss:Color"><xsl:value-of select="$border_color_table"/></xsl:attribute>
                        </Border>
                        <Border ss:Position="Bottom" ss:LineStyle="Continuous" ss:Weight="1">
                            <xsl:attribute name="ss:Color"><xsl:value-of select="$border_color_table"/></xsl:attribute>
                        </Border>
                        <Border ss:Position="Left" ss:LineStyle="Continuous" ss:Weight="1">
                            <xsl:attribute name="ss:Color"><xsl:value-of select="$border_color_table"/></xsl:attribute>
                        </Border>
                    </Borders>
                </Style>
                <Style ss:ID="ustime">
                    <Alignment ss:Horizontal="Right" ss:Vertical="Top"/>
                    <NumberFormat ss:Format="Short Time"/>
                    <Borders>
                        <Border ss:Position="Right" ss:LineStyle="Continuous" ss:Weight="1">
                            <xsl:attribute name="ss:Color"><xsl:value-of select="$border_color_table"/></xsl:attribute>
                        </Border>
                        <Border ss:Position="Bottom" ss:LineStyle="Continuous" ss:Weight="1">
                            <xsl:attribute name="ss:Color"><xsl:value-of select="$border_color_table"/></xsl:attribute>
                        </Border>
                        <Border ss:Position="Left" ss:LineStyle="Continuous" ss:Weight="1">
                            <xsl:attribute name="ss:Color"><xsl:value-of select="$border_color_table"/></xsl:attribute>
                        </Border>
                    </Borders>
                </Style>
                <Style ss:ID="ustimespan">
                    <Alignment ss:Horizontal="Right" ss:Vertical="Top"/>
                    <NumberFormat ss:Format="0"/>
                    <Borders>
                        <Border ss:Position="Right" ss:LineStyle="Continuous" ss:Weight="1">
                            <xsl:attribute name="ss:Color"><xsl:value-of select="$border_color_table"/></xsl:attribute>
                        </Border>
                        <Border ss:Position="Bottom" ss:LineStyle="Continuous" ss:Weight="1">
                            <xsl:attribute name="ss:Color"><xsl:value-of select="$border_color_table"/></xsl:attribute>
                        </Border>
                        <Border ss:Position="Left" ss:LineStyle="Continuous" ss:Weight="1">
                            <xsl:attribute name="ss:Color"><xsl:value-of select="$border_color_table"/></xsl:attribute>
                        </Border>
                    </Borders>
                </Style>
            </Styles>
            <!-- There is only one worksheet in this workbook -->
            <Worksheet ss:Name="Report">
                <Table>
                    <xsl:for-each select="/report/head/columns/*[local-name()='field']"> <!-- Column width -->
                        <Column ss:AutoFitWidth="1">
                            <xsl:attribute name="ss:Width"><xsl:value-of select="@width * 4"/></xsl:attribute>
                        </Column>
                    </xsl:for-each>
                    <Row>
                        <xsl:apply-templates select="/report/head/columns/*[local-name()='field']"/> <!-- Heading -->
                    </Row>
                    <xsl:apply-templates select="/report/data/row"/> <!-- Values -->
                    <xsl:apply-templates select="data/rowtotals"/> <!-- Totals -->
                </Table>
                <WorksheetOptions xmlns="urn:schemas-microsoft-com:office:excel">
                    <FrozenNoSplit/>
                    <SplitHorizontal>1</SplitHorizontal>
                    <TopRowBottomPane>1</TopRowBottomPane>
                </WorksheetOptions>
            </Worksheet>
        </Workbook>
    </xsl:template>

    <xsl:template match="/report/head/columns/*[local-name()='field']"> <!-- Heading -->
        <Cell ss:StyleID="head">
            <Data ss:Type="String">
                <xsl:value-of select="."/>
            </Data>
        </Cell>
    </xsl:template>

    <xsl:template match="/report/data/row"> <!-- Values -->
        <Row>
            <xsl:for-each select="*">
                <Cell>
                    <xsl:choose>
                        <xsl:when test="@type='number'">
                            <xsl:attribute name="ss:StyleID">usnumber</xsl:attribute>
                            <Data ss:Type="Number">
                                <xsl:value-of select="."/>
                            </Data>
                        </xsl:when>
                        <xsl:when test="@type='number2'">
                            <xsl:attribute name="ss:StyleID">usnumber</xsl:attribute>
                            <Data ss:Type="Number">
                                <xsl:value-of select="."/>
                            </Data>
                        </xsl:when>
                        <xsl:when test="@type='currency'">
                            <xsl:attribute name="ss:StyleID">uscurrency</xsl:attribute>
                            <Data ss:Type="String">
                                <xsl:value-of select="@usnumber"/>
                                <xsl:text> </xsl:text>
                                <xsl:value-of select="$company_currency"/>
                            </Data>
                        </xsl:when>
                        <xsl:when test="@type='date'">
                            <xsl:attribute name="ss:StyleID">usdate</xsl:attribute>
                            <Data ss:Type="DateTime">
                                <xsl:value-of select="@xlsdate"/>
                            </Data>
                        </xsl:when>
                        <xsl:when test="@type='time'">
                            <xsl:attribute name="ss:StyleID">ustime</xsl:attribute>
                            <xsl:choose>
                                <xsl:when test="@xlsdate = ''">
                                    <Data ss:Type="String"/>
                                </xsl:when>
                                <xsl:otherwise>
                                    <Data ss:Type="DateTime">
                                        <xsl:value-of select="@xlsdate"/>
                                    </Data>
                                </xsl:otherwise>
                            </xsl:choose>
                        </xsl:when>
                        <xsl:when test="@type='timespan'">
                            <xsl:attribute name="ss:StyleID">ustimespan</xsl:attribute>
                            <Data ss:Type="Number">
                                <xsl:value-of select="."/>
                            </Data>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:attribute name="ss:StyleID">content</xsl:attribute>
                            <Data ss:Type="String">
                                <xsl:value-of select="."/>
                            </Data>
                        </xsl:otherwise>
                    </xsl:choose>
                </Cell>
            </xsl:for-each>
        </Row>
    </xsl:template>

    <xsl:template match="data/rowtotals"> <!-- Totals -->
        <Row>
            <Cell ss:StyleID="content">
                <xsl:attribute name="ss:MergeAcross"><xsl:value-of select="count(/report/head/columns/*[local-name()='field'])-1"/></xsl:attribute>
                <Data ss:Type="String">
                    <xsl:for-each select="cell">
                        <xsl:text> </xsl:text>
                        <xsl:variable name="current" select="@name"/>
                        <xsl:variable name="fieldinfo" select="/report/head/columns/total/field[@name=$current]"/>
                        <xsl:value-of select="$fieldinfo/."/><xsl:text>: </xsl:text>
                        <xsl:choose>
                            <xsl:when test="@type='number'">
                                <xsl:value-of select="@usnumber"/>
                            </xsl:when>
                            <xsl:when test="@type='number2'">
                                <xsl:value-of select="@usnumber"/>
                            </xsl:when>
                            <xsl:when test="@type='currency'">
                                <xsl:value-of select="@usnumber"/>
                                <xsl:text> </xsl:text>
                                <xsl:value-of select="$company_currency"/>
                            </xsl:when>
                            <xsl:when test="@type='date'">
                                <xsl:value-of select="@usdate"/>
                            </xsl:when>
                            <xsl:when test="@type='time'">
                                <xsl:value-of select="@ustime"/>
                            </xsl:when>
                            <xsl:when test="@type='timespan'">
                                <xsl:value-of select="@ustimespan"/>
                            </xsl:when>
                            <xsl:otherwise>
                                <xsl:value-of select="."/>
                            </xsl:otherwise>
                        </xsl:choose>
                        <xsl:text> </xsl:text>
                    </xsl:for-each>
                </Data>
            </Cell>
        </Row>
    </xsl:template>

</xsl:stylesheet>