<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet
    version="1.0"
    xmlns:xhtml="http://www.w3.org/1999/xhtml"
    xmlns="http://www.w3.org/1999/xhtml"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    exclude-result-prefixes="xhtml xsl"
    >
    <xsl:import href="_html-style.xslt" />
    <xsl:import href="_html-basicinfo.xslt" />
    <xsl:import href="_html-footer.xslt" />

    <xsl:param name="background_color_heading" />
    <xsl:param name="background_color_subheading" />
    <xsl:param name="border_color_table" />
    <xsl:param name="font_global_html" />
    <xsl:param name="font_size_global" />
    <xsl:param name="font_weight_heading" />
    <xsl:param name="font_weight_subheading" />
    <xsl:param name="page_header" />
    <xsl:param name="page_footer" />
    <xsl:param name="company_currency" />
    <xsl:param name="us_version" />

    <xsl:output
        method="html"
        version="1.0"
        encoding="UTF-8"
        doctype-public="-//W3C//DTD XHTML 1.0 Strict//EN"
        doctype-system="http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd"
        indent="yes"
        />

    <xsl:variable name="fullcollspan_real"><xsl:value-of select="count(/preprocessedreport/head/columns/descendant::field/parent::group/field)"/></xsl:variable>
    <xsl:variable name="fullcollspan">
        <xsl:choose>
            <xsl:when test="$fullcollspan_real &lt; 1">
                <xsl:text>1</xsl:text>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="$fullcollspan_real" />
            </xsl:otherwise>
        </xsl:choose>
    </xsl:variable>
    
    <xsl:template match="/">
        <html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">
            <head>
                <title><xsl:value-of select="/preprocessedreport/head/title"/> - <xsl:value-of select="$page_owner" disable-output-escaping="yes"/></title>
                <meta charset="utf-8" />
                <meta http-equiv="content-type" content="text/html; charset=utf-8"/>
                <meta http-equiv="content-script-type" content="text/javascript" />
                <meta http-equiv="content-style-type" content="text/css" />
                <meta http-equiv="content-language" content="de-AT" />
                <meta name="author">
                    <xsl:attribute name="content">
                        <xsl:text>untermStrich </xsl:text>
                        <xsl:value-of select="$us_version" />
                    </xsl:attribute>
                </meta>
                <base>
                    <xsl:attribute name="href">
                        <xsl:value-of select="$base_url" />
                    </xsl:attribute>
                </base>
                <xsl:apply-imports/> <!-- Add the style information -->
            </head>
            <body>
                <pd4ml-page-header>
                    <xsl:value-of select="$page_header" disable-output-escaping="yes"/>
                </pd4ml-page-header>
                <span class="only_pdf">
                    <pd4ml-page-footer>
                        <xsl:value-of select="$page_footer" disable-output-escaping="yes"/>
                    </pd4ml-page-footer>
                </span>
                <xsl:apply-templates select="/preprocessedreport"/>
                <pd4ml-page-footer>
                    <xsl:value-of select="$page_footer" disable-output-escaping="yes"/>
                </pd4ml-page-footer>
                <xsl:call-template name="footer"/> <!-- Add the footer -->
            </body>
        </html>
    </xsl:template>

    <xsl:template match="/preprocessedreport">
        <xsl:apply-templates select="head"/> <!-- Add the head information -->
        <table>
            <xsl:attribute name="style">
                <xsl:text>border-color:</xsl:text>
                <xsl:value-of select="$border_color_table"/>
                <xsl:text>;</xsl:text>
            </xsl:attribute>
            <thead>
                <xsl:apply-templates select="head/columns"/> <!-- Heading -->
            </thead>
            <tbody>
                <xsl:apply-templates select="data"/> <!-- Values -->
            </tbody>
        </table>
    </xsl:template>

    <xsl:template match="head/columns"> <!-- Calls heading -->
        <xsl:call-template name="heading">
            <xsl:with-param name="grouplevel">0</xsl:with-param>
            <!-- Get all fields (via descendant) and then check via parent:group that we only get fields within a group, not within total -->
        </xsl:call-template>
    </xsl:template>

    <xsl:template name="heading"> <!-- Heading -->
        <xsl:param name="grouplevel" />
        <xsl:variable name="grouplevelindent">text-indent:<xsl:value-of select="$grouplevel*3" />mm;</xsl:variable>

        <xsl:for-each select="group">
            <tr>
                <th>
                    <xsl:attribute name="style">
                        <xsl:value-of select="$grouplevelindent"/>
                        <xsl:text>text-align:left;background-color:</xsl:text>
                        <xsl:value-of select="$background_color_heading"/>
                        <xsl:text>;</xsl:text>
                        <xsl:text>font-weight:</xsl:text>
                        <xsl:value-of select="$font_weight_heading"/>
                        <xsl:text>;</xsl:text>
                    </xsl:attribute>
                    <xsl:attribute name="colspan"><xsl:value-of select="$fullcollspan" /></xsl:attribute>
                    <xsl:for-each select="attr">
                        <xsl:value-of select="."/><xsl:text> </xsl:text>
                    </xsl:for-each>
                </th>
            </tr>
            <xsl:if test="count(field)>0">
                <tr>
                    <xsl:for-each select="field">
                        <th>
                            <xsl:attribute name="style">
                                <xsl:text>text-align:center;background-color:</xsl:text>
                                <xsl:value-of select="$background_color_heading"/>
                                <xsl:text>;</xsl:text>
                                <xsl:text>font-weight:</xsl:text>
                                <xsl:value-of select="$font_weight_heading"/>
                                <xsl:text>;</xsl:text>
                                <xsl:text>width:</xsl:text><xsl:value-of select="@width"/><xsl:text>mm;</xsl:text>
                            </xsl:attribute>
                            <xsl:value-of select="."/>
                        </th>
                    </xsl:for-each>
                </tr>
            </xsl:if>

            <xsl:call-template name="heading">
                <xsl:with-param name="grouplevel" select="$grouplevel+1" />
            </xsl:call-template>
        </xsl:for-each>
    </xsl:template>





    <xsl:template match="data"> <!-- Calls values -->

        <xsl:call-template name="values">
            <xsl:with-param name="grouplevel">0</xsl:with-param>
        </xsl:call-template>

        <xsl:call-template name="totals">
            <xsl:with-param name="grouplevelindent">
                <xsl:text>text-align:center;background-color:</xsl:text>
                <xsl:value-of select="$background_color_subheading"/>
                <xsl:text>;</xsl:text>
            </xsl:with-param>
            <xsl:with-param name="currentgroup" select="/preprocessedreport/head/columns" />
        </xsl:call-template>
    </xsl:template>



    <xsl:template name="values"> <!-- Values -->
        <xsl:param name="grouplevel" />
        <xsl:variable name="grouplevelindent">text-indent:<xsl:value-of select="$grouplevel*3" />mm;</xsl:variable>

        <xsl:for-each select="grouped">
            <xsl:variable name="currentgroupby" select="@by"/>
            <xsl:variable name="currentgroup" select="/preprocessedreport/head/columns/descendant::group[@by=$currentgroupby]"/>
            <xsl:for-each select="values">
                <!-- Subheading -->
                <tr>
                    <td>
                        <xsl:attribute name="style">
                            <xsl:value-of select="$grouplevelindent"/>
                            <xsl:text>background-color:</xsl:text>
                            <xsl:value-of select="$background_color_subheading"/>
                            <xsl:text>;</xsl:text>
                            <xsl:text>font-weight:</xsl:text>
                            <xsl:value-of select="$font_weight_subheading"/>
                            <xsl:text>;</xsl:text>
                        </xsl:attribute>
                        <xsl:attribute name="colspan"><xsl:value-of select="$fullcollspan" /></xsl:attribute>
                        
                        <xsl:for-each select="attr">
                            <!-- Add attr name -->
                            <span>
                                <xsl:attribute name="style">
                                    <xsl:text>font-weight:</xsl:text>
                                    <xsl:value-of select="$font_weight_heading"/>
                                    <xsl:text>;</xsl:text>
                                </xsl:attribute>
                                <xsl:variable name="currentgroupname" select="@name"/>
                                <xsl:value-of select="$currentgroup/attr[@name=$currentgroupname]"/>
                                <xsl:text>: </xsl:text>
                            </span>
                            <!-- Add attr value -->
                            <xsl:choose>
                                <xsl:when test="@type='number'">
                                    <xsl:value-of select="@usnumber"/>
                                </xsl:when>
                                <xsl:when test="@type='number2'">
                                    <xsl:value-of select="@usnumber"/>
                                </xsl:when>
                                <xsl:when test="@type='currency'">
                                    <xsl:value-of select="@usnumber"/>
                                    <xsl:text> </xsl:text>
                                    <xsl:value-of select="$company_currency"/>
                                </xsl:when>
                                <xsl:when test="@type='date'">
                                    <xsl:value-of select="@usdate"/>
                                </xsl:when>
                                <xsl:when test="@type='time'">
                                    <xsl:value-of select="@ustime"/>
                                </xsl:when>
                                <xsl:when test="@type='timespan'">
                                    <xsl:value-of select="@ustimespan"/>
                                </xsl:when>
                                <xsl:otherwise>
                                    <xsl:value-of select="."/>
                                </xsl:otherwise>
                            </xsl:choose>
                            <xsl:text> </xsl:text>
                        </xsl:for-each>
                    </td>
                </tr>

                <xsl:for-each select="row">
                    <tr>
                        <xsl:for-each select="cell">
                            <xsl:variable name="current" select="@name"/>
                            <td>
                                <xsl:choose>
                                    <xsl:when test="@type='number'">
                                        <xsl:attribute name="style">
                                            text-align:right;<xsl:if test="position()=1"><xsl:value-of select="$grouplevelindent"/></xsl:if>
                                        </xsl:attribute>
                                        <xsl:value-of select="@usnumber"/>
                                    </xsl:when>
                                    <xsl:when test="@type='number2'">
                                        <xsl:attribute name="style">
                                            text-align:right;<xsl:if test="position()=1"><xsl:value-of select="$grouplevelindent"/></xsl:if>
                                        </xsl:attribute>
                                        <xsl:value-of select="@usnumber"/>
                                    </xsl:when>
                                    <xsl:when test="@type='currency'">
                                        <xsl:attribute name="style">
                                            text-align:right;<xsl:if test="position()=1"><xsl:value-of select="$grouplevelindent"/></xsl:if>
                                        </xsl:attribute>
                                        <xsl:value-of select="@usnumber"/>
                                        <xsl:text> </xsl:text>
                                        <xsl:value-of select="$company_currency"/>
                                    </xsl:when>
                                    <xsl:when test="@type='bool'">
                                        <xsl:attribute name="style">text-align:center;</xsl:attribute>
                                        <xsl:if test=". = 'true'">
                                            <img alt="Yes">
                                                <xsl:attribute name="style">
                                                    width:<xsl:value-of select="$font_size_global"/>pt;
                                                    height:<xsl:value-of select="$font_size_global"/>pt;
                                                </xsl:attribute>
                                                <xsl:attribute name="src">
                                                    <xsl:value-of select="$bool_yes_img"/>
                                                </xsl:attribute>
                                            </img>
                                        </xsl:if>
                                        <xsl:if test=". = 'false'">
                                            <img alt="No">
                                                <xsl:attribute name="style">
                                                    width:<xsl:value-of select="$font_size_global"/>pt;
                                                    height:<xsl:value-of select="$font_size_global"/>pt;
                                                </xsl:attribute>
                                                <xsl:attribute name="src">
                                                    <xsl:value-of select="$bool_no_img"/>
                                                </xsl:attribute>
                                            </img>
                                        </xsl:if>
                                    </xsl:when>
                                    <xsl:when test="@type='date'">
                                        <xsl:attribute name="style">
                                            text-align:right;<xsl:if test="position()=1"><xsl:value-of select="$grouplevelindent"/></xsl:if>
                                        </xsl:attribute>
                                        <xsl:value-of select="@usdate"/>
                                    </xsl:when>
                                    <xsl:when test="@type='time'">
                                        <xsl:attribute name="style">
                                            text-align:right;<xsl:if test="position()=1"><xsl:value-of select="$grouplevelindent"/></xsl:if>
                                        </xsl:attribute>
                                        <xsl:value-of select="@ustime"/>
                                    </xsl:when>
                                    <xsl:when test="@type='timespan'">
                                        <xsl:attribute name="style">
                                            text-align:right;<xsl:if test="position()=1"><xsl:value-of select="$grouplevelindent"/></xsl:if>
                                        </xsl:attribute>
                                        <xsl:value-of select="@ustimespan"/>
                                    </xsl:when>
                                    <xsl:otherwise>
                                        <xsl:attribute name="style">
                                            text-align:left;<xsl:if test="position()=1"><xsl:value-of select="$grouplevelindent"/></xsl:if>
                                        </xsl:attribute>
                                        <xsl:value-of select="."/>
                                    </xsl:otherwise>
                                </xsl:choose>
                                <xsl:text> </xsl:text>
                            </td>
                        </xsl:for-each>
                    </tr>
                </xsl:for-each>

                <xsl:call-template name="values">
                    <xsl:with-param name="grouplevel" select="$grouplevel+1" />
                </xsl:call-template>

                <xsl:call-template name="totals">
                    <xsl:with-param name="grouplevelindent" select="$grouplevelindent" />
                    <xsl:with-param name="currentgroup" select="$currentgroup" />
                </xsl:call-template>

            </xsl:for-each>

        </xsl:for-each>

    </xsl:template>

    <xsl:template name="totals"> <!-- Totals -->
        <xsl:param name="grouplevelindent" />
        <xsl:param name="currentgroup" />

        <xsl:for-each select="rowtotals">
            <tr>
                <td>
                    <xsl:attribute name="style">
                        <xsl:text>font-weight:</xsl:text>
                        <xsl:value-of select="$font_weight_heading"/>
                        <xsl:text>;</xsl:text>
                        <xsl:value-of select="$grouplevelindent"/>
                    </xsl:attribute>
                    <xsl:attribute name="colspan"><xsl:value-of select="$fullcollspan" /></xsl:attribute>
                    <xsl:for-each select="*">
                        <xsl:variable name="current" select="@name"/>
                        <xsl:variable name="fieldinfo" select="$currentgroup/total/field[@name=$current]"/>
                        <span><xsl:text> </xsl:text></span>
                        <span>
                            <xsl:value-of select="$fieldinfo/."/><xsl:text>: </xsl:text>
                        </span>
                        <span>
                            <xsl:choose>
                                <xsl:when test="@type='number'">
                                    <xsl:attribute name="style">text-align:right;width:30mm;padding-right:3mm;</xsl:attribute>
                                    <xsl:value-of select="@usnumber"/>
                                </xsl:when>
                                <xsl:when test="@type='number2'">
                                    <xsl:attribute name="style">text-align:right;width:30mm;padding-right:3mm;</xsl:attribute>
                                    <xsl:value-of select="@usnumber"/>
                                </xsl:when>
                                <xsl:when test="@type='currency'">
                                    <xsl:attribute name="style">text-align:right;width:30mm;padding-right:3mm;</xsl:attribute>
                                    <xsl:value-of select="@usnumber"/>
                                    <xsl:text> </xsl:text>
                                    <xsl:value-of select="$company_currency"/>
                                </xsl:when>
                                <xsl:when test="@type='date'">
                                    <xsl:attribute name="style">text-align:right;width:30mm;padding-right:3mm;</xsl:attribute>
                                    <xsl:value-of select="@usdate"/>
                                </xsl:when>
                                <xsl:when test="@type='time'">
                                    <xsl:attribute name="style">text-align:right;width:30mm;padding-right:3mm;</xsl:attribute>
                                    <xsl:value-of select="@ustime"/>
                                </xsl:when>
                                <xsl:when test="@type='timespan'">
                                    <xsl:attribute name="style">text-align:right;width:30mm;padding-right:3mm;</xsl:attribute>
                                    <xsl:value-of select="@ustimespan"/>
                                </xsl:when>
                                <xsl:otherwise>
                                    <xsl:attribute name="style">text-align:left;width:30mm;padding-right:3mm;</xsl:attribute>
                                    <xsl:value-of select="."/>
                                </xsl:otherwise>
                            </xsl:choose>
                        </span>
                        <span><xsl:text> </xsl:text></span>
                    </xsl:for-each>
                </td>
            </tr>
        </xsl:for-each>
    </xsl:template>


</xsl:stylesheet>