<xsl:stylesheet version="1.0"
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:office="urn:oasis:names:tc:opendocument:xmlns:office:1.0"
                xmlns:style="urn:oasis:names:tc:opendocument:xmlns:style:1.0"
                xmlns:text="urn:oasis:names:tc:opendocument:xmlns:text:1.0"
                xmlns:table="urn:oasis:names:tc:opendocument:xmlns:table:1.0"
                xmlns:draw="urn:oasis:names:tc:opendocument:xmlns:drawing:1.0"
                xmlns:fo="urn:oasis:names:tc:opendocument:xmlns:xsl-fo-compatible:1.0"
                xmlns:xlink="http://www.w3.org/1999/xlink"
                xmlns:dc="http://purl.org/dc/elements/1.1/"
                xmlns:meta="urn:oasis:names:tc:opendocument:xmlns:meta:1.0"
                xmlns:number="urn:oasis:names:tc:opendocument:xmlns:datastyle:1.0"
                xmlns:svg="urn:oasis:names:tc:opendocument:xmlns:svg-compatible:1.0"
                xmlns:chart="urn:oasis:names:tc:opendocument:xmlns:chart:1.0"
                xmlns:dr3d="urn:oasis:names:tc:opendocument:xmlns:dr3d:1.0"
                xmlns:math="http://www.w3.org/1998/Math/MathML"
                xmlns:form="urn:oasis:names:tc:opendocument:xmlns:form:1.0"
                xmlns:script="urn:oasis:names:tc:opendocument:xmlns:script:1.0"
                xmlns:ooo="http://openoffice.org/2004/office"
                xmlns:ooow="http://openoffice.org/2004/writer"
                xmlns:oooc="http://openoffice.org/2004/calc"
                xmlns:dom="http://www.w3.org/2001/xml-events"
                xmlns:xforms="http://www.w3.org/2002/xforms"
                xmlns:xsd="http://www.w3.org/2001/XMLSchema"
                xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
                >
    <xsl:import href="_ods-style.xslt" />

    <xsl:output
        method="xml"
        encoding="UTF-8"
        indent="yes"
        />

    <xsl:template match="*">
        <office:document-content office:version="1.0">
            <office:scripts/>
            <office:font-face-decls>
                <style:font-face style:name="Arial" svg:font-family="Arial" style:font-family-generic="swiss" style:font-pitch="variable"/>
            </office:font-face-decls>
            <office:automatic-styles>
                <xsl:call-template name="columnswidthformat"/> <!-- Width of columns format -->
                <xsl:apply-imports/> <!-- Add the format information -->
            </office:automatic-styles>
            <office:body>
                <office:spreadsheet>
                    <table:table table:name="Report" table:print="false">
                        <xsl:call-template name="columnswidthassign"/> <!-- Width of columns assign -->
                        <xsl:apply-templates select="/preprocessedreport/head/columns"/> <!-- Heading -->
                        <xsl:apply-templates select="/preprocessedreport/data"/> <!-- Values -->
                    </table:table>
                </office:spreadsheet>
            </office:body>
        </office:document-content>
    </xsl:template>

    <xsl:template name="columnswidthformat"> <!-- Width of columns format -->
        <xsl:for-each select="/preprocessedreport/head/columns/descendant::field/parent::group/field">
            <style:style style:family="table-column">
                <xsl:attribute name="style:name">column_<xsl:value-of select="@name"/></xsl:attribute>
                <style:table-column-properties fo:break-before="auto">
                    <xsl:choose>
                        <xsl:when test="@width > 0">
                            <xsl:attribute name="style:column-width"><xsl:value-of select="@width div 10"/>cm</xsl:attribute>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:attribute name="style:column-width">1.5cm</xsl:attribute>
                        </xsl:otherwise>
                    </xsl:choose>
                </style:table-column-properties>
            </style:style>
        </xsl:for-each>
    </xsl:template>

    <xsl:template name="columnswidthassign"> <!-- Width of columns assign -->
        <xsl:for-each select="/preprocessedreport/head/columns/descendant::field/parent::group/field">
            <table:table-column table:default-cell-style-name="Default">
                <xsl:attribute name="table:style-name">column_<xsl:value-of select="@name"/></xsl:attribute>
            </table:table-column>
        </xsl:for-each>
    </xsl:template>




    <xsl:template match="/preprocessedreport/head/columns"> <!-- Calls heading -->
        <xsl:call-template name="heading">
            <xsl:with-param name="grouplevel">0</xsl:with-param>
            <xsl:with-param name="fullcollspan"><xsl:value-of select="count(/preprocessedreport/head/columns/descendant::field/parent::group/field)"/></xsl:with-param>
        </xsl:call-template>
    </xsl:template>

    <xsl:template name="heading"> <!-- Heading -->
        <xsl:param name="grouplevel" />
        <xsl:param name="fullcollspan" />

        <xsl:for-each select="group">
            <table:table-row>
                <table:table-cell office:value-type="string" table:number-rows-spanned="1">
                    <xsl:attribute name="table:style-name">heading<xsl:value-of select="$grouplevel" /></xsl:attribute>
                    <xsl:attribute name="table:number-columns-spanned"><xsl:value-of select="$fullcollspan" /></xsl:attribute>
                    <text:p>
                        <xsl:for-each select="attr">
                            <xsl:value-of select="."/><xsl:text> </xsl:text>
                        </xsl:for-each>
                    </text:p>
                </table:table-cell>
            </table:table-row>
            <xsl:if test="count(field)>0">
                <table:table-row>
                    <xsl:for-each select="field">
                        <table:table-cell office:value-type="string" table:style-name="headingcen">
                            <xsl:if test="position()=1">
                                <xsl:attribute name="table:style-name">headingcen<xsl:value-of select="$grouplevel" /></xsl:attribute>
                            </xsl:if>
                            <text:p>
                                <xsl:value-of select="."/>
                            </text:p>
                        </table:table-cell>
                    </xsl:for-each>
                </table:table-row>
            </xsl:if>


            <xsl:call-template name="heading">
                <xsl:with-param name="grouplevel" select="$grouplevel+1" />
                <xsl:with-param name="fullcollspan" select="$fullcollspan" />
            </xsl:call-template>
        </xsl:for-each>
    </xsl:template>



    <xsl:template match="/preprocessedreport/data"> <!-- Calls values -->
        <xsl:call-template name="values">
            <xsl:with-param name="grouplevel">0</xsl:with-param>
            <xsl:with-param name="fullcollspan"><xsl:value-of select="count(/preprocessedreport/head/columns/descendant::field/parent::group/field)"/></xsl:with-param>
        </xsl:call-template>
        
        <xsl:call-template name="totals">
            <xsl:with-param name="grouplevel">0</xsl:with-param>
            <xsl:with-param name="fullcollspan"><xsl:value-of select="count(/preprocessedreport/head/columns/descendant::field/parent::group/field)"/></xsl:with-param>
            <xsl:with-param name="currentgroup" select="/preprocessedreport/head/columns" />
        </xsl:call-template>
    </xsl:template>



    <xsl:template name="values"> <!-- Values -->
        <xsl:param name="grouplevel" />
        <xsl:param name="fullcollspan" />

        <xsl:for-each select="grouped">
            <xsl:variable name="currentgroupby" select="@by"/>

            <xsl:for-each select="values">
                <table:table-row>
                    <table:table-cell office:value-type="string" table:number-rows-spanned="1">
                        <xsl:attribute name="table:number-columns-spanned"><xsl:value-of select="$fullcollspan" /></xsl:attribute>
                        <xsl:attribute name="table:style-name">subheading<xsl:value-of select="$grouplevel" /></xsl:attribute>
                        <text:p>
                            <xsl:for-each select="attr">
                                <xsl:call-template name="cellout" />
                                <xsl:text> </xsl:text>
                            </xsl:for-each>
                        </text:p>
                    </table:table-cell>
                </table:table-row>

                <xsl:for-each select="row">
                    <table:table-row>
                        <xsl:apply-templates select="*[local-name()='cell']"/>
                    </table:table-row>
                </xsl:for-each>

                <xsl:call-template name="values">
                    <xsl:with-param name="grouplevel" select="$grouplevel+1" />
                    <xsl:with-param name="fullcollspan" select="$fullcollspan" />
                </xsl:call-template>

                <xsl:call-template name="totals">
                    <xsl:with-param name="grouplevel" select="$grouplevel+1" />
                    <xsl:with-param name="fullcollspan" select="$fullcollspan" />
                    <xsl:with-param name="currentgroup" select="/preprocessedreport/head/columns/descendant::group[@by=$currentgroupby]" />
                </xsl:call-template>

            </xsl:for-each>
        </xsl:for-each>
    </xsl:template>

    <xsl:template name="totals"> <!-- Totals -->
        <xsl:param name="grouplevel" />
        <xsl:param name="fullcollspan" />
        <xsl:param name="currentgroup" />

        <xsl:for-each select="rowtotals">
            <table:table-row>
                <table:table-cell office:value-type="string" table:number-rows-spanned="1">
                    <xsl:attribute name="table:number-columns-spanned"><xsl:value-of select="$fullcollspan" /></xsl:attribute>
                    <xsl:attribute name="table:style-name">total<xsl:value-of select="$grouplevel" /></xsl:attribute>
                    <text:p>
                        <xsl:for-each select="cell">
                            <xsl:variable name="current" select="@name"/>
                            <xsl:call-template name="tablecell">
                                <xsl:with-param name="fieldinfo" select="$currentgroup/total/field[@name=$current]"/>
                            </xsl:call-template>
                        </xsl:for-each>
                    </text:p>
                </table:table-cell>
            </table:table-row>
        </xsl:for-each>
    </xsl:template>

    <xsl:include href="_ods-value.xslt" />

</xsl:stylesheet>