<?php
/**
 * CodeIgniter library
 *
 * @category CodeIgniter
 * @license /_licenses/untermStrich/agb_normal.pdf
 * @author Christian Wengg
 * @copyright 2009 untermStrich
 * @package us_libraries
 * @version $Id$
*/

if (!defined('BASEPATH')) exit('No direct script access allowed');

/**
 * Used to create, save, load and assign data for the report
 *
 * @property    My_Controller           $CI                     The main CI object
 */
class S_overview_data
{
    protected $overtime_projects = null;
    
    /**
     * The main CI object
     */
    protected $CI;
    
    /**
     * Constructor
     */
    public function __construct()
    {
        $this->CI = CI_Controller::get_instance(); //$this does not work - Libs only get assigned to the module at load
        
        $this->CI->load->model('Staff_model', 'staff');
        $this->CI->load->model('Staff_groups_model', 'staff_groups');
        $this->CI->load->model('Target_hours_model', 'target_hours');
        $this->CI->load->model('Vacations_model', 'vacations');
        $this->CI->load->model('Overtime_model', 'overtime');
        $this->CI->load->model('Overtime_rates_model', 'overtime_rates');
        $this->CI->load->model('Projects_model', 'projects');
        $this->CI->load->model('Total_times_model', 'total_times');        
        $this->CI->load->model('Times_model', 'times');        
    }
    
    public function get_overtime_rates()
    {
        return $this->CI->staff->overtime_rates->get_list(false, false, false, true);
    }        
    
    public function get_overtime_projects()
    {
        return $this->overtime_projects = $this->CI->staff->projects->get_list_for_overtime_calculation();
    }       
    
    public function get_staff_list($request_vars) {
        
        if ($this->CI->login->getbool('respect_project_manager')) {
            $request_vars->project_manager_id = $this->CI->login->get('project_manager_id');
        }        
        if ($this->CI->login->getbool('respect_division_manager')) {
            $request_vars->division_manager_id = $this->CI->login->get('division_manager_id');
        }        
        
        return $this->CI->staff->get_list(true, false, false, null, null, 'staff_name', 1, true, false, $request_vars);
    }
    
    public function get_weekly_target_hours($date)
    {
        $this->CI->db->from('target_hours_daily AS thd');

        $this->CI->db->select('thd.staff_id');
        $this->CI->db->select('(thd.norm_work_mon+thd.norm_work_tue+thd.norm_work_wed+thd.norm_work_thu+thd.norm_work_fri+thd.norm_work_sat+thd.norm_work_sun) as norm_work_sum');

        
        $this->CI->db->subquery_by_main('thd', 'fr_om_date', 'target_hours_daily', 'fr_om_date', 'staff_id', 'staff_id', 'max', array('fr_om_date <= "'.$date->mysql().'"'=>false));
        $this->CI->db->group_by('thd.staff_id');
        
        $query = $this->CI->db->get();

        if ($query->num_rows() > 0) {
            $result_objects = $query->result_object();
            $query->free_result();
            return $result_objects;
        } else {
            $query->free_result();
            return array();
        }        
    }        
    
    public function get_balances($date)
    {
        return $this->CI->vacations->get_staff_balances(false, $date);
    }        
    
    public function get_deadlines($date)
    {
        return $this->CI->vacations->get_staff_deadline_date_limit(false, $date);
    }        
    
    public function get_staff_entitlement_date_limit($date)
    {
        return $this->CI->vacations->get_staff_entitlement_date_limit(false, $date);
        
    }    
    
    public function get_overtime($staff_id, $date)
    {
        return $this->CI->overtime->get_overtime_pool($staff_id, true, null, $date);
    }   
    
    public function get_overtime_project_times($staff_id, $date)
    {
        if ( $this->overtime_projects === null) {
            $this->get_overtime_projects();
        }
        
        $project_list = array(0);
        foreach ( $this->overtime_projects as  $project) {
            $project_list[] = intval($project->id);
        }
        
        $this->CI->db->select_sum('number', 'number');
        $this->CI->db->select('project_id');
        
        $this->CI->db->from('times AS '. Times_model::DB_ALIAS);
        
        $this->CI->db->where('staff_id', $staff_id);
        $this->CI->db->where_in('project_id', $project_list);
        $from = new UsDateTime($date->format('Y').'-01-01');
        $this->CI->times->filter_date_from_to(Times_model::DB_ALIAS, $from, $date);
                
        $this->CI->db->group_by('project_id');     
        
        $query = $this->CI->db->get();

        if ($query->num_rows() > 0) {
            $result_objects = $query->result_object();
            $query->free_result();
            return assoc_array_from_db($result_objects, 'project_id', 'number');
        } else {
            $query->free_result();
            return array();
        }        
    }        
    
    
} // END class S_overview_data
