<?php
/**
 * CodeIgniter library
 *
 * @category CodeIgniter
 * @license /_licenses/untermStrich/agb_normal.pdf
 * @author Christian Koller
 * @copyright 2009 untermStrich
 * @package us_libraries
 * @version $Id$
*/

if (!defined('BASEPATH')) exit('No direct script access allowed');

/**
 * Used to create, save, load and assign data for the report
 *
 * @property    My_Controller           $CI                     The main CI object
 */
class S_projectmatrix_times
{
    protected $staff_list = null;
    
    /**
     * The main CI object
     */
    protected $CI;
    
    /**
     * Constructor
     */
    public function __construct()
    {
        $this->CI = CI_Controller::get_instance(); //$this does not work - Libs only get assigned to the module at load
    }
    
    /**
     * Init staff list
     *
     * @param   mixed   $staff_id                   Optional Filter for staff ID or false
     * @param   mixed   $staff_group_id             Optional Filter for staff group ID or false
     * @param   object  $min_end_date               Optional Filter for min end date of staff
     * @param   object  $max_start_date             Optional Filter for max start date of staff
     * @param   mixed   $cost_centre                Optional Filter for cost centre
     * @param   mixed   $project_manager_id         Optional Filter for project_manager_id
     * @param   mixed   $division_manager_id        Optional Filter for division_manager_id
     * @param   mixed   $project_manager_id_other   Optional Filter for project_manager_id_other
     * @param   mixed   $division_manager_id_other  Optional Filter for division_manager_id_other
     * @return  array
     */
    public function get_staff($staff_id=false, $staff_group_id=false, $min_end_date=false, $max_start_date=false, $cost_centre=false, $project_manager_id=false, $division_manager_id=false, $project_manager_id_other=false, $division_manager_id_other=false)
    {
        $this->CI->load->model('Staff_model', 'staff');
        
        $request_vars = new stdClass();
        if ($staff_id!==false) {
            $request_vars->staff_id = $staff_id;
        }
        if ($staff_group_id!==false) {
            $request_vars->staff_group_id = $staff_group_id;
        }
        if (!$staff_id && $min_end_date!==false && $min_end_date!==null) {
            $request_vars->min_end_date = $min_end_date;
        }
        if (!$staff_id && $max_start_date!==false && $max_start_date!==null) {
            $request_vars->max_start_date = $max_start_date;
        }
        if ($cost_centre!==false && $cost_centre!==null) {
            $request_vars->cost_centre = $cost_centre;
        }
        if ($project_manager_id!==false && $project_manager_id!==null) {
            $request_vars->project_manager_id = $project_manager_id;
            if ($project_manager_id_other!==false && $project_manager_id_other!==null) {
                $request_vars->project_manager_id_other = $project_manager_id_other;
            }
        }
        if ($division_manager_id!==false && $division_manager_id!==null) {
            $request_vars->division_manager_id = $division_manager_id;
            if ($division_manager_id_other!==false && $division_manager_id_other!==null) {
                $request_vars->division_manager_id_other = $division_manager_id_other;
            }
        }
        
        $staffs = $this->CI->staff->get_list(true, false, false, null, null, 'staff_name', 1, true, false, $request_vars);
        
        $this->staff_list = array();
        foreach ($staffs as $staff) {
            array_push($this->staff_list, $staff->id);
        }
        
        return $staffs;
    }
    
    /**
     * Get the total times list
     *
     * @param   mixed   $project_id         Optional Project ID or false
     * @param   mixed   $project_group_id   Optional Project group ID or false
     * @param   object  $from               Optional From date
     * @param   object  $to                 Optional To date
     * @return  array
     */
    public function get_for_total_times($project_id=null, $project_group_id=null, UsBasicDateTime $from=null, UsBasicDateTime $to=null)
    {
        if (!isset($this->staff_list)) {
            basic_error('Call self::get_staff() before!', null, __CLASS__.'::'.__FUNCTION__);
        }
        
        $this->CI->load->model('Total_times_model', 'total_times');
        
        return $this->CI->total_times->get_for_staff_list($this->staff_list, $project_id, $project_group_id, $from, $to, true);
    }
    

} // END class S_projectmatrix_times
