<?php
/**
 * CodeIgniter library
 *
 * @category CodeIgniter
 * @license /_licenses/untermStrich/agb_normal.pdf
 * @author Christian Koller
 * @copyright 2009 untermStrich
 * @package us_libraries
 * @version $Id: address_values.php 26725 2024-01-11 07:37:15Z christian.w $
*/

if (!defined('BASEPATH')) exit('No direct script access allowed');

require_once APPPATH.'us_libraries/vendor/autoload.php';
require_once APPPATH.'us_libraries/Value_object_addi.php';

/**
 * Used to save an address
 *
 * @property    My_Controller           $CI                     The main CI object
 */
class Address_values extends Value_object_addi
{
    /**
     * Suffix used by the derived information
     */
    const DERIVED_SUFFIX = '_deri';

    protected $name;
    protected $comp_supp;
    protected $title;
    protected $academic_title;
    protected $master_title;
    protected $first_name;
    protected $contact_person;
    protected $street;
    protected $street2;
    protected $zip;
    protected $town;
    protected $country;
    protected $phone1;
    protected $phone2;
    protected $mobile_phone;
    protected $fax;
    protected $modem;
    protected $url;
    protected $email;
    protected $vat;
    protected $creditor_no;
    protected $debtor_no;
    protected $active;
    protected $dsgvo_delete_date = null;
    protected $dsgvo_data_from = null;
    protected $categories = array();
    protected $category_force_have = null;
    protected $category_force_have_action = null;
    protected $image;
    protected $longitude;
    protected $latitude;
    protected $sub_address_list;
    protected $parent_id;

    protected $name_derive;
    protected $comp_supp_derive;
    protected $title_derive;
    protected $academic_title_derive;
    protected $master_title_derive;
    protected $first_name_derive;
    protected $contact_person_derive;
    protected $street_derive;
    protected $street2_derive;
    protected $zip_town_derive;
    protected $country_derive;
    protected $phone1_derive;
    protected $phone2_derive;
    protected $mobile_phone_derive;
    protected $fax_derive;
    protected $modem_derive;
    protected $url_derive;
    protected $email_derive;
    protected $vat_derive;
    protected $categories_derive;
    protected $active_derive;
    protected $longitude_derive;
    protected $latitude_derive;
    protected $comment_main_all;
    
    /**
     * @var Sabre\VObject\Document $sabre_calendar_data_obj
     */
    protected $sabre_card_data_obj;
    protected $sabre_uri;
    protected $sabre_uid;

    protected $import_categories_string;
    protected $import_emails_string;
    
    protected $import_no_addi_derive;

    /**
     * The currently exisiting one, if any
     */
    private $_existing;
    /**
     * The parent one, if any
     */
    private $_parent;

    /**
     * Create an empty Address_values object
     *
     * @param   integer     $add_or_id              The item ID or 'add' for adding
     * @param   string      $name                   Company / Name
     * @param   string      $comp_supp              Company supplement
     * @param   string      $title                  Title
     * @param   string      $academic_title         Title of honours
     * @param   string      $master_title           Maser-Title
     * @param   string      $first_name             First name
     * @param   string      $contact_person         Contact person
     * @param   string      $street                 Street
     * @param   string      $street2                Street2
     * @param   string      $zip                    Zip code
     * @param   string      $town                   Town
     * @param   string      $country                Country
     * @param   string      $phone1                 Phone 1
     * @param   string      $phone2                 Phone 2
     * @param   string      $mobile_phone           Mobile phone
     * @param   string      $fax                    Fax
     * @param   string      $modem                  Modem
     * @param   string      $url                    URL
     * @param   string      $email                  E-mail
     * @param   string      $vat                    VAT No.
     * @param   string      $creditor_no            Creditor no
     * @param   string      $debtor_no              Debtor no
     * @param   string      $longitude              Longitude
     * @param   string      $latitude               Latitude
     * @param   boolean     $active                 Active
     * @param   array       $addi_vars              Additional variables as name=>value
     *                                              Ensure that you always use Addresses_model::get_addi_vars_post() to create those variables.
     *                                              This function only adds vars to the addi_vars array, where the user has the right to do so.
     * @param   array       $categories             Address categories
     * @param   string      $image                  Image
     * @param   mixed       $sub_address_list       List of sub addresses (Updates derived flags for those sub addresses)
     * @param   mixed       $parent_id              Parent ID or false for none (Is set by the model, if the item ID is not 'add')
     * @param   boolean     $name_derive            Derive?
     * @param   boolean     $comp_supp_derive       Derive?
     * @param   boolean     $title_derive           Derive?
     * @param   boolean     $academic_title_derive  Derive?
     * @param   boolean     $master_title_derive    Derive?
     * @param   boolean     $first_name_derive      Derive?
     * @param   boolean     $contact_person_derive  Derive?
     * @param   boolean     $street_derive          Derive?
     * @param   boolean     $street2_derive         Derive?
     * @param   boolean     $zip_town_derive        Derive?
     * @param   boolean     $country_derive         Derive?
     * @param   boolean     $phone1_derive          Derive?
     * @param   boolean     $phone2_derive          Derive?
     * @param   boolean     $mobile_phone_derive    Derive?
     * @param   boolean     $fax_derive             Derive?
     * @param   boolean     $modem_derive           Derive?
     * @param   boolean     $url_derive             Derive?
     * @param   boolean     $email_derive           Derive?
     * @param   boolean     $vat_derive             Derive?
     * @param   boolean     $categories_derive      Derive?
     * @param   boolean     $active_derive          Derive?
     * @param   boolean     $longitude_derive       Derive?
     * @param   boolean     $latitude_derive        Derive?
     * @return  object  An Address_values object
     */
    public static function init_from_values(
        $add_or_id,
        $name,
        $comp_supp,
        $title,
        $academic_title,
        $master_title,
        $first_name,
        $contact_person,
        $street,
        $street2,
        $zip,
        $town,
        $country,
        $phone1,
        $phone2,
        $mobile_phone,
        $fax,
        $modem,
        $url,
        $email,
        $vat,
        $creditor_no,
        $debtor_no,
        $longitude,
        $latitude,
        $comment_main_all,
        $active,
        array $addi_vars,
        array $categories,
        $image,
        $sub_address_list,
        $parent_id,
        $name_derive,
        $comp_supp_derive,
        $title_derive,
        $academic_title_derive,
        $master_title_derive,
        $first_name_derive,
        $contact_person_derive,
        $street_derive,
        $street2_derive,
        $zip_town_derive,
        $country_derive,
        $phone1_derive,
        $phone2_derive,
        $mobile_phone_derive,
        $fax_derive,
        $modem_derive,
        $url_derive,
        $email_derive,
        $vat_derive,
        $categories_derive,
        $active_derive,
        $longitude_derive,
        $latitude_derive,
        ?stdClass $sabre=null
    ) {
        $obj = new Address_values();
        $obj->CI = CI_Controller::get_instance();
        $obj->add_or_id = $add_or_id;
        $obj->name = $name;
        $obj->comp_supp = $comp_supp;
        $obj->title = $title;
        $obj->academic_title = $academic_title;
        $obj->master_title = $master_title;
        $obj->first_name = $first_name;
        $obj->contact_person = $contact_person;
        $obj->street = $street;
        $obj->street2 = $street2;
        $obj->zip = $zip;
        $obj->town = $town;
        $obj->country = $country; //Validate!
        $obj->phone1 = $phone1;
        $obj->phone2 = $phone2;
        $obj->mobile_phone = $mobile_phone;
        $obj->fax = $fax;
        $obj->modem = $modem;
        $obj->url = $url;
        $obj->email = $email;
        $obj->vat = trim($vat);
        $obj->creditor_no = $creditor_no;
        $obj->debtor_no = $debtor_no;
        $obj->active = $active;
        $obj->longitude = $longitude;
        $obj->latitude = $latitude;
        $obj->comment_main_all = $comment_main_all;
        
        $obj->addi_vars = $addi_vars;
        $obj->categories = $categories;
        $obj->image = $image;
        $obj->sub_address_list = ($sub_address_list) ? explode(',',$sub_address_list) : false;
        $obj->parent_id = $parent_id;

        $obj->name_derive = $name_derive;
        $obj->comp_supp_derive = $comp_supp_derive;
        $obj->title_derive = $title_derive;
        $obj->academic_title_derive = $academic_title_derive;
        $obj->master_title_derive = $master_title_derive;
        $obj->first_name_derive = $first_name_derive;
        $obj->contact_person_derive = $contact_person_derive;
        $obj->street_derive = $street_derive;
        $obj->street2_derive = $street2_derive;
        $obj->zip_town_derive = $zip_town_derive;
        $obj->country_derive = $country_derive;
        $obj->phone1_derive = $phone1_derive;
        $obj->phone2_derive = $phone2_derive;
        $obj->mobile_phone_derive = $mobile_phone_derive;
        $obj->fax_derive = $fax_derive;
        $obj->modem_derive = $modem_derive;
        $obj->url_derive = $url_derive;
        $obj->email_derive = $email_derive;
        $obj->vat_derive = $vat_derive;
        $obj->categories_derive = $categories_derive;
        $obj->active_derive = $active_derive;
        $obj->longitude_derive = $longitude_derive;
        $obj->latitude_derive = $latitude_derive;
        
        if (!isset($sabre)) {
            $sabre = new stdClass();
        }
        if (isset($sabre->sabre_card_data_obj)) {
            $obj->sabre_card_data_obj = $sabre->sabre_card_data_obj;
        }
        if (isset($sabre->sabre_uri)) {
            $obj->sabre_uri = $sabre->sabre_uri;
        }
        if (isset($sabre->sabre_uid)) {
            $obj->sabre_uid = $sabre->sabre_uid;
            if (!isset($sabre->sabre_uri)) {
                $obj->sabre_uri = $sabre->sabre_uid.'.ics';
            }
        }
        
        return $obj;
    }

    /**
     * Init the address values from $_POST
     *
     * @return  object  An Address_values object with all values out of the post array
     */
    public static function init_from_post()
    {
        $obj = new Address_values();
        $obj->CI = CI_Controller::get_instance();
        
        $obj->CI->load->model('System_settings_model', 'system_settings');
        
        $addresses_creditor_show = $obj->CI->system_settings->getForms_addresses_creditor_show();
        $addresses_modem_show = $obj->CI->system_settings->getForms_addresses_modem_show();
        $addresses_longitude_show = $obj->CI->system_settings->getForms_addresses_longitude_show();
        
        $obj->add_or_id = $obj->CI->input->post('addresses_edit');
        $obj->name = $obj->CI->input->post('addresses_name');
        $obj->comp_supp = $obj->CI->input->post('addresses_comp_supp');
        $obj->title = $obj->CI->input->post('addresses_title');
        $obj->academic_title = $obj->CI->input->post('addresses_academic_title');
        $obj->master_title = $obj->CI->input->post('addresses_master_title');
        $obj->first_name = $obj->CI->input->post('addresses_first_name');
        $obj->contact_person = $obj->CI->input->post('addresses_contact_person');
        $obj->street = $obj->CI->input->post('addresses_street');
        $obj->street2 = $obj->CI->input->post('addresses_street2');
        $obj->zip = $obj->CI->input->post('addresses_zip_town__zip');
        $obj->town = $obj->CI->input->post('addresses_zip_town__town');
        $obj->country = $obj->CI->input->post('addresses_country'); //Validate!
        $obj->phone1 = $obj->CI->input->post('addresses_phone1');
        $obj->phone2 = $obj->CI->input->post('addresses_phone2');
        $obj->mobile_phone = $obj->CI->input->post('addresses_mobile_phone');
        $obj->fax = $obj->CI->input->post('addresses_fax');
        $obj->modem = ($addresses_modem_show) ? $obj->CI->input->post('addresses_modem') : '';
        $obj->url = trim(base64_decode($obj->CI->input->post('addresses_url')));
        $obj->email = $obj->CI->input->post('addresses_email');
        $obj->vat = trim($obj->CI->input->post('addresses_vat'));
        $obj->creditor_no = ($addresses_creditor_show) ? $obj->CI->input->post('addresses_creditor_no') : '';
        $obj->debtor_no = ($addresses_creditor_show) ? $obj->CI->input->post('addresses_debtor_no') : '';
        $obj->active = $obj->CI->input->post('addresses_active') ? true : false;
        $obj->dsgvo_delete_date = new UsDate($obj->CI->input->post('addresses_dsgvo_delete_date'), null, true, null);
        $obj->dsgvo_data_from = $obj->CI->input->post('addresses_dsgvo_data_from');
        $obj->addi_vars = $obj->CI->addresses->get_addi_vars_post();
        $obj->categories = $obj->CI->input->post('addresses_categories');
        $obj->image = $obj->CI->input->post('addresses_image');
        $obj->longitude = ($addresses_longitude_show) ? usnumber_to_float($obj->CI->input->post('addresses_longitude')) : null;
        $obj->latitude = ($addresses_longitude_show) ? usnumber_to_float($obj->CI->input->post('addresses_latitude')) : null;
        $obj->comment_main_all = $obj->CI->input->post('addresses_comment_main_all');
        $obj->sub_address_list = ($obj->CI->input->post('addresses_sub_address_list')) ? explode(',', $obj->CI->input->post('addresses_sub_address_list')) : false;
        $obj->parent_id = $obj->CI->input->post('addresses_parent_id');
        $obj->duplicate_old_id = $obj->CI->input->post('addresses_duplicate_old_id');
        $obj->duplicate_children = (strtolower($obj->CI->input->post('addresses_duplicate_children')) === 'true') ? true : false;

        $obj->name_derive = $obj->CI->input->post('addresses_name_derive') ? true : false;
        $obj->comp_supp_derive = $obj->CI->input->post('addresses_comp_supp_derive') ? true : false;
        $obj->title_derive = $obj->CI->input->post('addresses_title_derive') ? true : false;
        $obj->academic_title_derive = $obj->CI->input->post('addresses_academic_title_derive') ? true : false;
        $obj->master_title_derive = $obj->CI->input->post('addresses_master_title_derive') ? true : false;
        $obj->first_name_derive = $obj->CI->input->post('addresses_first_name_derive') ? true : false;
        $obj->contact_person_derive = $obj->CI->input->post('addresses_contact_person_derive') ? true : false;
        $obj->street_derive = $obj->CI->input->post('addresses_street_derive') ? true : false;
        $obj->street2_derive = $obj->CI->input->post('addresses_street2_derive') ? true : false;
        $obj->zip_town_derive = $obj->CI->input->post('addresses_zip_town_derive') ? true : false;
        $obj->country_derive = $obj->CI->input->post('addresses_country_derive') ? true : false;
        $obj->phone1_derive = $obj->CI->input->post('addresses_phone1_derive') ? true : false;
        $obj->phone2_derive = $obj->CI->input->post('addresses_phone2_derive') ? true : false;
        $obj->mobile_phone_derive = $obj->CI->input->post('addresses_mobile_phone_derive') ? true : false;
        $obj->fax_derive = $obj->CI->input->post('addresses_fax_derive') ? true : false;
        $obj->modem_derive = $obj->CI->input->post('addresses_modem_derive') ? true : false;
        $obj->url_derive = $obj->CI->input->post('addresses_url_derive') ? true : false;
        $obj->email_derive = $obj->CI->input->post('addresses_email_derive') ? true : false;
        $obj->vat_derive = $obj->CI->input->post('addresses_vat_derive') ? true : false;
        $obj->categories_derive = $obj->CI->input->post('addresses_categories_derive') ? true : false;
        $obj->active_derive = $obj->CI->input->post('addresses_active_derive') ? true : false;
        $obj->longitude_derive = $obj->CI->input->post('addresses_longitude_derive') ? true : false;
        $obj->latitude_derive = $obj->CI->input->post('addresses_latitude_derive') ? true : false;
        return $obj;
    }
    
    /**
     * Is the address new
     *
     * @return  boolean
     */
    public function get_new()
    {
        return ($this->add_or_id==='add');
    }

    /**
     * Validate and ensure all values are appropriate
     *
     * @return  mixed   false on success, else the error message
     */
    public function validate()
    {
        if ($this->add_or_id!=='add' && !is_numeric($this->add_or_id)) {
            return 'Address ID has to be "add" or an ID';
        }

        if (!$this->name_derive && trim($this->name)==='') {
            return $this->CI->lang->linef('required_derive', $this->CI->lang->line('addresses_name'));
        }

        if (!$this->categories_derive && empty($this->categories)) {
            return $this->CI->lang->linef('required_derive', $this->CI->lang->line('addresses_categories'));
        }

        if ($this->add_or_id!=='add') {
            $this->_existing = $this->CI->addresses->get_item($this->add_or_id, true);
            if (!$this->_existing) {
                return 'Invalid address ID';
            }
            $this->parent_id = $this->_existing->parent;
        }

        if (!is_numeric($this->parent_id)) {
            $this->parent_id = false;
        }

        if ($this->parent_id!==false && $this->parent_id>0) {
            $this->_parent = $this->CI->addresses->get_item($this->parent_id);
            if (!$this->_parent) {
                return 'Invalid parent address ID';
            }
        }

        return false;
    }

    /**
     * Get basic values
     *
     * @return  array
     */
    public function get_basics()
    {
        $reply = array(
            'name'              =>    $this->name,
            'comp_supp'         =>    $this->comp_supp,
            'title'             =>    $this->title,
            'academic_title'    =>    $this->academic_title,
            'master_title'      =>    $this->master_title,
            'first_name'        =>    $this->first_name,
            'contact_person'    =>    $this->contact_person,
            'street'            =>    $this->street,
            'street2'           =>    $this->street2,
            'zip'               =>    $this->zip,
            'town'              =>    $this->town,
            'country'           =>    $this->country,
            'phone1'            =>    $this->phone1,
            'phone2'            =>    $this->phone2,
            'mobile_phone'      =>    $this->mobile_phone,
            'fax'               =>    $this->fax,
            'modem'             =>    $this->modem,
            'url'               =>    $this->url,
            'email'             =>    $this->email,
            'vat'               =>    $this->vat,
            'creditor_no'       =>    $this->creditor_no,
            'debtor_no'         =>    $this->debtor_no,
            'active'            =>    $this->active,
            'dsgvo_delete_date' =>    isset($this->dsgvo_delete_date) ? $this->dsgvo_delete_date->mysql() : null,
            'dsgvo_data_from'   =>    isset($this->dsgvo_data_from) ? $this->dsgvo_data_from : '',
            'image'             =>    $this->image,
            'parent'            =>    $this->parent_id,
            'longitude'         =>    $this->longitude,
            'latitude'          =>    $this->latitude,
            'comment_main_all'  =>    $this->comment_main_all,
        );
        
        return $reply;
    }

    /**
     * Get address values to save
     *
     * Not for multi value fields
     *
     * @return  array
     */
    public function get_address_to_save()
    {
        $data = array(
            'name'              =>    $this->rem_newlines($this->name),
            'comp_supp'         =>    $this->comp_supp,
            'title'             =>    $this->rem_newlines($this->title),
            'academic_title'    =>    $this->rem_newlines($this->academic_title),
            'master_title'      =>    $this->rem_newlines($this->master_title),
            'first_name'        =>    $this->rem_newlines($this->first_name),
            'contact_person'    =>    $this->rem_newlines($this->contact_person),
            'street'            =>    $this->rem_newlines($this->street),
            'street2'           =>    $this->rem_newlines($this->street2),
            'zip'               =>    $this->rem_newlines($this->zip),
            'town'              =>    $this->rem_newlines($this->town),
            'country'           =>    $this->rem_newlines($this->country),
            'phone1'            =>    $this->rem_newlines($this->phone1),
            'phone2'            =>    $this->rem_newlines($this->phone2),
            'mobile_phone'      =>    $this->rem_newlines($this->mobile_phone),
            'fax'               =>    $this->rem_newlines($this->fax),
            'modem'             =>    $this->rem_newlines($this->modem),
            'url'               =>    $this->rem_newlines($this->url),
            'email'             =>    $this->rem_newlines($this->email),
            'vat'               =>    $this->rem_newlines($this->vat),
            'creditor_no'       =>    $this->rem_newlines($this->creditor_no),
            'debtor_no'         =>    $this->rem_newlines($this->debtor_no),
            'active'            =>    $this->active,
            'dsgvo_delete_date' =>    isset($this->dsgvo_delete_date) ? $this->dsgvo_delete_date->mysql() : null,
            'dsgvo_data_from'   =>    isset($this->dsgvo_data_from) ? $this->dsgvo_data_from : '',
        );

        foreach ($data as $k => $v) {
            if (!isset($this->filter_fields)) break;

            if (!isset($this->filter_fields->$k)) {
                unset($data[$k]);
            }
        }
        
        if ($this->get_import_no_addi_derive()!==true) {
            $data_derive = $this->_get_derived_info();
            $data = array_merge($data, $data_derive);
            $data['parent'] = $this->parent_id;
        }
        
        if ($this->longitude!==false && !empty($this->longitude)) {
            $data['longitude'] = $this->rem_newlines($this->longitude);
        }
        if ($this->latitude!==false && !empty($this->latitude)) {
            $data['latitude'] = $this->rem_newlines($this->latitude);
        }
        
        if ($this->image!==false && !empty($this->image)) {
            if($this->image=='#DELETEME#') {
                $data['image'] = null;
            } else {
                $data['image'] = $this->image;
            }
        }

        if (isset($this->sabre_card_data_obj)) {
            $data['sabre_card_data'] = $this->sabre_card_data_obj->serialize();
        }
        if (!empty((string)$this->sabre_uid)) {
            $data['sabre_uid'] = (string)$this->sabre_uid;
        } else {
            $data['sabre_uid'] = \Sabre\DAV\UUIDUtil::getUUID();
        }
        if ($this->add_or_id === 'add') {
            if (!empty((string)$this->sabre_uri)) {
                $data['sabre_uri'] = (string)$this->sabre_uri;
            } else {
                $data['sabre_uri'] = $data['sabre_uid'].'.vcf';
            }
        }
        
        if ($this->_parent) {
            $this->CI->load->us_library('logic/address_derive_logic');
            foreach (Address_derive_logic::get_deriveable() as $name=>$name_derived) {
                $this->_derive_from($data, $name, $name_derived);
            }
        }
        
        if (isset($this->comment_main_all) && $this->comment_main_all!==false) {
            $data['comment_main'] = '';
            $data['comment_main_more'] = '';
            $comment_main_alls = preg_split('/\r\n|\r|\n/', $this->comment_main_all);
            if (count($comment_main_alls)>=1) {
                $comment_main_arr = mb_str_split(array_shift($comment_main_alls), 140);
                $data['comment_main'] = (string)array_shift($comment_main_arr);

                if (count($comment_main_arr)>=1) {
                    array_unshift($comment_main_alls, implode('', $comment_main_arr));
                }
                unset($comment_main_arr);

                if (count($comment_main_alls)>=1) {
                    $data['comment_main_more'] = implode("\n", $comment_main_alls);
                }
            }
        }

        return $data;
    }

    /**
     * Get categories to save
     *
     * @return  array
     */
    public function get_categories_to_save()
    {
        if ($this->_parent && $this->categories_derive) {
            $data = array();
            foreach ($this->_parent->categories as $category) {
                array_push($data, $category->address_category_id);
            }
        } else {
            $data = $this->categories;
        }
        
        if (isset($this->category_force_have)) {
            $data = array_unique(array_merge($data, $this->category_force_have));
        }
        
        if (isset($this->category_force_have_action)) {
            if (!in_array($this->category_force_have_action, $data)) {
                array_push($data, $this->category_force_have_action);
            }
        }
        
        return $data;
    }

    /**
     * Set the data var to the parents value if it is derived
     *
     * @param   array   &$data          Reference The data array
     * @param   string  $name           Name of the field
     * @param   mixed   $name_derived   Optional Set this variable to use a different name for the derive info (Default=false)
     * @return  void
     */
    private function _derive_from(array &$data, $name, $name_derived=false)
    {
        $name_derived = ($name_derived===false) ? $name : $name_derived;
        if ($this->{$name_derived.'_derive'}) $data[$name] = $this->_parent->$name;
    }
    
    /**
     * Get the sub-addresses-info
     *
     * @return mixed  An array if sub addresses exists, else false
     */
    public function get_sub_address_info()
    {
        if(!is_array($this->sub_address_list) || count($this->sub_address_list) <= 0){
            return false;
        }
        
        $data = array('sub_address_list' => $this->sub_address_list, 'derived_info' => $this->_get_derived_info(), 'parent' => $this->parent_id);
        return $data;
    }
    
    /**
     * Get information for duplication
     * 
     * @return mixed   Either an data array or false if no duplication
     */
    public function get_sub_address_duplication_info()
    {
        if (!isset($this->duplicate_old_id) || !is_numeric($this->duplicate_old_id)) {
            return false;
        }
        
        $dup_children = (!isset($this->duplicate_children)) ? false : $this->duplicate_children;
        
        $data = array('old_id' => $this->duplicate_old_id, 'derived_info' => $this->_get_derived_info(), 'duplicate_children' => $dup_children);
        return $data;
    }        
    
    /*
     * Get the derived information
     * 
     * @return array
     */
    private function _get_derived_info()
    {
        return array(
            'name'.self::DERIVED_SUFFIX                 => $this->name_derive,
            'comp_supp'.self::DERIVED_SUFFIX            => $this->comp_supp_derive,
            'title'.self::DERIVED_SUFFIX                => $this->title_derive,
            'academic_title'.self::DERIVED_SUFFIX       => $this->academic_title_derive,
            'master_title'.self::DERIVED_SUFFIX         => $this->master_title_derive,
            'first_name'.self::DERIVED_SUFFIX           => $this->first_name_derive,
            'contact_person'.self::DERIVED_SUFFIX       => $this->contact_person_derive,
            'street'.self::DERIVED_SUFFIX               => $this->street_derive,
            'street2'.self::DERIVED_SUFFIX               => $this->street2_derive,
            'zip_town'.self::DERIVED_SUFFIX             => $this->zip_town_derive,
            'country'.self::DERIVED_SUFFIX              => $this->country_derive,
            'phone1'.self::DERIVED_SUFFIX               => $this->phone1_derive,
            'phone2'.self::DERIVED_SUFFIX               => $this->phone2_derive,
            'mobile_phone'.self::DERIVED_SUFFIX         => $this->mobile_phone_derive,
            'fax'.self::DERIVED_SUFFIX                  => $this->fax_derive,
            'modem'.self::DERIVED_SUFFIX                => $this->modem_derive,
            'url'.self::DERIVED_SUFFIX                  => $this->url_derive,
            'email'.self::DERIVED_SUFFIX                => $this->email_derive,
            'vat'.self::DERIVED_SUFFIX                  => $this->vat_derive,
            'categories'.self::DERIVED_SUFFIX           => $this->categories_derive,
            'active'.self::DERIVED_SUFFIX               => $this->active_derive,
            'longitude'.self::DERIVED_SUFFIX            => $this->longitude_derive,
            'latitude'.self::DERIVED_SUFFIX             => $this->latitude_derive
        );   
    }        
    
    /**
     * Get the sabre_card_data object
     * 
     * @return Sabre\VObject\Document
     */
    public function getSabre_card_data_obj(): Sabre\VObject\Document
    {
        return $this->sabre_card_data_obj;
    }
    
    /**
     * Returns the names of all variables of this object that should be serialized
     *
     * @return  array
     */
    public function __sleep()
    {
        return array(
            "add_or_id",
            "addi_vars",
            "name",
            "comp_supp",
            "title",
            "academic_title",
            "master_title",
            "first_name",
            "contact_person",
            "street",
            "street2",
            "zip",
            "town",
            "country",
            "phone1",
            "phone2",
            "mobile_phone",
            "fax",
            "modem",
            "url",
            "email",
            "vat",
            "creditor_no",
            "debtor_no",
            "active",
            "categories",
            "category_force_have",
            "category_force_have_action",
            "image",
            "longitude",
            "latitude",
            "sub_address_list",
            "parent_id",

            "name_derive",
            "comp_supp_derive",
            "title_derive",
            "academic_title_derive",
            "master_title_derive",
            "first_name_derive",
            "contact_person_derive",
            "street_derive",
            "street2_derive",
            "zip_town_derive",
            "country_derive",
            "phone1_derive",
            "phone2_derive",
            "mobile_phone_derive",
            "fax_derive",
            "modem_derive",
            "url_derive",
            "email_derive",
            "vat_derive",
            "categories_derive",
            "active_derive",
            "longitude_derive",
            "latitude_derive",
            "comment_main_all",
            
            "sabre_uri",
            "sabre_uid",
            
            "import_categories_string",
            "import_emails_string",
            
            "import_no_addi_derive",
        );
    }
    
    /**
     * Set import only vars
     *
     * @param   string  $import_categories_string       Categories
     * @param   string  $import_emails_string           Addition Email addresses
     * @return  void
     */
    public function set_import_vars($import_categories_string, $import_emails_string)
    {
        $this->import_categories_string = $import_categories_string;
        $this->import_emails_string = $import_emails_string;
    }
    
    /**
     * Set categories
     *
     * @param   array   $categories     Categories
     * @return  void
     */
    public function set_categories(array $categories)
    {
        $this->categories = $categories;
    }
    
    /**
     * Force add the following category
     * 
     * @param   array     $category_force_have            Category IDs
     * @return  void
     */
    public function set_category_force_have(array $category_force_have): void
    {
        $this->category_force_have = $category_force_have;
    }

    /**
     * Force add the following category for action
     * 
     * @param   int     $category_force_have_action     Category ID
     * @return  void
     */
    public function set_category_force_have_action(int $category_force_have_action): void
    {
        $this->category_force_have_action = $category_force_have_action;
    }

    /**
     * Add category
     *
     * @param   int     $category       Category to add
     * @return  void
     */
    public function add_category(int $category)
    {
        $this->categories[] = $category;
    }
    
    /**
     * Get import only vars
     *
     * @return  array
     */
    public function get_import_vars()
    {
        return array(
            'categories'    =>      $this->import_categories_string,
            'emails'        =>      $this->import_emails_string,
        );
    }
    
    /**
     * Get VAT
     * 
     * @return  string
     */
    public function get_vat()
    {
        return $this->vat;
    }
    
    /**
     * Set import save no addi/derive vars flag
     * 
     * @param   bool    $import_no_addi     Import save no addi vars flag
     * @return  void
     */
    public function set_import_no_addi_derive(bool $import_no_addi): void
    {
        $this->import_no_addi_derive = $import_no_addi;
    }

    /**
     * Get import save no addi/derive vars flag
     * 
     * @return bool
     */
    public function get_import_no_addi_derive(): bool
    {
        return $this->import_no_addi_derive ?? false;
    }
    
    /**
     * Reconstruct any resources
     *
     * @return  void
     */
    public function __wakeup()
    {
        $this->CI = CI_Controller::get_instance();
    }

} // END class Address_values
