<?php
/**
 * CodeIgniter library
 *
 * @category CodeIgniter
 * @license /_licenses/untermStrich/agb_normal.pdf
 * @author Christian Wengg
 * @copyright 2009 untermStrich
 * @package us_libraries
 * @version $Id: calcexp_entry_values.php 25951 2023-07-12 13:37:20Z pansidom $
*/

if (!defined('BASEPATH')) exit('No direct script access allowed');

require_once APPPATH.'us_libraries/Value_object_addi.php';

/**
 * Used to save a calcexp entry
 *
 * @property    My_Controller           $CI                     The main CI object
 */
class Calcexp_entry_values extends Value_object_addi
{

    protected $project_id;
    protected $staff_id;
    protected $date;
    protected $description;
    protected $is_project;
    protected $is_woa;
    protected $is_staff;

    /**
     * Init the calcexp values from $_POST
     *
     * @return  object  An Calcexp_entry_values object with all values out of the post array
     */
    public static function init_from_post()
    {
        $obj = new Calcexp_entry_values();
        $obj->CI = CI_Controller::get_instance();
        $obj->add_or_id = $obj->CI->input->post('calcexp_edit');
        $obj->project_id = $obj->CI->input->post('calcexp_project_id');
        $obj->staff_id = $obj->CI->input->post('calcexp_staff_id');
        $obj->date = $obj->CI->input->post('calcexp_date');
        $obj->description = $obj->CI->input->post('calcexp_description');
        $obj->is_project = $obj->CI->input->post('calcexp_is_project');
        $obj->is_woa = $obj->CI->input->post('calcexp_is_woa');
        $obj->is_staff = $obj->CI->input->post('calcexp_is_staff');
        
        return $obj;
    }

    /**
     * Init the calcexp values from a given StdClass object
     *
     * @param   object  $values StdClass object with calcexp values to initialize a valid Calcexp_entry_values object out of it
     *
     * @return  object  An Calcexp_entry_values object with all values out of the given StdClass object
     */
    public static function init_from_values(StdClass $values)
    {
        $obj = new Calcexp_entry_values();
        $obj->CI = CI_Controller::get_instance();
        $obj->add_or_id = $values->id;
        $obj->project_id = $values->project_id;
        $obj->staff_id = $values->staff_id;
        $obj->date = $values->date;
        $obj->description = $values->description;
        $obj->is_project = $values->is_project;
        $obj->is_woa = $values->is_woa;
        $obj->is_staff = $values->is_staff;
        
        return $obj;
    }

    /**
     * Validate and ensure all values are appropriate
     *
     * @return  mixed   false on success, else the error message
     */
    public function validate()
    {
        if ($this->add_or_id!=='add' && !is_numeric($this->add_or_id)) {
            return 'Calcexp ID has to be "add" or an ID';
        }     
        
        return false;
    }

    /**
     * Get staff to save
     *
     * @return  array
     */
    public function get_calcexp_to_save()
    {
        $data = array(
            'project_id' => $this->project_id,
            'staff_id' => $this->staff_id,
            'date' => $this->date,
            'description' => $this->description,
            'is_project' => $this->is_project,
            'is_woa' => $this->is_woa,
            'is_staff' => $this->is_staff
        );
        
        return $data;
    }

} // END class Calcexp_entry_values
