<?php
/**
 * CodeIgniter library
 *
 * @category CodeIgniter
 * @license /_licenses/untermStrich/agb_normal.pdf
 * @author Christian Koller
 * @copyright 2009 untermStrich
 * @package us_libraries
 * @version $Id: correspondence_values.php christian $
*/

if (!defined('BASEPATH')) exit('No direct script access allowed');

require_once APPPATH.'us_libraries/Value_object_addi.php';

/**
 * Used to save a calculate according to expenditure item
 *
 * @property    My_Controller           $CI                     The main CI object
 */
class Correspondence_values extends Value_object_addi
{
    protected $replace;

    protected $project_id;
    protected $is_valid;
    protected $work_package_id;
    protected $type_id;
    protected $is_serial;
    protected $address_or_group_id;
    protected $subject;
    protected $date;
    protected $fee_date;
    protected $due_date;
    protected $fee_no;
    protected $fee_value;
    protected $vat;
    protected $copies;
    protected $orig_id;//for duplicating documents
    protected $duplicate_document_new;//for duplicating documents
    protected $keep_attachments;//for duplicating documents
    protected $keep_original_valid;//for duplicating documents
    
    protected $fee_currency_id;
    protected $fee_exchange_rate;
    
    protected $related_id = null;//Related item ID
    protected $related_uuid = null;//Related UUID
    protected $related_type = null;//Related item Type
    
    protected $discount = 0;
    protected $discount_date = null;
    protected $fee_account_id = null;

    protected $from_date;
    protected $to_date;
    

    public static function init_from_values(
        $add_or_id,
        $project_id,
        $work_package_id,
        $type_id,
        $is_serial,
        $address_or_group_id,
        $subject,
        UsDate $date,
        UsDate $fee_date,
        UsDate $due_date,
        $fee_no,
        $fee_value,
        $vat,
        array $copies=null,
        $orig_id=null,
        $keep_attachments=null,            
        $is_valid=true,
        $keep_original_valid=false,
        $discount=0,
        $discount_date=null,
        $duplicate_document_new=false,
        $fee_currency_id=null,
        $fee_exchange_rate=null,
        $fee_account_id=null,
        $from_date=null,
        $to_date=null,
        ?array $addi_vars=null
    ) {
        $obj = new Correspondence_values();
        $obj->CI = CI_Controller::get_instance();
        
        $obj->add_or_id = $add_or_id;
        $obj->project_id = $project_id;
        $obj->work_package_id = $work_package_id;
        $obj->type_id = $type_id;
        $obj->is_serial = $is_serial;
        $obj->address_or_group_id = $address_or_group_id;
        $obj->subject = trim($subject);
        $obj->date = $date;
        $obj->fee_date = $fee_date;
        $obj->due_date = $due_date;
        $obj->fee_no = $fee_no;
        $obj->fee_value = $fee_value;
        $obj->vat = $vat;
        $obj->copies = $copies;
        $obj->orig_id = $orig_id;
        $obj->keep_attachments = $keep_attachments;
        $obj->is_valid = $is_valid;
        $obj->keep_original_valid = $keep_original_valid;
        
        $obj->fee_currency_id = $fee_currency_id;
        $obj->fee_exchange_rate = $fee_exchange_rate;
        
        $obj->discount = $discount;
        $obj->discount_date = $discount_date;
        $obj->duplicate_document_new = $duplicate_document_new;
        
        $obj->fee_account_id = $fee_account_id;
        
        if (isset($from_date)) {
            $obj->from_date = $from_date;
        }
        if (isset($to_date)) {
            $obj->to_date = $to_date;
        }
        
        if (isset($addi_vars)) {
            $obj->addi_vars = $addi_vars;
        }
        
        return $obj;
    }

    /**
     * Init the values from $_POST
     *
     * @return  object  An Correspondence_values object with all values out of the post array
     */
    public static function init_from_post()
    {
        $obj = new Correspondence_values();
        $obj->CI = CI_Controller::get_instance();
        
        $obj->CI->load->model('System_settings_model', 'system_settings');

        $obj->add_or_id = $obj->CI->input->post('correspondence_edit');
        $obj->project_id = intval($obj->CI->input->post('correspondence_project'));
        $obj->is_valid = $obj->CI->input->post('correspondence_is_valid') ? true : false;
        $obj->work_package_id = intval($obj->CI->input->post('correspondence_work_package'));
        $obj->type_id = intval($obj->CI->input->post('correspondence_type'));
        $obj->is_serial = $obj->CI->input->post('correspondence_is_serial') ? true : false;
        if ($obj->is_serial) {
            $obj->address_or_group_id = intval($obj->CI->input->post('correspondence_receiver_group'));
        } else {
            $obj->address_or_group_id = intval($obj->CI->input->post('correspondence_receiver_one'));
        }
        $obj->subject = trim($obj->CI->input->post('correspondence_subject'));
        $obj->date = new UsDate($obj->CI->input->post('correspondence_date'), null, true, null);
        $obj->fee_date = new UsDate($obj->CI->input->post('correspondence_fee_date'), null, true, null);
        $obj->due_date = new UsDate($obj->CI->input->post('correspondence_due_date'), null, true, null);
        $obj->fee_no = $obj->CI->input->post('correspondence_fee_no');
        $obj->fee_value = $obj->CI->input->post('correspondence_fee_value');
        $obj->vat = $obj->CI->input->post('correspondence_vat');
        $obj->discount = usnumber_to_float($obj->CI->input->post('correspondence_discount'));
        $obj->discount_date = new UsDate($obj->CI->input->post('correspondence_discount_date'), null, true, null);
        $obj->fee_account_id = $obj->CI->input->post('correspondence_fee_account_name');
        $obj->from_date = new UsDate($obj->CI->input->post('correspondence_from_date'), null, true, null);
        $obj->to_date = new UsDate($obj->CI->input->post('correspondence_to_date'), null, true, null);
        
        $obj->update_document = $obj->CI->input->post('correspondence_update_document');
        
        $obj->copies = $obj->CI->input->post('correspondence_copies');
        if ($obj->copies===false) {
            $obj->copies = array();
        } else {
            $obj->copies = (array)$obj->copies;
        }
        
        $obj->orig_id = $obj->CI->input->post('correspondence_orig_id');
        $obj->duplicate_document_new = (strtolower($obj->CI->input->post('correspondence_duplicate_document_new')) === 'true') ? true : false;
        $obj->keep_attachments = (strtolower($obj->CI->input->post('correspondence_keep_attachments')) === 'true') ? true : false;
        $obj->keep_original_valid = (strtolower($obj->CI->input->post('correspondence_keep_original_valid')) === 'true') ? true : false;
        
        $obj->fee_currency_id = null;
        $obj->fee_exchange_rate = null;
        if ($obj->CI->system_settings->getboolProject_use_currency()) {
            $obj->fee_currency_id = $obj->CI->input->post('correspondence_fee_currency_id');
            $obj->fee_exchange_rate = $obj->CI->input->post('correspondence_fee_exchange_rate');
            if (empty($obj->fee_exchange_rate)) {
                $obj->fee_exchange_rate = 1;
            }
            if ($obj->fee_currency_id==='') {
                $obj->fee_currency_id = null;
            }
        }
        
        $obj->addi_vars = $obj->CI->correspondence->get_addi_vars_post();
        return $obj;
    }

    /**
     * Init the values from REST $_POST
     *
     * @param   mixed   $add_or_id  String 'add' or id as integer
     * @return  object  An Correspondence_values object with all values out of the post array
     */
    public static function init_from_rest($add_or_id)
    {
        $obj = new Correspondence_values();
        $obj->CI = CI_Controller::get_instance();
        $obj->add_or_id = $add_or_id;
        $obj->project_id = intval($obj->CI->input->post('project'));
        $obj->is_valid = self::rest_bool_to_boolean($obj->CI->input->post('is_valid'));
        if ($obj->is_valid===null) {
            $obj->is_valid = true;
        }
        $obj->work_package_id = intval($obj->CI->input->post('work_package'));
        $obj->type_id = intval($obj->CI->input->post('type'));
        
        if(!empty(intval($obj->CI->input->post('existing_correspondence_type')))) {
            $obj->type_id = intval($obj->CI->input->post('existing_correspondence_type'));
        }
        
        $obj->is_serial = false;
        $obj->address_or_group_id = intval($obj->CI->input->post('address')); //One - Address, in gui receiver_one
        $obj->subject = trim($obj->CI->input->post('subject'));
        $date = $obj->CI->input->post('date');
        if ($date!==false) {
            $obj->date = new UsDate($date, null, false, null);
        } else {
            $obj->date = new UsDate('now');
        }
        $fee_date = $obj->CI->input->post('fee_date');
        if($fee_date !== false) {
            $obj->fee_date = new UsDate($fee_date, null, false, null);
        } else {
            $obj->fee_date = new UsDate(false);
        }
        $due_date = $obj->CI->input->post('due_date');
        if($due_date !== false) {
            $obj->due_date = new UsDate($due_date, null, false, null);
        } else {
            $obj->due_date = new UsDate(false);
        }
        $obj->fee_no = trim($obj->CI->input->post('fee_no'));
        $obj->fee_value = floatval($obj->CI->input->post('fee_value'));
        $obj->vat = floatval($obj->CI->input->post('vat'));
        $obj->discount = floatval($obj->CI->input->post('discount'));
        $discount_date = $obj->CI->input->post('discount_date');
        if($discount_date !== false) {
            $obj->discount_date = new UsDate($discount_date, null, false, null);
        } else {
            $obj->discount_date = new UsDate(false);
        }
        $obj->fee_account_id = intval($obj->CI->input->post('fee_account_id'));
        
        $obj->update_document = false;
        
        $obj->copies = array();
        
        $obj->orig_id = '';
        $obj->duplicate_document_new = false;
        $obj->keep_attachments = false;
        $obj->keep_original_valid = false;
        
        $obj->fee_currency_id = $obj->CI->input->post('fee_currency_id');
        
        $obj->fee_exchange_rate = null;
        
        #$obj->addi_vars = $obj->CI->correspondence->get_addi_vars_post();
        return $obj;
    }

    /**
     * Validate and ensure all values are appropriate
     *
     * @return  mixed   false on success, else the error message
     *
     * @todo    implement!!
     */
    public function validate()
    {
        if (!is_string($this->add_or_id) && $this->add_or_id!=='add') {
            return 'Item ID has to be an ID';
        }
        
        if ($this->date===null) {
            $this->date = new UsDate('now');
        }
        
        if ($this->is_duplicate_existing()) {
            $this->CI->load->model('Correspondence_types_model', 'correspondence_types', true); 
            
            $orig_id = $this->get_orig_id();
            $orig = $this->CI->correspondence->get_item($orig_id, false);
            if (empty($orig) || empty($orig->file)) {
                return 'Invalid correspondence ID or no file set';
            }
            
            $corr_filename = $orig->file;
            $corr_type = $this->get_type_id();
            
            if (!$this->CI->correspondence_types->is_valid_file_type($corr_type, $corr_filename, false, false, true)) {
                return $this->CI->lang->line('correspondence_dup_type_mismatch').' ('.$corr_type.' - '.$corr_filename.')';
            }            
        }
        
        return false;
    }
    
    /**
     * Get correspondence to save
     *
     * @return  array
     */
    public function get_correspondence_to_save()
    {
        $copies = $this->get_copies();
        $data = array(
            'is_valid'                  => (boolean)$this->is_valid,
            'work_package_id'           => (integer)$this->work_package_id,
            'subject'                   => (!$this->is_really_empty($this->subject)) ? mb_substr($this->subject, 0, 80) : '',
            'date'                      => $this->date->mysql(),
            'copies'                    => ($this->get_is_serial() || empty($copies)) ? null : count($copies),
        );
        
        if (!is_array($this->address_or_group_id)) {
            $data['address_id'] = (integer)$this->address_or_group_id;
        }
        
        $now = new UsDateTime('now');
        if ($this->date->mysqlday()==$now->mysqlday()) {
            if ($this->isNew()) {
                $data['date'] = $now->mysql();
            } else {
                $date = clone $this->date;
                $date->setTime($now->format('H'), $now->format('i'), $now->format('s'));
                $data['date'] = $date->mysql();
            }
        }
        
        if ($this->isNew()) {
            $data['project_id'] = (integer)$this->project_id;//@todo : changing project => move files
            $data['correspondence_type_id'] = (integer)$this->type_id;//@todo : changing type => what happens???
            $data['is_serial'] = (boolean)$this->is_serial;
            $data['related_id'] = $this->related_id;
            $data['related_uuid'] = $this->related_uuid;
            $data['related_type'] = $this->related_type;
            if (!isset($data['address_id'])) {
                $data['address_id'] = null;
            }
            if(isset($this->add_image)) {
                $data['add_image'] = $this->add_image;
            }
        }
        
        if ($this->is_fee()) {
            $data['fee_date'] = $this->fee_date->mysql();
            $data['due_date'] = $this->due_date->mysql();
            $data['fee_no'] = $this->fee_no;
            $vat = usnumber_to_float($this->vat);
            $data['vat'] = ($vat === null) ? 0 : $vat; //ensure fee_value and vat are not null
            
            if (isset($this->fee_account_id) && is_numeric($this->fee_account_id)) {
                $data['fee_account_id'] = (integer)$this->fee_account_id;
            } else {
                $data['fee_account_id'] = null;
            }
            
            if (isset($this->fee_currency_id) && isset($this->fee_exchange_rate)) {
                $value_otcurr = usnumber_to_float($this->fee_value);
                $exchange_rate = usnumber_to_float($this->fee_exchange_rate);
                $data['fee_currency_id'] = $this->fee_currency_id;
                $data['fee_value'] = $value_otcurr*$exchange_rate;
                $data['fee_value_otcurr'] = $value_otcurr;
            } else {
                $fee_value = usnumber_to_float($this->fee_value);
                $data['fee_value'] = ($fee_value === null) ? 0 : $fee_value; //ensure fee_value and vat are not null
            }
            
            if (isset($this->from_date)) {
                $data['from_date'] = $this->from_date->mysql();
            }
            if (isset($this->to_date)) {
                $data['to_date'] = $this->to_date->mysql();
            }

        } else {
            $data['fee_value'] = 0;
        }
        
        if ($this->discount>0 && isset($this->discount_date) && !$this->discount_date->is_null()) {
            $data['discount'] = (float)$this->discount/100;
            $data['discount_date'] = $this->discount_date->mysql();
        }
        
        return $data;
    }
    
    /**
     * Get is serial
     * 
     * @return  boolean
     */
    public function get_is_serial()
    {
        return (boolean)$this->is_serial;
    }
    
    /**
     * Get address or group id
     * 
     * @return  mixed   Address or group ID as integer or array of address IDs
     */
    public function get_address_or_group_id()
    {
        if (!is_array($this->address_or_group_id)) {
            return (integer)$this->address_or_group_id;
        } else {
            return (array)$this->address_or_group_id;
        }
    }
    
    /**
     * Get subject
     * 
     * @return  string
     */
    public function get_subject()
    {
        return $this->subject;
    }

    /**
     * Is fee
     * 
     * @return  boolean
     */
    public function is_fee()
    {
        return ($this->fee_no === '') ? false : true;
    }
    
    /**
     * Get type ID
     * 
     * @return  integer
     */
    public function get_type_id()
    {
        return $this->type_id;
    }
    
    /**
     * Get type item
     * 
     * @return  mixed   Either null or an stdClass object
     */    
    public function get_type()
    {
        $this->CI->load->model('Correspondence_types_model', 'correspondence_types', true); 
        
        $type_id = $this->get_type_id();
        return $this->CI->correspondence_types->get_item($type_id);
    }
    
    /**
     * Get the full ID
     *
     * Only valid if self::isNew() === false
     *
     * @return  integer     The ID
     */
    public function getFullId()
    {
        return (string)$this->add_or_id;
    }  
    
    /**
     * Get the ID
     *
     * Only valid if self::isNew() === false
     *
     * @return  integer     The ID
     */
    public function getId()
    {
        $a_correspondence_id = explode('__', $this->add_or_id);
        return intval($a_correspondence_id[0]);
    }    
    
    /**
     * Get is serial
     * 
     * @return  boolean
     */
    public function get_update_document()
    {
        return (boolean)$this->update_document;
    }    
    
    /**
     * Get copies
     * 
     * @return array
     */
    public function get_copies()
    {
        return $this->copies;
    }    
    
    /**
     * Is duplicate
     * 
     * @return  boolean
     */
    public function is_duplicate()
    {
        return ($this->orig_id === null || $this->orig_id === '') ? false : true;
    }
    
    /**
     * Duplicate document new from preset
     * 
     * @return  boolean
     */
    function get_duplicate_document_new()
    {
        return $this->duplicate_document_new ? true : false;
    }

    /**
     * Is duplicate from existing document
     * 
     * @return  boolean
     */
    public function is_duplicate_existing()
    {
        $duplicate = $this->is_duplicate();
        $document_new = $this->get_duplicate_document_new();
        return $duplicate && !$document_new;
    }
    
    /**
     * Get the original ID
     * 
     * @param   boolean $full_id        Optional: get full id or just the document id
     * 
     * @return  string
     */
    public function get_orig_id($full_id=false)
    {
        if ($full_id === true) {
            return $this->orig_id;
        } else {
            $a_orig_id = explode('__', $this->orig_id);
            return intval($a_orig_id[0]);
        }
    }     
    
    /**
     * Copy the attachments or not
     * 
     * @return  boolean
     */
    public function copy_attachments()
    {
        return ($this->is_duplicate() === true && $this->keep_attachments === true) ? true : false;
    }  
    
    /**
     * Keep original valid or not
     * 
     * @return  boolean
     */
    public function keep_original_valid()
    {
        return ($this->is_duplicate() === true && $this->keep_original_valid === true) ? true : false;
    }
    
    /**
     * Set related Info - Only used on new
     * 
     * @param   integer         $related_id     Related ID
     * @param   string          $related_type   Related Type
     * @param   string|null     $related_uuid   Related UUID
     * @return  void
     */
    public function set_related_id(int $related_id, $related_type, ?string $related_uuid=null)
    {
        $this->related_id = $related_id;
        $this->related_uuid = $related_uuid;
        $this->related_type = $related_type;
    }
    
    /**
     * Set add_image
     * 
     * @param string        $image          Image Data String
     */
    public function set_image($image)
    {
        $this->add_image = $image;
    }
    
    /**
     * Get date
     * 
     * @return  object
     */
    function get_date()
    {
        return $this->date;
    }
    
} // END class Correspondence_values
