<?php
/**
 * CodeIgniter library
 *
 * @category CodeIgniter
 * @license /_licenses/untermStrich/agb_normal.pdf
 * @author Christian Koller
 * @copyright 2009 untermStrich
 * @package us_libraries
 * @version $Id: drawings_values.php 23227 2021-12-03 13:33:05Z christian $
*/

if (!defined('BASEPATH')) exit('No direct script access allowed');

require_once APPPATH.'us_libraries/Value_object_addi.php';

/**
 * Used to save a calculate according to expenditure item
 *
 * @property    My_Controller           $CI                     The main CI object
 */
class Drawings_values extends Value_object_addi
{
    protected $replace;
    protected $set_sub_other;

    protected $cad_file;
    protected $plot_file;
    protected $area;
    protected $date;
    protected $received_from;
    protected $approved;
    protected $is_done;
    
    protected $project_id;
    protected $plan_no;
    protected $plan_content;
    protected $plan_index;
    protected $type_id;
    protected $size_id;
    protected $sub_plan;
    protected $approved_external;
    protected $approval_deadline;
    protected $purpose;
    protected $clipboard;
    protected $signed_by;
    protected $scale;
    protected $redline_remarks;
    protected $is_autopath;
    
    /**
     * Init empty object
     * 
     * @param   string  $add_or_id      Add or ID
     * @return  object  An Drawings_values object with all values out of the post array
     */
    public static function init_empty(
        $add_or_id
    ) {
        $obj = new Drawings_values();
        $obj->CI = CI_Controller::get_instance();
        
        $obj->add_or_id = $add_or_id;
        $obj->plan_no = '';
        
        return $obj;
    }

    /**
     * Init from values
     * 
     * @param   string  $add_or_id      Add or ID
     * ...
     * @return  object  An Drawings_values object with all values out of the post array
     */
    public static function init_from_values(
        $add_or_id,
        $project_id,
        $type_id,
        $plan_no,
        $plan_index,
        $sub_plan,
        $plan_content,
        $redline_remarks,
        
        UsDate $date,
        $scale,
        $size_id,
        $area,
        $cad_file,
        $plot_file,
        $purpose,
        $clipboard,
        $approved,
        UsDate $approval_deadline,
        $approved_external,
        
        $is_done,
        $is_autopath,
        $received_from = false
    ) {
        $obj = new Drawings_values();
        $obj->CI = CI_Controller::get_instance();
        
        $obj->add_or_id = $add_or_id;
        $obj->project_id = $project_id;
        $obj->type_id = $type_id;
        $obj->plan_no = $plan_no;
        $obj->plan_index = $plan_index;
        $obj->sub_plan = $sub_plan;
        $obj->plan_content = $plan_content;
        $obj->redline_remarks = $redline_remarks;
        
        $obj->date = $date;
        $obj->scale = $scale;
        $obj->size_id = $size_id;
        $obj->area = $area;
        $obj->cad_file = $cad_file;
        $obj->plot_file = $plot_file;
        $obj->purpose = $purpose;
        $obj->clipboard = $clipboard;
        $obj->signed_by = '';
        $obj->approved = $approved;
        $obj->approval_deadline = $approval_deadline;
        $obj->approved_external = $approved_external;
        
        $obj->is_done = $is_done;
        $obj->is_autopath = $is_autopath;
        $obj->received_from = $received_from;
        
        return $obj;
    }

    /**
     * Init the values from $_POST
     *
     * @return  object  An Drawings_values object with all values out of the post array
     */
    public static function init_from_post()
    {
        $obj = new Drawings_values();
        $obj->CI = CI_Controller::get_instance();
        $obj->add_or_id = $obj->CI->input->post('drawings_edit');
        $obj->replace = $obj->CI->input->post('drawings_replace');
        $obj->set_sub_other = $obj->CI->input->post('drawings_set_sub_other');
        $obj->project_id = intval($obj->CI->input->post('drawings_project'));
        $obj->type_id = intval($obj->CI->input->post('drawings_drawing_type'));
        $obj->plan_no = $obj->CI->input->post('drawings_plan_no');
        $obj->plan_index = $obj->CI->input->post('drawings_plan_index');
        $obj->sub_plan = $obj->CI->input->post('drawings_sub_plan');
        $obj->plan_content = $obj->CI->input->post('drawings_plan_content');
        $obj->redline_remarks = $obj->CI->input->post('drawings_redline_remarks');
        
        $obj->date = new UsDate($obj->CI->input->post('drawings_date'), null, true, null);
        $obj->received_from = $obj->CI->input->post('drawings_received_from');
        $obj->scale = $obj->CI->input->post('drawings_scale');
        $obj->size_id = intval($obj->CI->input->post('drawings_drawing_size'));
        $obj->area = usnumber_to_float($obj->CI->input->post('drawings_area'));
        $obj->cad_file = $obj->CI->input->post('drawings_cad_file');
        $obj->plot_file = $obj->CI->input->post('drawings_plot_file');
        $obj->purpose = $obj->CI->input->post('drawings_purpose');
        $obj->clipboard = $obj->CI->input->post('drawings_clipboard');
        $obj->signed_by = $obj->CI->input->post('drawings_signed_by');
        $obj->approved = $obj->CI->input->post('drawings_approved') ? true : false;;
        $obj->approval_deadline = new UsDate($obj->CI->input->post('drawings_approval_deadline'), null, true, null);
        $obj->approved_external = $obj->CI->input->post('drawings_approved_external_yn') ? true : false;
        
        $obj->is_done = $obj->CI->input->post('drawings_is_valid') ? false : true;  //is_valid = !is_done
        $obj->is_autopath = false;
        
        $obj->addi_vars = $obj->CI->drawings->get_addi_vars_post();
        return $obj;
    }
    
    /**
     * Init the values from REST $_POST
     *
     * @param   mixed   $add_or_id  String 'add' or id as integer
     * @return  object  An Drawings_values object with all values out of the post array
     */
    public static function init_from_rest($add_or_id)
    {
        $obj = new Drawings_values();
        $obj->CI = CI_Controller::get_instance();
        $obj->add_or_id = $add_or_id;
        $obj->cad_file = $obj->CI->input->post('cad_file');
        $obj->plot_file = $obj->CI->input->post('plot_file');
        $obj->area = $obj->CI->input->post('area');
        $date = $obj->CI->input->post('date');
        if ($date!==false) {
            $obj->date = new UsDate($date, null, false, null);
        } else {
            $obj->date = new UsDate(false);
        }
        $obj->approved = self::rest_bool_to_boolean($obj->CI->input->post('approved'));
        $obj->is_done = self::rest_bool_to_boolean($obj->CI->input->post('is_done'));
        
        $obj->project_id = intval($obj->CI->input->post('project_id'));
        $obj->plan_no = $obj->CI->input->post('plan_no');
        $obj->plan_content = $obj->CI->input->post('plan_content');
        $obj->plan_index = $obj->CI->input->post('plan_index');
        $obj->type_id = intval($obj->CI->input->post('type_id'));
        $obj->size_id = intval($obj->CI->input->post('size_id'));
        $obj->sub_plan = $obj->CI->input->post('sub_plan');
        $obj->approved_external = self::rest_bool_to_boolean($obj->CI->input->post('approved_external'));
        $approval_deadline= $obj->CI->input->post('approval_deadline');
        if ($approval_deadline!==false) {
            $obj->approval_deadline = new UsDate($approval_deadline, null, false, null);
        } else {
            $obj->approval_deadline = new UsDate(false);
        }
        $obj->purpose = $obj->CI->input->post('purpose');
        $obj->clipboard = $obj->CI->input->post('clipboard');
        $obj->signed_by = '';
        $obj->scale = $obj->CI->input->post('scale');
        $obj->redline_remarks = $obj->CI->input->post('redline_remarks');
        $obj->is_autopath = false;
        
        #$obj->addi_vars = $obj->CI->drawings->get_addi_vars_post();
        return $obj;
    }

    /**
     * Validate and ensure all values are appropriate
     *
     * @return  mixed   false on success, else the error message
     *
     * @todo    implement!!
     */
    public function validate()
    {
        if (!is_numeric($this->add_or_id) && $this->add_or_id!=='add') {
            return 'Item ID has to be an ID';
        }
        
        if ($this->date===null) {
            $this->date = new UsDate('now');
        }
        
        return false;
    }
    
    /**
     * Get drawing update cad file values
     *
     * @param   boolean     $file_only      Optional File only
     * @return  array
     */
    public function get_drawing_update_cad_file($file_only=false)
    {
        $data = array(
            'cad_file'          => $this->cad_file,
        );
        
        if ($file_only) {
            return $data;
        }
        
        if ($this->area!==false) {
            $data['area'] = (float)$this->area;
        }
        if ($this->date!==null) {
            $data['date'] = $this->date->mysql();
        }
        if ($this->approved!==null) {
            $data['approved'] = $this->approved;
        }
        if ($this->is_done!==null) {
            $data['is_done'] = $this->is_done;
        }
        
        return $data;
    }
    
    /**
     * Get drawing update plot file values
     *
     * @param   boolean     $file_only      Optional File only
     * @return  array
     */
    public function get_drawing_update_plot_file($file_only=false)
    {
        $data = array(
            'plot_file'         => $this->plot_file,
        );
        
        if ($file_only) {
            return $data;
        }
        
        if ($this->area!==false) {
            $data['area'] = (float)$this->area;
        }
        if ($this->date!==null) {
            $data['date'] = $this->date->mysql();
        }
        if ($this->approved!==null) {
            $data['approved'] = $this->approved;
        }
        if ($this->is_done!==null) {
            $data['is_done'] = $this->is_done;
        }
        
        return $data;
    }
    
    /**
     * Get drawing to save
     *
     * @return  array
     */
    public function get_drawing_to_save()
    {
        if ($this->is_done) {
            $this->approved = false;
            $this->approved_external = false;
        }
        
        $data = array(
            'project_id'        => (integer)$this->project_id,
            'plan_no'           => (!$this->is_really_empty($this->plan_no)) ? mb_substr($this->plan_no, 0, 255) : '',
            'plan_content'      => (!$this->is_really_empty($this->plan_content)) ? mb_substr($this->plan_content, 0, 255) : '',
            'plan_index'        => (!$this->is_really_empty($this->plan_index)) ? mb_substr($this->plan_index, 0, 10) : '',
            'type_id'           => (integer)$this->type_id,
            
            'area'              => (float)$this->area,
            'date'              => $this->date->mysql(),
            'received_from'     => null,
            'is_done'           => (boolean)$this->is_done,
            'sub_plan'          => (!$this->is_really_empty($this->sub_plan)) ? mb_substr($this->sub_plan, 0, 50) : '',
            
            'approved'          => (boolean)$this->approved,
            'approval_deadline' => $this->approval_deadline->mysql(),
            'approved_external' => (boolean)$this->approved_external,
            'purpose'           => (!$this->is_really_empty($this->purpose)) ? $this->purpose : '',
            
            'clipboard'         => (!$this->is_really_empty($this->clipboard)) ? mb_substr($this->clipboard, 0, 120) : '',
            'signed_by'         => ($this->signed_by) ? $this->signed_by : null,
            'size_id'           => (integer)$this->size_id,
            'scale'             => (!$this->is_really_empty($this->scale)) ? mb_substr($this->scale, 0, 30) : '',
            'redline_remarks'   => (!$this->is_really_empty($this->redline_remarks)) ? mb_substr($this->redline_remarks, 0, 100) : '',
            'is_autopath'       => (boolean)$this->is_autopath,
        );
        
        if ($this->cad_file!==false && !empty($this->cad_file)) {
            if($this->cad_file=='#DELETEME#') {
                $data['cad_file'] = null;
            } else {
                $data['cad_file'] = $this->cad_file;
            }
        }
        if ($this->plot_file!==false && !empty($this->plot_file)) {
            if($this->plot_file=='#DELETEME#') {
                $data['plot_file'] = null;
            } else {
                $data['plot_file'] = $this->plot_file;
            }
        }
        if ($this->received_from!==false && !empty($this->received_from)) {
            $data['received_from'] = intval($this->received_from);
        }
        
        return $data;
    }
    
    /**
     * Get CAD file
     * 
     * @return  string
     */
    public function get_cad_file()
    {
        return $this->cad_file;
    }

    /**
     * Set CAD file
     * 
     * @param   string  $cad_file   File name
     */
    public function set_cad_file($cad_file)
    {
        $this->cad_file = $cad_file;
    }

    /**
     * Get Plot file
     * 
     * @return  string
     */
    public function get_plot_file()
    {
        return $this->plot_file;
    }

    /**
     * Set Plot file
     * 
     * @param   string  $cad_file   File name
     */
    public function set_plot_file($plot_file)
    {
        $this->plot_file = $plot_file;
    }

    /**
     * Get project ID
     * 
     * @return  integer
     */
    public function get_project_id()
    {
        return $this->project_id;
    }
    
    /**
     * Replace old
     * 
     * @return  integer
     */
    public function get_replace()
    {
        return $this->replace;
    }
    
    /**
     * Set sub other ID
     * 
     * @return  integer
     */
    public function get_set_sub_other()
    {
        return $this->set_sub_other;
    }
    
    /**
     * Get details
     * 
     * @return  object
     */
    public function get_plan_details()
    {
        $reply = new stdClass();
            $reply->type = $this->type_id;
            $reply->no = $this->plan_no;
            $reply->sub = $this->sub_plan;
            $reply->index = $this->plan_index;
        return $reply;
    }
    
} // END class Drawings_values