<?php
/**
 * CodeIgniter library
 *
 * @category CodeIgniter
 * @license /_licenses/untermStrich/agb_normal.pdf
 * @author Christian Koller
 * @copyright 2009 untermStrich
 * @package us_libraries
 * @version $Id: drawings_values.php 20419 2020-01-09 14:04:51Z christian $
*/

if (!defined('BASEPATH')) exit('No direct script access allowed');

require_once APPPATH.'us_libraries/US_exmodel.php';
require_once APPPATH.'us_libraries/Value_object_addi.php';

/**
 * Used to save addi edit values
 *
 * @property    My_Controller           $CI                     The main CI object
 */
class Edit_addi_values extends Value_object_addi
{
    /**
     * Init the values from $_POST
     *
     * @return  object  An Edit_addi_values object with all values out of the post array
     */
    public static function init_from_post(US_exmodel $model, $section)
    {
        $obj = new Edit_addi_values();
        $obj->CI = CI_Controller::get_instance();
        $obj->add_or_id = $obj->CI->input->post($section.'edit');
        $obj->addi_vars = $model->get_addi_vars_post($section);
        
        return $obj;
    }
    
    /**
     * Init the values from $_POST for multiple
     *
     * @return  object  An Edit_addi_values object with all values out of the post array
     */
    public static function init_from_post_multi(US_exmodel $model, $section)
    {
        $obj = new Edit_addi_values();
        $obj->CI = CI_Controller::get_instance();
        $obj->addi_vars = $model->get_addi_vars_post($section, true);
        
        return $obj;
    }
    
    /**
     * Get the ID
     *
     * Only valid if self::isNew() === false
     *
     * @return  string      The ID
     */
    public function getId()
    {
        return $this->add_or_id;
    }
    
    /**
     * Set the ID
     * 
     * @param   string      $id     The ID
     * @return  void
     */
    public function setId($id)
    {
        $this->add_or_id = $id;
    }
    
    /**
     * Validate and ensure all values are appropriate
     *
     * @return  mixed   false on success, else the error message
     *
     * @todo    implement!!
     */
    public function validate()
    {
        if (!is_numeric($this->add_or_id)) {
            
            $a_id = explode('___',$this->add_or_id);
            if (!is_array($a_id) && count($a_id) != 2) {
                return 'Item ID has to be an ID';
            }
            
        }
        
        return false;
    }
    
    /**
     * Get the invoiced type of the Fee or Externalexpense
     *
     * @return  boolean
     */
    public function get_invoiced_type()
    {
        $id = $this->getId();
        if( $id == 'add') {
            $type = $this->is_invoiced;
        } else {
            $a = explode('___', $id);
            if(!is_array($a) || count($a) !== 2){
                basic_error('get_invoiced_type - The id "'.$id.'" cannot be splitted.', null, __CLASS__.'::'.__FUNCTION__);
            }
            $type = $a[1];
        }

        switch ($type) {
            case 1:
                return 1;
                break;
            case 0:
                return 0;
                break;
            default:
                basic_error('get_invoiced_type - Invoiced Type is invalid!', null, __CLASS__.'::'.__FUNCTION__);
                break;
        }
    }
    
}
