<?php
/**
 * CodeIgniter library
 *
 * @category CodeIgniter
 * @license /_licenses/untermStrich/agb_normal.pdf
 * @author Christian Wengg
 * @copyright 2009 untermStrich
 * @package us_libraries
 * @version $Id: mailinbox_values.php 23227 2021-12-03 13:33:05Z christian $
*/

if (!defined('BASEPATH')) exit('No direct script access allowed');

require_once APPPATH.'us_libraries/Value_object_addi.php';

/**
 * Used to save a mailinbox entry
 *
 * @property    My_Controller           $CI                     The main CI object
 */
class Mailinbox_values extends Value_object_addi
{
    protected $email_allocation_id;
    protected $recipient_id;
    protected $project_id;
    protected $date_date;
    protected $date_time;
    protected $no;
    protected $inbox_index;
    protected $content;
    protected $dispatch_mode_id;
    protected $work_package_id;
    protected $address_id;
    protected $attached_file;
    protected $security;
    protected $delete_emls;
    protected $projectmail_account_id = 0;
    private $_is_projectmail_autoimport = false;

    /**
     * Init the mailinbox values from $_POST
     *
     * @return  object  An Mailinbox_values object with all values out of the post array
     */
    public static function init_from_post()
    {
        $obj = new Mailinbox_values();
        $obj->CI = CI_Controller::get_instance();
        $obj->add_or_id = $obj->CI->input->post('mailinbox_edit');
        $obj->email_allocation_id = $obj->CI->input->post('mailinbox_email_allocation_id');
        $obj->project_id = $obj->CI->input->post('mailinbox_project');
        $obj->date_date = new UsDate($obj->CI->input->post('mailinbox_date__date'), null, true, null);
        $obj->date_time = new UsTime($obj->CI->input->post('mailinbox_date__time'));        
        $obj->no = $obj->CI->input->post('mailinbox_no');
        $obj->inbox_index = $obj->CI->input->post('mailinbox_inbox_index');
        $obj->content = $obj->CI->input->post('mailinbox_content');
        $obj->dispatch_mode_id = $obj->CI->input->post('mailinbox_dispatch_mode');
        $obj->work_package_id = $obj->CI->input->post('mailinbox_work_package_name');
        $obj->address_id = $obj->CI->input->post('mailinbox_address');
        $obj->attached_file = $obj->CI->input->post('mailinbox_attached_file');
        $obj->security = $obj->CI->input->post('mailinbox_security');
        $obj->recipient_id = $obj->CI->input->post('mailinbox_recipient');
        $obj->delete_emls = (strtolower($obj->CI->input->post('mailinbox_delete_emls')) === 'true') ? true : false;
        $obj->addi_vars = $obj->CI->inbox->get_addi_vars_post();
        return $obj;
    }
    
    /**
     * Init the mailinbox values from a given StdClass object
     *
     * @param   object  $values StdClass object with mailoutbox values to initialize a valid Mailinbox_values object out of it
     *
     * @return  object  An Mailinbox_values object with all values out of the given StdClass object
     */
    public static function init_from_values(StdClass $values)
    {
        $obj = new Mailinbox_values();
        $obj->CI = CI_Controller::get_instance();
        $obj->add_or_id = $values->id;
        $obj->email_allocation_id = $values->email_allocation_id;
        $obj->project_id = $values->project_id;
        $obj->date_date = new UsDate($values->date_date, null, true, null);
        $obj->date_time = new UsTime($values->date_time);        
        $obj->no = (isset($values->no)) ? $values->no : '';
        $obj->inbox_index = (isset($values->inbox_index)) ? $values->inbox_index : '';
        $obj->content = (isset($values->content)) ? $values->content : '';
        $obj->dispatch_mode_id = $values->dispatch_mode_id;
        $obj->work_package_id = (isset($values->work_package_id)) ? $values->work_package_id : 0;
        $obj->attached_file = (isset($values->attached_file)) ? $values->attached_file : null;
        $obj->security = $values->security;
        $obj->address_id = $values->address_id;
        $obj->recipient_id = $values->recipient_id;
        $obj->delete_emls = (isset($values->delete_emls)) ? $values->delete_emls : false;
        $obj->projectmail_account_id = (isset($values->projectmail_account_id)) ? $values->projectmail_account_id : 0;
        
        if (isset($values->addi_fields)) {
            $a_addi = array();
            foreach ($values->addi_fields as $addi) {
                $addi_field = $addi->field;
                $a_addi[$addi_field] = $values->$addi_field;
            }
            $obj->addi_vars = $a_addi;            
        }

        return $obj;
    }    
    
    /**
     * Getter/Setter for projectmail_autoimport flag
     * 
     * @param mixed     $set    If setter - a boolean value
     * @return mixed
     */
    public function is_projectmail_autoimport($set=null) {
        if ($set === null) {
            return boolval($this->_is_projectmail_autoimport);
        } else {
            $this->_is_projectmail_autoimport = boolval($set);
        }
    }
    
    /**
     * Should we delete the eml file from server after saving
     * 
     * @return boolean
     */
    public function is_delete_emls()
    {
        return ($this->delete_emls === true) ? true : false;
    }

    /**
     * Validate and ensure all values are appropriate
     *
     * @return  mixed   false on success, else the error message
     *
     * @todo    implement!!
     */
    public function validate()
    {
        if ($this->add_or_id!=='add' && !is_numeric($this->add_or_id)) {
            return 'Mailinbox ID has to be "add" or an ID';
        }

        return false;
    }
    
    /**
     * Is email alloaction or not
     * 
     * @return  boolan 
     */
    public function is_email_allocation()
    {
        return (is_string($this->email_allocation_id) && strlen(trim($this->email_allocation_id)) > 1) ? true : false;
    }        
    
    /**
     * Get the email alloaction id
     * 
     * @return mixed     Either the id string or false
     */
    public function get_email_allocation_id()
    {
        return ($this->is_email_allocation()) ? $this->email_allocation_id : false;
    }            

    /**
     * Get externalexpense to save
     *
     * @return  array
     */
    public function get_mailinbox_to_save()
    {
        $data = array(
            'project_id'                =>  intval($this->project_id),
            'recipient_id'              =>  intval($this->recipient_id),
            'no'                        =>  $this->no,
            'inbox_index'               =>  $this->inbox_index,
            'content'                   =>  $this->content,
            'dispatch_mode_id'          =>  intval($this->dispatch_mode_id),
            'work_package_id'           =>  intval($this->work_package_id),
            'address_id'                =>  intval($this->address_id),
            'attached_file'             =>  $this->attached_file,
            'security'                  =>  intval($this->security),
        );

        if($this->isNew()) {
            $data['projectmail_account_id'] = intval($this->projectmail_account_id);
        }
        
        if (!$this->date_date->is_null()) {
            $date = $this->date_date->mysqlday();
            if($this->date_time->is_null()){
                $time = '00:00:00';
            } else {
                $time = $this->date_time->us_ltime();
            }
            $data['date'] = $date.' '.$time;
        } else {
            $data['date'] = null;
        }        
        
        if ($this->attached_file!==false && !empty($this->attached_file)) {
            if($this->attached_file=='#DELETEME#') {
                $data['attached_file'] = null;
            } else {
                $data['attached_file'] = $this->attached_file;
            }
        }        

        if ($this->isNew() && $this->is_email_allocation()) {
            $this->CI->load->model('Email_cache_model', 'email_cache_model', true);
            list($mailbox_id, $in_out, $mbox_folder, $uid, $email_cache_data_id) = $this->CI->email_cache_model->split_id($this->get_email_allocation_id());
            
            $email_cache_item = $this->CI->email_cache_model->get_item_by_id($email_cache_data_id);
            if ($email_cache_item !== null) {
                $data['email_cache_hash'] = $email_cache_item->email_cache_hash;
            }

            $data['email_cache_id'] = intval($email_cache_data_id);
            $data['email_account_id'] = intval($mailbox_id);
        }

        return $data;
    }

    /**
     * Allocate a email now or later
     * 
     * @return boolean
     */
    public function allocate_email_later() 
    {
        if (!$this->is_email_allocation()) return false;
        
        
        $this->CI->load->model('Email_cache_model', 'email_cache_model', true);
        list($mailbox_id, $in_out, $mbox_folder, $uid, $email_cache_data_id) = $this->CI->email_cache_model->split_id($this->get_email_allocation_id());
        
        if ($this->isNew() && $this->CI->email_cache_model->get_email_accounts_import_eml_later(intval($mailbox_id))) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * Get the ID
     *
     * Only valid if self::isNew() === false
     *
     * @return  integer     The ID
     */
    public function getId()
    {
        return $this->add_or_id;
    }
    
    /**
     * Get the project ID
     * 
     * @return  integer
     */
    public function get_project()
    {
        return $this->project_id;
    }
    
    /**
     * Get security flag
     * 
     * @return  integer
     */
    public function get_security()
    {
        return $this->security;
    }
    
} // END class Mailinbox_values
