<?php
/**
 * CodeIgniter library
 *
 * @category CodeIgniter
 * @license /_licenses/untermStrich/agb_normal.pdf
 * @author Christian Koller
 * @copyright 2009 untermStrich
 * @package us_libraries
 * @version $Id: personalapp_dav_values.php 26862 2024-02-01 17:03:14Z christian $
*/

if (!defined('BASEPATH')) exit('No direct script access allowed');

require_once APPPATH.'us_libraries/value_objects/personalapp_values.php';

/**
 * Used to save a personal appointment via DAV
 *
 * @property    My_Controller           $CI                     The main CI object
 */
class Personalapp_dav_values extends Personalapp_values
{
    /**
     * Init for webdav call
     * 
     * @param   string  $calendarData   ICS File data
     * @param   string  $objectUri      The URI
     * @param   object  $personalapp    Current personalappointment, if any
     * @param   int     $calendarId     Calendar/Staff ID
     * @return  object  An Personalapp_values object with all values out of the given calendarData
     * @throws \InvalidArgumentException
     */
    public static function init_from_dav(string $calendarData, string $objectUri, stdClass $personalapp=null, int $calendarId=null): Personalapp_values
    {
        $CI = CI_Controller::get_instance();
        
        $CI->load->library('import_parser/US_icalendar_parser', null, 'icalendar_parser');
        
        if (isset($calendarId)) {
            $CI->icalendar_parser->setStaff_to_id($calendarId);
        }
        
        $ok = $CI->icalendar_parser->parse_string($calendarData, 'ics');
        if ($ok!==false) {
            throw new \InvalidArgumentException('ICS Parser: '.$ok);
        }
        
        $ics_list = $CI->icalendar_parser->getList();
        if (count($ics_list)<=0) {
            throw new \InvalidArgumentException('ICS Parser did not get any data to read');
        }
        $obj = array_shift($ics_list);
        
        $obj->cache_appointments_to_save = null;
        if ($personalapp===null) {
            $obj->add_or_id = 'add';
            if (!isset($obj->project)) {
                $obj->project = $CI->icalendar_parser->getDefault_project_id();
            }
        } else {
            $obj->add_or_id = $personalapp->id;
            if (empty($obj->project)) {
                $obj->project = null;
            }
        }
        $obj->sabre_uri = $objectUri;
        $obj->sabre_calendar_data_obj = $CI->icalendar_parser->getVcalendar();
        $obj->mobile_dav_mode = true;
        
        $obj->editconfirm_dialog_sel_user = null;
        $obj->editconfirm_dialog_all_users = null;
        $obj->editconfirm_dialog_from_today = false;
        $obj->allow_change_changed = false;
        
        return $obj;
    }

}