<?php
/**
 * CodeIgniter library
 *
 * @category CodeIgniter
 * @license /_licenses/untermStrich/agb_normal.pdf
 * @author Christian Wengg
 * @copyright 2009 untermStrich
 * @package us_libraries
 * @version $Id: project_offers_values.php 26738 2024-01-12 14:03:58Z pansidom $
*/

if (!defined('BASEPATH')) exit('No direct script access allowed');

require_once APPPATH.'us_libraries/Value_object.php';

/**
 * Used to save the project offers
 *
 * @property    My_Controller           $CI                     The main CI object
 */
class Project_offers_values extends Value_object
{
    const EXTRAS_TYPE_VALUE = 0;
    const EXTRAS_TYPE_PCTG = 1;
    
    const REBATE_TYPE_VALUE = 0;
    const REBATE_TYPE_PCTG = 1;
    
    const CALC_EXTRAS_FIRST = 0;
    const CALC_REBATE_FIRST = 1;
    
    protected $project_id;
    protected $offer_no;
    protected $title;
    protected $date;
    protected $time;
    protected $address_id;
    protected $status;
    protected $name;
    protected $valid_to_date;
    protected $order_no;
    protected $order_date;
    protected $order_probability;
    protected $file;
    protected $staff_id;
    protected $is_calcexp;
    protected $is_estimation;
    protected $portlet_desc;
    protected $portlet_data;
    protected $fee_allocation_vars;
    
    protected $retention_type;
    protected $retention_years;
    protected $retention;
    protected $extras_type;
    protected $rebate_type;
    protected $calc_rebate_first;

    protected $currency_id;
    protected $exchange_rate;
    
    /**
     * Init the project offers values from $_POST
     *
     * @return  object  An Project_offers_values object with all values out of the post array
     */
    public static function init_from_post()
    {
        $obj = new Project_offers_values();
        $obj->CI = CI_Controller::get_instance();
        
        $obj->add_or_id = $obj->CI->input->post('project_offers_edit');
        $obj->project_id = $obj->CI->input->post('project_offers_project');
        $obj->offer_no = $obj->CI->input->post('project_offers_offer_no');
        $obj->title = $obj->CI->input->post('project_offers_title');
        $obj->date = new UsDate($obj->CI->input->post('project_offers_date'), null, true, null);
        $obj->time = new UsTime($obj->CI->input->post('project_offers_time'));
        $obj->address_id = $obj->CI->input->post('project_offers_address');
        $obj->status = $obj->CI->input->post('project_offers_status');
        $obj->name = $obj->CI->input->post('project_offers_name');
        $obj->valid_to_date = new UsDate($obj->CI->input->post('project_offers_valid_to_date'), null, true, null);
        $obj->order_no = $obj->CI->input->post('project_offers_order_no');
        $obj->order_date = new UsDate($obj->CI->input->post('project_offers_order_date'), null, true, null);
        $obj->order_probability = $obj->CI->input->post('project_offers_order_probability');
        $obj->vat_rate = usnumber_to_float($obj->CI->input->post('project_offers_vat_rate'));
        $obj->file = $obj->CI->input->post('project_offers_file');
        $obj->staff_id = $obj->CI->input->post('project_offers_staff_id');
        $obj->is_calcexp = false;
        $obj->is_estimation = (boolean) $obj->CI->input->post('project_offers_estimation');
        $obj->portlet_desc = $obj->CI->input->post('project_offers_portlet_desc');
        $obj->portlet_data = $obj->CI->input->post('project_offers_portlet_data');
        
        $obj->retention_type = intval($obj->CI->input->post('project_offers_retention_type'));
        $obj->retention_years = intval($obj->CI->input->post('project_offers_retention_years'));
        $obj->retention = abs(usnumber_to_float($obj->CI->input->post('project_offers_retention')) ?? 0);
        
        $obj->extras_type = intval($obj->CI->input->post('project_offers_extras_type'));
        $obj->rebate_type = intval($obj->CI->input->post('project_offers_rebate_type'));
        $obj->calc_rebate_first = intval($obj->CI->input->post('project_offers_calc_rebate_first'));
        
        $obj->repetition = intval($obj->CI->input->post('project_offers_hoai_repetition'));
        
        $obj->currency_id = null;
        $obj->exchange_rate = null;
        if ($obj->CI->system_settings->getboolProject_use_currency()) {
            $obj->currency_id = $obj->CI->input->post('project_offers_currency_id');
            $obj->exchange_rate = $obj->CI->input->post('project_offers_exchange_rate');
        }
        
        $obj->fee_allocation_vars = $obj->_get_fee_allocation_vars_post();
        return $obj;
    }
    
    /**
     * Init the project offers values from a given StdClass object
     * 
     * @param   object  $values StdClass object with offers values to initialize a valid Project_offers_values object out of it
     *
     * @return  object  An Project_offers_values object with all values out of the given StdClass object
     */
    public static function init_from_values(StdClass $values)
    {
        $obj = new Project_offers_values();
        $obj->CI = CI_Controller::get_instance();
        
        $obj->add_or_id = $values->id;
        $obj->project_id = $values->project_id;
        $obj->offer_no = (isset($values->offer_no)) ? $values->offer_no : '';
        $obj->title = (isset($values->title)) ? $values->title : '';
        $obj->date = $values->date;
        $obj->time = (isset($values->time)) ? $values->time : null;
        $obj->address_id = (isset($values->address_id)) ? $values->address_id : null;
        $obj->status = $values->status;
        $obj->name = (isset($values->name)) ? $values->name : '';
        $obj->valid_to_date = (isset($values->valid_to_date)) ? $values->valid_to_date : new UsDate('');
        $obj->order_no = (isset($values->order_no)) ? $values->order_no : '';
        $obj->order_date = (isset($values->order_date)) ? $values->order_date : new UsDate('');
        $obj->order_probability = (isset($values->order_probability)) ? $values->order_probability : null;
        $obj->vat_rate = (isset($values->vat_rate)) ? $values->vat_rate : null;
        $obj->file = (isset($values->file)) ? $values->file : null;
        $obj->staff_id = (isset($values->staff_id)) ? $values->staff_id : 0;
        $obj->is_calcexp = (isset($values->is_calcexp)) ? $values->is_calcexp : false;
        $obj->is_estimation = (isset($values->is_estimation)) ? $values->is_estimation : false;
        $obj->portlet_desc = (isset($values->portlet_desc)) ? $values->portlet_desc : '';
        $obj->portlet_data = (isset($values->portlet_data)) ? $values->portlet_data : '';
        
        $obj->repetition = 0;
        if(!empty($obj->portlet_data)) {
            $portlet_data = new stdClass();
            if (!empty($obj->portlet_data)) {
                $portlet_data = @json_decode($obj->portlet_data);
            }
            
            if(!empty($portlet_data->repetition)) {
                $obj->repetition = $portlet_data->repetition;
            }
        }
        
        $obj->currency_id = null;
        if (isset($values->currency_id)) {
            $obj->currency_id = $values->currency_id;
        }
        $obj->exchange_rate = null;
        if (isset($values->exchange_rate)) {
            $obj->exchange_rate = float_to_usnumber($values->exchange_rate);
        }
            
        $obj->retention_type = 0;
        if (isset($values->retention_type)) {
            $obj->retention_type = intval($values->retention_type);
        }
        $obj->retention_years = 0;
        if (isset($values->retention_years)) {
            $obj->retention_years = intval($values->retention_years);
        }
        $obj->retention = 0;
        if (isset($values->retention)) {
            $obj->retention = abs(floatval($values->retention));
        }
        $obj->extras_type = 1;
        if (isset($values->extras_type)) {
            $obj->extras_type = intval($values->extras_type);
        }
        $obj->rebate_type = 1;
        if (isset($values->rebate_type)) {
            $obj->rebate_type = intval($values->rebate_type);
        }
        $obj->calc_rebate_first = 0;
        if (isset($values->calc_rebate_first)) {
            $obj->calc_rebate_first = intval($values->calc_rebate_first);
        }
        $extras_is_percentage =  (intval($obj->extras_type) === self::EXTRAS_TYPE_PCTG) ? true : false;
        $rebate_is_percentage =  (intval($obj->rebate_type) === self::REBATE_TYPE_PCTG) ? true : false;
        $extras_allocation_name = $extras_is_percentage ? 'project_offers_extras_' : 'project_offers_extras_value_';
        $rebate_allocation_name = $rebate_is_percentage ? 'project_offers_rebate_' : 'project_offers_rebate_value_';
        
        $obj->fee_allocation_vars = array(
            'percent_sum'   => 0,
            'costs_sum'     => 0,
            'is_costs_sum'  => 0,
        );
        foreach ($values->costs as $costkey => $costvalue) {
            if (is_array($costvalue)) {
                if(!empty($costvalue['custom_cost'])) {
                    $obj->fee_allocation_vars['project_offers_custom_costs_fee_view_'.$costkey] = $costvalue['custom_cost'];
                } else {
                    $obj->fee_allocation_vars['project_offers_custom_costs_fee_view_'.$costkey] = null;
                }
                if (isset($costvalue['fee'])) {
                    $obj->fee_allocation_vars['project_offers_costs_fee_'.$costkey] = $costvalue['fee'];
                }
                if (isset($costvalue['fee_view'])) {
                    $obj->fee_allocation_vars['project_offers_costs_fee_view_'.$costkey] = $costvalue['fee_view'];
                }
                if (isset($costvalue['supplement'])) {
                    $obj->fee_allocation_vars['project_offers_supplement_'.$costkey] = $costvalue['supplement'];
                }
                if (isset($costvalue['extras'])) {
                    $obj->fee_allocation_vars[$extras_allocation_name.$costkey] = $costvalue['extras'];
                }
                if (isset($costvalue['rebate'])) {
                    $obj->fee_allocation_vars[$rebate_allocation_name.$costkey] = $costvalue['rebate'];
                }
                if(isset($costvalue['calc_rep'])) {
                    $obj->fee_allocation_vars['project_offers_calc_rep_'.$costkey] = $costvalue['calc_rep'];
                }
                if(isset($costvalue['calc_rep_wp'])) {
                    $obj->fee_allocation_vars['project_offers_calc_rep_wp_'.$costkey] = $costvalue['calc_rep_wp'];
                }
                if(isset($costvalue['costs_rep'])) {
                    $obj->fee_allocation_vars['project_offers_costs_rep_'.$costkey] = $costvalue['costs_rep'];
                }
            } else {
                $obj->fee_allocation_vars['project_offers_costs_fee_view_'.$costkey] = $costvalue;
            }
        }
        if (isset($values->totalsum)) {
            $obj->fee_allocation_vars['costs_sum'] = $values->totalsum;
        }
        
        return $obj;
    }
    
    /**
     * Get an array with the fee allocation fields values
     *
     * This function loads the value form $_POST
     *
     * @return  array   Values
     */
    private function _get_fee_allocation_vars_post()
    {
        $vars = array();
        $vars['percent_sum'] = usnumber_to_float($this->CI->input->post('project_offers_percent_sum'));
        $vars['costs_sum'] = usnumber_to_float($this->CI->input->post('project_offers_costs_sum'));
        $vars['is_costs_sum'] = $this->CI->input->post('project_offers_is_costs_sum') ? true : false;
        
        $post_keys = preg_grep("@^project_offers_(percent_fee_|custom_costs_fee_view_|costs_fee_view_|supplement_|extras_|extras_value_|rebate_|rebate_value_|calc_rep_|calc_rep_wp_|costs_rep_)[0-9]+@", array_keys($_POST));
        foreach ($post_keys as $key) {
            $vars[$key] = usnumber_to_float($this->CI->input->post($key));
        }
        
        return $vars;
    }
    
    /**
     * Get main project offer values to save (without the fee_allocation values)
     *
     * @return  array
     */
    public function get_main_vars_to_save()
    {
        $data = array(
            'project_id'        =>  intval($this->project_id),
            'currency_id'       =>  isset($this->currency_id) ? intval($this->currency_id) : null,
            'address_id'        =>  intval($this->address_id),
            'offer_no'          =>  $this->offer_no,
            'title'             =>  $this->title,
            'date'              =>  $this->date->mysql(),
            'time'              =>  isset($this->time) ? ($this->time->us_stime() !== '' ? $this->time->us_stime() : null) : null,
            'status'            =>  intval($this->status),
            'name'              =>  $this->name,
            'valid_to_date'     =>  $this->valid_to_date->mysql(),
            'order_no'          =>  $this->order_no,
            'order_date'        =>  $this->order_date->mysql(),
            'order_probability' =>  intval($this->order_probability),
            'vat_rate'          =>  floatval($this->vat_rate),
            'staff_id'          =>  intval($this->staff_id),
            'is_calcexp'        =>  (boolean)$this->is_calcexp,
            'is_estimation'     =>  (boolean)$this->is_estimation,
            'retention_type'    =>  intval($this->retention_type),
            'retention_years'   =>  intval($this->retention_years),
            'retention'         =>  floatval($this->retention),
            'extras_type'       =>  intval($this->extras_type),
            'rebate_type'       =>  intval($this->rebate_type),
            'calc_rebate_first' =>  intval($this->calc_rebate_first)
        );
        
        if ($this->file!==false && !empty($this->file)) {
            if($this->file=='#DELETEME#') {
                $data['file'] = null;
            } else {
                $data['file'] = $this->file;
            }
        }
        
        if (!empty($this->portlet_desc)) {
            $data['portlet_desc'] = $this->portlet_desc;
        }
        if (!empty($this->portlet_data)) {
            $data['portlet_data'] = $this->portlet_data;
        }

        return $data;
    }    

    /**
     * Get projects offer fee allocations to save
     *
     * @return  array
     */
    public function get_fee_allocation_vars_to_save()
    {
        $this->CI->load->model('Work_packages_model', 'work_packages', true);
        
        $this_request_vars = new stdClass();
        $this_request_vars->project = $this->get_project_id();
        $work_packages_list = $this->CI->work_packages->get_list($this_request_vars->project, Work_packages_model::WHICH_FEE_AREA_TREE);
        $extras_is_percentage =  (intval($this->extras_type) === self::EXTRAS_TYPE_PCTG) ? true : false;
        $rebate_is_percentage =  (intval($this->rebate_type) === self::REBATE_TYPE_PCTG) ? true : false;
        $calc_rebate_first =  (intval($this->calc_rebate_first) === self::CALC_REBATE_FIRST) ? true : false;
        $repetition = isset($this->repetition) ? intval($this->repetition) : 0;
        
        $data = array();
        
        foreach ($work_packages_list as $work_package_item) {
            $custom_costs = null;
            if (isset($this->fee_allocation_vars['project_offers_custom_costs_fee_view_'.$work_package_item->id])) {
                $custom_costs = $this->fee_allocation_vars['project_offers_custom_costs_fee_view_'.$work_package_item->id];
            }
            $costs_percent = 0;
            if (isset($this->fee_allocation_vars['project_offers_percent_fee_'.$work_package_item->id])) {
                $costs_percent = $this->fee_allocation_vars['project_offers_percent_fee_'.$work_package_item->id];
            }
            $costs_overall = null;
            if (isset($this->fee_allocation_vars['project_offers_costs_fee_'.$work_package_item->id])) {
                $costs_overall = $this->fee_allocation_vars['project_offers_costs_fee_'.$work_package_item->id];
            }
            $costs_view = 0;
            if (isset($this->fee_allocation_vars['project_offers_costs_fee_view_'.$work_package_item->id])) {
                $costs_view = $this->fee_allocation_vars['project_offers_costs_fee_view_'.$work_package_item->id];
            }
            $supplement = 0;
            if (isset($this->fee_allocation_vars['project_offers_supplement_'.$work_package_item->id])) {
                $supplement = $this->fee_allocation_vars['project_offers_supplement_'.$work_package_item->id];
            }
            $extras_percentage = 0;
            $extras_value = 0;
            if ($extras_is_percentage) {
                if (isset($this->fee_allocation_vars['project_offers_extras_'.$work_package_item->id])) {
                    $extras_percentage = $this->fee_allocation_vars['project_offers_extras_'.$work_package_item->id];
                }
            } else {
                if (isset($this->fee_allocation_vars['project_offers_extras_value_'.$work_package_item->id])) {
                    $extras_value = $this->fee_allocation_vars['project_offers_extras_value_'.$work_package_item->id];
                }
            }
            $rebate_percentage = 0;
            $rebate_value = 0;
            if ($rebate_is_percentage) {
                if (isset($this->fee_allocation_vars['project_offers_rebate_'.$work_package_item->id])) {
                    $rebate_percentage = $this->fee_allocation_vars['project_offers_rebate_'.$work_package_item->id];
                }
            } else {
                if (isset($this->fee_allocation_vars['project_offers_rebate_value_'.$work_package_item->id])) {
                    $rebate_value = $this->fee_allocation_vars['project_offers_rebate_value_'.$work_package_item->id];
                }
            }
            $calc_rep = false;
            if (isset($this->fee_allocation_vars['project_offers_calc_rep_'.$work_package_item->id])) {
                $calc_rep = $this->fee_allocation_vars['project_offers_calc_rep_'.$work_package_item->id];
            }
            $calc_rep_wp = 0;
            if (isset($this->fee_allocation_vars['project_offers_calc_rep_wp_'.$work_package_item->id])) {
                $calc_rep_wp = $this->fee_allocation_vars['project_offers_calc_rep_wp_'.$work_package_item->id];
            }
            $costs_rep = 0;
            if (isset($this->fee_allocation_vars['project_offers_costs_rep_'.$work_package_item->id])) {
                $costs_rep = $this->fee_allocation_vars['project_offers_costs_rep_'.$work_package_item->id];
            }
            
            if (!isset($costs_overall)) {
                $costs_plus_supplement = $costs_view+$supplement;
                if($calc_rebate_first) {
                    $rebate_val = round(($rebate_percentage/100)*$costs_plus_supplement, 2);
                    $costs_minus_rebate = $costs_plus_supplement-$rebate_val-$rebate_value;
                    $extras_val = round(($extras_percentage/100)*$costs_minus_rebate, 2);
                    $costs_overall = $costs_minus_rebate+$extras_val+$extras_value;
                } else {
                    $extras_val = round(($extras_percentage/100)*$costs_plus_supplement, 2);
                    $costs_plus_extras = $costs_plus_supplement+$extras_val+$extras_value;
                    $rebate_val = round(($rebate_percentage/100)*$costs_plus_extras, 2);
                    $costs_overall = $costs_plus_extras-$rebate_val-$rebate_value;
                }
            }
            
            if (!isset($this->currency_id)) {
                $data[$work_package_item->id] = array(
                    'custom_costs'          => $custom_costs,
                    'costs_percent'         => $costs_percent,
                    'costs_view'            => $costs_view,
                    'costs'                 => $costs_overall,
                    'supplement'            => $supplement,
                    'extras'                => $extras_is_percentage ? $extras_percentage : $extras_value,
                    'extras_is_percentage'  => $extras_is_percentage,
                    'rebate'                => $rebate_is_percentage ? $rebate_percentage : $rebate_value,
                    'rebate_is_percentage'  => $rebate_is_percentage,
                    'repetition'            => $costs_rep,
                    'calc_rep'              => $calc_rep,
                    'calc_rep_wp'           => $calc_rep_wp,
                    'calc_rebate_first'     => $calc_rebate_first,
                    'is_costs_sum'          => $this->fee_allocation_vars['is_costs_sum'],
                    'percent_sum'           => $this->fee_allocation_vars['percent_sum'],
                    'wp_lft'                => $work_package_item->lft,
                    'wp_rgt'                => $work_package_item->rgt,
                    'is_fee_area'           => $work_package_item->is_fee_area,
                );
            } else {
                $data[$work_package_item->id] = array(
                    'custom_costs'          => $custom_costs*usnumber_to_float($this->exchange_rate),
                    'custom_costs_otcurr'   => $custom_costs,
                    'costs_percent'         => $costs_percent,
                    'costs_view'            => $costs_view*usnumber_to_float($this->exchange_rate),
                    'costs'                 => $costs_overall*usnumber_to_float($this->exchange_rate),
                    'costs_otcurr'          => $costs_view,
                    'supplement'            => $supplement*usnumber_to_float($this->exchange_rate),
                    'supplement_otcurr'     => $supplement,
                    'extras'                => $extras_is_percentage ? $extras_percentage : $extras_value*usnumber_to_float($this->exchange_rate),
                    'extras_otcurr'         => $extras_is_percentage ? $extras_percentage : $extras_value,
                    'extras_is_percentage'  => $extras_is_percentage,
                    'rebate'                => $rebate_is_percentage ? $rebate_percentage : $rebate_value*usnumber_to_float($this->exchange_rate),
                    'rebate_otcurr'         => $rebate_is_percentage ? $rebate_percentage : $rebate_value,
                    'rebate_is_percentage'  => $rebate_is_percentage,
                    'repetition'            => $costs_rep,
                    'calc_rep'              => $calc_rep,
                    'calc_rep_wp'           => $calc_rep_wp,
                    'calc_rebate_first'     => $calc_rebate_first,
                    'is_costs_sum'          => $this->fee_allocation_vars['is_costs_sum'],
                    'percent_sum'           => $this->fee_allocation_vars['percent_sum'],
                    'wp_lft'                => $work_package_item->lft,
                    'wp_rgt'                => $work_package_item->rgt,
                    'is_fee_area'           => $work_package_item->is_fee_area,
                );
            }
        }
        
        $costs_sum = 0;
        $costs_sum_otcurr = 0;
        foreach ($data as $data_key=>$data_item) {
            if (!$data_item['is_fee_area'] && $data_item['wp_rgt']-$data_item['wp_lft']!==1) {
                $costs_sub_view = 0;
                $costs_sub = 0;
                $costs_sub_otcurr = 0;
                foreach ($data as $sub_data_item) {
                    if (!$sub_data_item['is_fee_area']) {
                        continue;
                    }
                    if ($sub_data_item['wp_lft']<$data_item['wp_lft'] || $sub_data_item['wp_rgt']>$data_item['wp_rgt']) {
                        continue;
                    }
                    $costs_sub_view += $sub_data_item['costs_view'];
                    $costs_sub += $sub_data_item['costs'];
                    if (isset($sub_data_item['costs_otcurr'])) {
                        $costs_sub_otcurr += $sub_data_item['costs_otcurr'];
                    }
                }
                $data[$data_key]['costs_view'] = $costs_sub_view;
                $data[$data_key]['costs'] = $costs_sub;
                if (isset($this->currency_id)) {
                    $data[$data_key]['costs_otcurr'] = $costs_sub_otcurr;
                }
            } else {
                $costs_sum += $data_item['costs_view'];
                if (isset($data_item['costs_otcurr'])) {
                    $costs_sum_otcurr += $data_item['costs_otcurr'];
                }
            }
        }
        
        foreach ($data as $data_key=>$data_item) {
            $data[$data_key]['costs_sum'] = $costs_sum;
            if (isset($this->currency_id)) {
                $data[$data_key]['costs_sum_otcurr'] = $costs_sum_otcurr;
            }
            unset($data[$data_key]['wp_lft']);
            unset($data[$data_key]['wp_rgt']);
            unset($data[$data_key]['is_fee_area']);
        }
        
        return $data;
    }

    /**
     * Validate and ensure all values are appropriate
     *
     * @return  mixed   false on success, else the error message
     */
    public function validate()
    {
        if ($this->add_or_id!=='add' && !is_numeric($this->add_or_id)) {
            return 'Project offer ID has to be "add" or an ID';
        }
        
        if (!empty($this->portlet_data)) {
            $portlet_data = @json_decode($this->portlet_data);
            if (isset($portlet_data->sums)) {
                return $this->CI->lang->line('portlet_update_hoai').'<br /><br />Cannot add offer with pre X3 HOAI Butler.';
            }
        }
        
        $extras_is_percentage =  (intval($this->extras_type) === self::EXTRAS_TYPE_PCTG) ? true : false;
        $rebate_is_percentage =  (intval($this->rebate_type) === self::REBATE_TYPE_PCTG) ? true : false;
        $costs_lang = $this->CI->lang->line('projectadmin_costs');
        $extras_lang = $this->CI->lang->line('offers_management_extras');
        $rebate_lang = $this->CI->lang->line('offers_management_rebate');
        foreach ($this->fee_allocation_vars as $fee_allocation_name => $fee_allocation_var) {
            if (stripos($fee_allocation_name, 'project_offers_costs_fee_')!==false) {
                if ($fee_allocation_var<0) {
                    $cost_msg = $this->_create_costs_fee_message($this->fee_allocation_vars, $fee_allocation_name, $fee_allocation_var, $extras_is_percentage, $rebate_is_percentage);
                    return $this->CI->lang->linef('is_natural', $costs_lang).$cost_msg.$this->_create_value_message($fee_allocation_name, $fee_allocation_var);
                }
            }
            if (stripos($fee_allocation_name, 'project_offers_extras_')!==false) {
                if ($fee_allocation_var<0) {
                    return $this->CI->lang->linef('is_natural', $extras_lang).$this->_create_value_message($fee_allocation_name, $fee_allocation_var);
                }
                if ($extras_is_percentage && $fee_allocation_var>100) {
                    return $this->CI->lang->linef('less_than', $extras_lang, 100).$this->_create_value_message($fee_allocation_name, $fee_allocation_var);
                }
            }
            if (stripos($fee_allocation_name, 'project_offers_rebate_')!==false) {
                if ($fee_allocation_var<0) {
                    return $this->CI->lang->linef('is_natural', $rebate_lang).$this->_create_value_message($fee_allocation_name, $fee_allocation_var);
                }
                if ($rebate_is_percentage && $fee_allocation_var>100) {
                    return $this->CI->lang->linef('less_than', $rebate_lang, 100).$this->_create_value_message($fee_allocation_name, $fee_allocation_var);
                }
            }
            if (stripos($fee_allocation_name, 'project_offers_rebate_value_')!==false) {
                if ($fee_allocation_var<0) {
                    return $this->CI->lang->linef('is_natural', $rebate_lang).$this->_create_value_message($fee_allocation_name, $fee_allocation_var);
                }
            }
        }
        
        return false;
    }
    
    /**
     * Create message validate message
     * 
     * @param   string  $fee_allocation_name        Field name and key
     * @param   float   $fee_allocation_var         Field value
     * @return  string
     */
    private function _create_value_message(string $fee_allocation_name, float $fee_allocation_var): string
    {
        return '<br /><br /><em>Cannot add for '.$fee_allocation_name.' = '.$fee_allocation_var.'</em><br /><br />';
    }
    
    /**
     * Create wrong costs validate message
     * 
     * @param   array   $fee_allocation_vars        The full fees array
     * @param   string  $fee_allocation_name        Field name and key
     * @param   float   $fee_allocation_var         Field value
     * @param   bool    $extras_is_percentage       Extras is percentage, not value
     * @param   bool    $rebate_is_percentage       Rebate is percentage, not value
     * @return  string
     */
    private function _create_costs_fee_message(array $fee_allocation_vars, string $fee_allocation_name, float $fee_allocation_var, bool $extras_is_percentage, bool $rebate_is_percentage): string
    {
        $fee_allocation_key = intval(str_replace('project_offers_costs_fee_', '', $fee_allocation_name));
        if ($fee_allocation_key<1) {
            return 'No project_offers_costs_fee ID';
        }
        $msg = '<br /><kbd>';
        if (isset($fee_allocation_vars['project_offers_costs_fee_view_'.$fee_allocation_key])) {
            $msg .= float_to_usnumber($fee_allocation_vars['project_offers_costs_fee_view_'.$fee_allocation_key], 2);
        }
        if ($extras_is_percentage && isset($fee_allocation_vars['project_offers_extras_'.$fee_allocation_key])) {
            $msg .= ' + ' . float_to_usnumber($fee_allocation_vars['project_offers_extras_'.$fee_allocation_key], 2).'%';
        } elseif (isset($fee_allocation_vars['project_offers_extras_value_'.$fee_allocation_key])) {
            $msg .= ' + ' . float_to_usnumber($fee_allocation_vars['project_offers_extras_value_'.$fee_allocation_key], 2);
        }
        if ($rebate_is_percentage && isset($fee_allocation_vars['project_offers_rebate_'.$fee_allocation_key])) {
            $msg .= ' - ' . float_to_usnumber($fee_allocation_vars['project_offers_rebate_'.$fee_allocation_key], 2).'%';
        } elseif (isset($fee_allocation_vars['project_offers_rebate_value_'.$fee_allocation_key])) {
            $msg .= ' - ' . float_to_usnumber($fee_allocation_vars['project_offers_rebate_value_'.$fee_allocation_key], 2);
        }
        $msg .= ' = ' . float_to_usnumber($fee_allocation_var, 2).'</kbd>';
        return $msg;
    }
    
    /**
     * Get the id of the project we save the offer for
     *
     * @return  integer
     */    
    public function get_project_id()
    {
        return intval($this->project_id);
    }
} // END class Project_offers_values
