<?php
/**
 * CodeIgniter library
 *
 * @category CodeIgniter
 * @license /_licenses/untermStrich/agb_normal.pdf
 * @author Christian Koller
 * @copyright 2009 untermStrich
 * @package us_libraries
 * @version $Id$
*/

if (!defined('BASEPATH')) exit('No direct script access allowed');

require_once APPPATH.'us_libraries/Value_object.php';

/**
 * Used to save a vacation
 *
 * @property    My_Controller           $CI                     The main CI object
 */
class Vacation_values extends Value_object
{
    protected $add_or_id;
    protected $start_date;
    protected $end_date;
    protected $days;
    protected $comment;
    protected $type;
    protected $absent_social;
    protected $status;
    protected $status_pre;

    protected $staff;
    protected $project;
    
    protected $force_increased;

    /**
     * Init the vacation values from $_POST
     *
     * @return  object  An Vacation_values object with all values out of the post array
     */
    public static function init_from_post()
    {
        $obj = new Vacation_values();
        $obj->CI = CI_Controller::get_instance();

        $obj->add_or_id = $obj->CI->input->post('holidayadmin_edit');
        $obj->start_date = new UsDate($obj->CI->input->post('holidayadmin_date__from'), null, true, null);
        $obj->end_date = new UsDate($obj->CI->input->post('holidayadmin_date__to'), null, true, null);
        $obj->days = usnumber_to_float($obj->CI->input->post('holidayadmin_days'));
        $obj->comment = $obj->CI->input->post('holidayadmin_comment');
        $obj->project = $obj->CI->input->post('holidayadmin_project');
        $obj->type = $obj->CI->input->post('holidayadmin_type');
        $obj->absent_social = $obj->CI->input->post('holidayadmin_absent_social') ? true : false;
        $obj->status = $obj->CI->input->post('holidayadmin_status');
        $obj->status_pre = $obj->CI->input->post('holidayadmin_status_pre');
        $obj->staff = $obj->CI->input->post('holidayadmin_staff');
        
        $obj->force_increased = false;
        
        return $obj;
    }
    
    /**
     * Init the vacation values from a given StdClass object
     *
     * @param   object  $values StdClass object with vacation values to initialize a valid Vacation_values object out of it
     *
     * @return  object  An Vacation_values object with all values out of the given StdClass object
     */
    public static function init_from_values(StdClass $values) {

        $obj = new self();
        $obj->CI = CI_Controller::get_instance();

        $obj->add_or_id = $values->id;
        $obj->start_date = $values->date_from;
        $obj->end_date = $values->date_to;
        $obj->days = floatval($values->days);
        $obj->comment = isset($values->comment) ? $values->comment : '';
        $obj->project = intval($values->project);
        $obj->type = $values->type;
        $obj->absent_social = $values->absent_social ? true : false;
        $obj->status = $values->status;
        $obj->status_pre = isset($values->status_pre) ? $values->status_pre : false;
        $obj->staff = intval($values->staff);
        
        $obj->force_increased = false;

        return $obj;
    }
    
    /**
     * Init the increased vacation values from $_POST
     *
     * @return  object  An Vacation_values object with all values out of the post array
     */
    public static function init_new_increased_from_post()
    {
        $obj = new Vacation_values();
        $obj->CI = CI_Controller::get_instance();

        $obj->add_or_id = 'add';
        $obj->staff = $obj->CI->input->post('holidayadmin_increase_staff');
        $obj->start_date = new UsDate($obj->CI->input->post('holidayadmin_increase_date'), null, true, null);
        $obj->end_date = $obj->start_date;
        $obj->days = usnumber_to_float($obj->CI->input->post('holidayadmin_increase_days'));
        $obj->comment = $obj->CI->input->post('holidayadmin_increase_comment');
        $obj->project = $obj->CI->input->post('holidayadmin_increase_project');
        $obj->type = Vacations_model::TYPE_HOLIDAY;
        $obj->absent_social = false;
        $obj->status = 'ignore';
        $obj->status_pre = false;
        
        $obj->force_increased = true;
        
        return $obj;
    }

    /**
     * Init the annual closing vacation values from $_POST
     *
     * @param   integer     $staff_id       Staff ID
     * @return  object      An Vacation_values object with all values out of the post array
     */
    public static function init_new_annual_closing_from_post($staff_id)
    {
        $obj = new Vacation_values();
        $obj->CI = CI_Controller::get_instance();

        $obj->add_or_id = 'add';
        $obj->staff = $staff_id;
        $obj->start_date = new UsDate($obj->CI->input->post('holidayadmin_annual_closing_date__from'), null, true, null);
        $obj->end_date = new UsDate($obj->CI->input->post('holidayadmin_annual_closing_date__to'), null, true, null);
        $obj->comment = $obj->CI->input->post('holidayadmin_annual_closing_comment');
        $obj->project = $obj->CI->input->post('holidayadmin_annual_closing_project');
        $obj->type = Vacations_model::TYPE_HOLIDAY;
        $obj->absent_social = false;
        $obj->status = Vacations_model::STATUS_APPROVED;
        $obj->status_pre = false;
        
        $params = array(
            'staff_id'                => $staff_id,
            'start_date'              => $obj->start_date,
            'end_date'                => $obj->end_date,
        );
        $obj->CI->load->varmodel('Working_period_model', $params, 'working_period');
        $obj->CI->working_period = new Working_period_model($params);

        $obj->days = $obj->CI->working_period->get_working_days_for_period();
        
        $obj->force_increased = false;
        
        return $obj;
    }

    /**
     * Validate and ensure all values are appropriate
     *
     * @return  mixed   false on success, else the error message
     */
    public function validate()
    {
        if ($this->add_or_id!=='add' && !is_numeric($this->add_or_id)) {
            return 'Vacation ID has to be "add" or an ID';
        }

        if (!$this->CI->vacations->user_can_add($this->staff)) {
            return 'User can not add vacation entries for this member of staff (VAL)';
        }

        if (!$this->CI->vacations->user_can_approve()) {
            $this->status = false;
        }

        return false;
    }

    /**
     * Get vacation values to save
     *
     * @return  array
     */
    public function get_vacation_to_save()
    {
        $holiday_ok = 0;
        if ($this->type==Vacations_model::TYPE_HOLIDAY) {
            $holiday_ok = $this->status==Vacations_model::STATUS_APPROVED ? 1 : 0;
        }
        
        $data = array(
            'id'                =>  intval($this->add_or_id),
            'project_id'        =>  intval($this->project),
            'start_date'        =>  $this->start_date->mysql(),
            'end_date'          =>  $this->end_date->mysql(),
            'comment'           =>  $this->comment,
            'holiday_is_ok'     =>  $holiday_ok,
            'is_ok_pre'         =>  $this->status_pre==Vacations_model::STATUS_APPROVED_PRE ? 1 : 0,
            'is_submitted'      =>  $this->status===false ? 1 : 0,
            'is_refused'        =>  $this->status==Vacations_model::STATUS_REFUSED ? 1 : 0,
            'is_absent'         =>  $this->type==Vacations_model::TYPE_ABSENT ? 1 : 0,
            'is_absent_social'  => (boolean)$this->absent_social,
        );
        
        if ($this->isNew()) {
            if ($this->force_increased) {
                $data['days'] = floatval($this->days);
                $data['increased'] = 1;
            } else {
                $data['days'] = -abs($this->days);
                $data['increased'] = 0;
            }
            $data['staff_id'] = intval($this->staff);
            
        } else {
            $orig_item = $this->CI->vacations->get_item($this->add_or_id);
            
            if ($orig_item===null) {
                basic_error('Save - missing existing item', null, __CLASS__.'::'.__FUNCTION__);
            }
            
            if ($orig_item->increased) {
                unset($data['holiday_is_ok']);
                unset($data['is_refused']);
                unset($data['is_absent']);
                
                $data['days'] = abs($this->days);
            } else {
                $data['days'] = -abs($this->days);
            }
        }

        return $data;
    }
    
    /**
     * Has the entry changed?
     *
     * @param   object  $old    The old entry
     * @return  boolean 
     */
    public function has_changed(stdClass $old)
    {
        $is_absent = $this->type==Vacations_model::TYPE_ABSENT ? 1 : 0;
        if ($old->is_absent!=$is_absent) {
            return true;
        }
        
        if ($old->start_date->mysqlday()!=$this->start_date->mysqlday()) {
            return true;
        }
        
        if ($old->end_date->mysqlday()!=$this->end_date->mysqlday()) {
            return true;
        }
        
        if ($old->days*-1!=$this->days) {
            return true;
        }
        
        if ($old->project_id!=$this->project) {
            return true;
        }
        
        return false;
    }
    
    /**
     * Get start date as ISO Date
     * 
     * @return  string
     */
    function get_start_date()
    {
        return $this->start_date->mysql();
    }

    /**
     * Get end date as ISO Date
     * 
     * @return  string
     */
    function get_end_date()
    {
        return $this->end_date->mysql();
    }

    /**
     * Get staff id
     * 
     * @return  integer
     */
    function get_staff()
    {
        return intval($this->staff);
    }


    
} // END class Vacation_values
