<?php

/**
 * CodeIgniter library
 *
 * @category CodeIgniter
 * @license /_licenses/untermStrich/agb_normal.pdf
 * @author Michael Pfeilstöcker
 * @copyright 2009 untermStrich
 * @package us_libraries
 * @version $Id: whatiswhere_values.php 23723 2022-04-08 05:26:06Z christian $
 */
if (!defined('BASEPATH'))
    exit('No direct script access allowed');

require_once APPPATH . 'us_libraries/Value_object_addi.php';

/**
 * Used to save a whatiswhere item
 *
 * @property    My_Controller           $CI                     The main CI object
 */
class Whatiswhere_values extends Value_object_addi {

    protected $article_fulltitle;
    protected $title;
    protected $author;
    protected $on_page;
    protected $themes = array();
    protected $note;
    protected $file;
    protected $address;
    protected $project_names;
    protected $projects = array();

    /**
     * Init the whatiswhere values from $_POST
     *
     * @return  object  An Whatiswhere_values object with all values out of the post array
     */
    public static function init_from_post() {
        $obj = new Whatiswhere_values();
        $obj->CI = CI_Controller::get_instance();
        $obj->add_or_id = $obj->CI->input->post('whatiswhere_edit');
        $obj->article_fulltitle = intval($obj->CI->input->post('whatiswhere_article_fulltitle'));
        $obj->title = $obj->CI->input->post('whatiswhere_title');
        $obj->author = $obj->CI->input->post('whatiswhere_author');
        $obj->page = $obj->CI->input->post('whatiswhere_on_page');
        $obj->themes = $obj->CI->input->post('whatiswhere_themes');
        $obj->note = $obj->CI->input->post('whatiswhere_note');
        $obj->file = $obj->CI->input->post('whatiswhere_file');
        $obj->address_id = intval($obj->CI->input->post('whatiswhere_address'));
        $obj->projects = $obj->CI->input->post('whatiswhere_project_names');
        $obj->addi_vars = $obj->CI->whatiswhere->get_addi_vars_post();
        return $obj;
    }

    /**
     * Validate and ensure all values are appropriate
     *
     * @return  mixed   false on success, else the error message
     *
     * @todo    implement!!
     */
    public function validate() {
        if ($this->add_or_id !== 'add' && !is_numeric($this->add_or_id)) {
            return 'Whatiswnere ID has to be "add" or an ID';
        }

        return false;
    }

    /**
     * Get task to save
     *
     * @return  array
     */
    public function get_items_to_save() {
        $data = array(
            'title' => $this->title,
            'article_id' => intval($this->article_fulltitle),
            'author' => $this->author,
            'page' => $this->page,
            'note' => $this->note,
            'file' => $this->file,
            'address_id' => intval($this->address_id),
        );

        if ($this->file !== false && !empty($this->file)) {
            if ($this->file == '#DELETEME#') {
                $data['file'] = null;
            } else {
                $data['file'] = $this->file;
            }
        }

        return $data;
    }

    /**
     * Get themes to save
     *
     * @return  array
     */
    public function get_themes_to_save() {
        $data = array();
        if (!empty($this->themes)) {
            foreach ($this->themes as $theme) {
                array_push($data, $theme);
            }
        }
        return $data;
    }

    /**
     * Get themes to save
     *
     * @return  array
     */
    public function get_projects_to_save() {
        $data = array();
        if (!empty($this->projects)) {
            foreach ($this->projects as $project) {
                array_push($data, $project);
            }
        }
        return $data;
    }

}

