<?php
/*
* Copyright 2007 ZXing authors
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*      http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

namespace Zxing\Common;

use Zxing\NotFoundException;

/**
 * @author Sean Owen
 */
final class DefaultGridSampler extends GridSampler
{
	public function sampleGrid(
		$image,
		$dimensionX,
		$dimensionY,
		$p1ToX,
		$p1ToY,
		$p2ToX,
		$p2ToY,
		$p3ToX,
		$p3ToY,
		$p4ToX,
		$p4ToY,
		$p1FromX,
		$p1FromY,
		$p2FromX,
		$p2FromY,
		$p3FromX,
		$p3FromY,
		$p4FromX,
		$p4FromY
	) {
		$transform = PerspectiveTransform::quadrilateralToQuadrilateral(
			$p1ToX,
			$p1ToY,
			$p2ToX,
			$p2ToY,
			$p3ToX,
			$p3ToY,
			$p4ToX,
			$p4ToY,
			$p1FromX,
			$p1FromY,
			$p2FromX,
			$p2FromY,
			$p3FromX,
			$p3FromY,
			$p4FromX,
			$p4FromY
		);

		return $this->sampleGrid_($image, $dimensionX, $dimensionY, $transform);
	}

	public function sampleGrid_(
		$image,
		$dimensionX,
		$dimensionY,
		$transform
	) {
		if ($dimensionX <= 0 || $dimensionY <= 0) {
			throw NotFoundException::getNotFoundInstance();
		}
		$bits = new BitMatrix($dimensionX, $dimensionY);
		$points = fill_array(0, 2 * $dimensionX, 0.0);
		for ($y = 0; $y < $dimensionY; $y++) {
			$max = is_countable($points) ? count($points) : 0;
			$iValue = (float)$y + 0.5;
			for ($x = 0; $x < $max; $x += 2) {
				$points[$x] = (float)($x / 2) + 0.5;
				$points[$x + 1] = $iValue;
			}
			$transform->transformPoints($points);
			self::checkAndNudgePoints($image, $points);
			try {
				for ($x = 0; $x < $max; $x += 2) {
					if ($image->get((int)$points[$x], (int)$points[$x + 1])) {
						$bits->set($x / 2, $y);
					}
				}
			} catch (\Exception) {//ArrayIndexOutOfBoundsException
				throw NotFoundException::getNotFoundInstance();
			}
		}

		return $bits;
	}
}
