<?php

namespace PhpOffice\PhpSpreadsheet\Style\NumberFormat;

use PhpOffice\PhpSpreadsheet\Shared\Date;

class DateFormatter
{
    /**
     * Search/replace values to convert Excel date/time format masks to PHP format masks.
     *
     * @var array
     */
    private static $dateFormatReplacements = [
        '\\' => '',
        'am/pm' => 'A',
        'e' => 'Y',
        'yyyy' => 'Y',
        'yy' => 'y',
        'mmmmm' => 'M',
        'mmmm' => 'F',
        'mmm' => 'M',
        ':mm' => ':i',
        'mm:' => 'i:',
        'mm' => 'm',
        'm' => 'n',
        'dddd' => 'l',
        'ddd' => 'D',
        'dd' => 'd',
        'd' => 'j',
        'ss' => 's',
        '.s' => '',
    ];

    /**
     * Search/replace values to convert Excel date/time format masks hours to PHP format masks (24 hr clock).
     *
     * @var array
     */
    private static $dateFormatReplacements24 = [
        'hh' => 'H',
        'h' => 'G',
    ];

    /**
     * Search/replace values to convert Excel date/time format masks hours to PHP format masks (12 hr clock).
     *
     * @var array
     */
    private static $dateFormatReplacements12 = [
        'hh' => 'h',
        'h' => 'g',
    ];

    public static function format($value, string $format): string
    {
        $format = preg_replace('/^(\[DBNum\d\])*(\[\$[^\]]*\])/i', '', $format);

        $format = preg_replace_callback('/(?:^|")([^"]*)(?:$|")/', ['self', 'setLowercaseCallback'], $format);

        $blocks = explode('"', $format);
        foreach ($blocks as $key => &$block) {
            if ($key % 2 == 0) {
                $block = strtr($block, self::$dateFormatReplacements);
                if (!strpos($block, 'A')) {
                    if (false !== strpos($block, '[h]')) {
                        $hours = (int) ($value * 24);
                        $block = str_replace('[h]', $hours, $block);

                        continue;
                    }
                    $block = strtr($block, self::$dateFormatReplacements24);
                } else {
                    $block = strtr($block, self::$dateFormatReplacements12);
                }
            }
        }
        $format = implode('"', $blocks);

        $format = preg_replace_callback('/"(.*)"/U', ['self', 'escapeQuotesCallback'], $format);

        $dateObj = Date::excelToDateTimeObject($value);
        $format = \preg_replace('/\\\\:m/', ':i', $format);

        return $dateObj->format($format);
    }

    private static function setLowercaseCallback($matches): string
    {
        return mb_strtolower($matches[0]);
    }

    private static function escapeQuotesCallback($matches): string
    {
        return '\\' . implode('\\', str_split($matches[1]));
    }
}
