<?php
/**
 * CodeIgniter library
 *
 * @category CodeIgniter
 * @license /_licenses/untermStrich/agb_normal.pdf
 * @author Christian Koller
 * @copyright 2009 untermStrich
 * @package views
 * @version $Id: Builtin.php 22685 2021-06-18 06:52:57Z christian $
*/

if (!defined('BASEPATH')) exit('No direct script access allowed');

/**
 * PHP built-in pdf processer
 */
class Pdf_builtin
{
    /**
     * The main CI object
     */
    protected $CI;

    /**
     * Constructor
     */
    public function __construct()
    {
        $this->CI = CI_Controller::get_instance();
    }
    
    /**
     * Is this process useable
     *
     * @param   integer $mode       One of the My_controller::m_... modes
     * @param   integer $file_size  File size
     * @return  boolean
     */
    public function is_useable($mode, $file_size)
    {
        return true; //I'm the fallback - so always use me :-)
    }

    /**
     * Process the pdf
     *
     * @param   integer             $mode           One of the US_report_view_result::M_... modes
     * @param   SimpleXMLElement    $viewinfo       The view info
     * @param   string              $file_name_html File path to the report html
     * @param   SimpleXMLElement    $rephead        The head of the report XML
     * @param   array               $designinfo     The design settings
     * @return  void
     *
     * @see US_report_view_result::M_DIRECT
     * @see US_report_view_result::M_FILE
     */
    public function process(
        $mode,
        SimpleXMLElement $viewinfo,
        $file_name_html,
        SimpleXMLElement $rephead,
        array $designinfo
    ) {
        
        Report::set_header($viewinfo);
        
        $this->CI->load->helper('report/report_pdf');
        
        $format = (string)$rephead->title['format'];
        $orientation = (string)$rephead->title['orientation'];
        
        Report_pdf_helper::create($mode, $format, $orientation, $file_name_html);
    }

} // END class Pdf_builtin
