{*
 *
 * Creates a table list with editable links.
 *
 * @param   tabname          string     ID of the table and name of the corrosponding javascript action object
 *                                      The javascript object has to provide the following methodes:
 *                                      <ul>
 *                                          <li>open            Opens the edit popup - parameter:
 *                                                              <ul>
 *                                                                 <li>mixed   Either 'add' or the item ID
 *                                                                 <li>bool    Are the values NOT editable
 *                                                                 <li>string  The message to show if it is not editable
 *                                                              </ul>
 *                                          </li>
 *                                          <li>delete_entry    Delete entry - parameter:
 *                                                              <ul>
 *                                                                 <li>int     The item ID to delete
 *                                                              </ul>
 *                                          <li>order_by        Only active if the "format" parameter is an array holding the order by name - parameter:
 *                                                              <ul>
 *                                                                 <li>string  The name of the field in the database to order by
 *                                                              </ul>
 *                                          </li>
 *                                          <li>hover_update    Only active if the "hover_script" parameter is set to TRUE - parameter:
 *                                                              <ul>
 *                                                                 <li>int     The item ID
 *                                                              </ul>
 *                                          </li>
 *                                          <li>hover_stop      Only active if the "hover_script" parameter is set to TRUE.
 *                                                              Stops the hover script
 *                                          </li>
 *                                      </ul>
 * @param   header              array   An assoc array holding the header infos.
 *                                      The key holds the name (key) of the translation
 *                                      Each item is one of:
 *                                          Either an array:
 *                                          <ul>
 *                                              <li>width       string  The width of the table column for CSS
 *                                              <li>stylehead   string  Additional CSS properties
 *                                              <li>sort        string  The name of the field in the database to order by (see order_by above)
 *                                          </ul>
 *                                          Or it holds a string with the formating CSS
 * @param   whatview            array   An assoc array holding the row infos.
 *                                      The key holds the field name in the "list" item object
 *                                      Each item is one of:
 *                                          Either an array:
 *                                          <ul>
 *                                              <li>0   string  Format - Same as below at string only
 *                                              <li>1   string  A string with the formating CSS for each cell
 *                                          </ul>
 *                                          Or it holds a string with field formating infos.
 *                                          It can be either "bool", "isnew", "urgent" for boolean values or it holds the truncate value.
 * @param   order_by_def        mixed   Optional The default value for order by
 * @param   order_by_dir_def    mixed   Optional The default value for order by direction
 * @param   list                array   The values array. It holds the following objects:
 *                                      <ul>
 *                                          <li>rowinfo                 object  Optional Formatting infos:
 *                                                                              <ul>
 *                                                                                  <li>title   string  Title for the row
 *                                                                                  <li>class   string  A CSS class for the row
 *                                                                              </ul>
 *                                          </li>
 *                                          <li>object_is_editable      bool    Is the object editable
 *                                          </li>
 *                                          <li>object_is_editable_msg  string  Optional The message to show when the object is not editable
 *                                          </li>
 *                                          <li>id                      mixed   The item ID
 *                                          </li>
 *                                          <li>...                     mixed   Any type of fields (string, object) defined via whatview
 *                                          </li>
 *                                      </ul>
 * @param   no_rows_msg         string  The message to show, if there are no results (One of $i.basic_no_result1, $i.basic_no_result2)
 *}
{foreach key="item" item="format" from=$header}{strip}
    {if {$format.lang}}
        "{$i.{$format.lang}|forcsv}"
    {else}
        "{$format.title|forcsv}"
    {/if}{chr(9)}
{/strip}{/foreach}
{foreach item="item" name="index" from=$list}{"\n"}{strip}
    {foreach key="whatitem" item="whatformatstyle" from=$whatview}
        {* The current value *}
        {assign var="current" value=$item->$whatitem}
        {* Get data format and CSS style *}
        {assign var="is_addi" value=FALSE}
        {if is_array($whatformatstyle)}
            {assign var="whatformat" value=$whatformatstyle[0]}
            {assign var="whatstyle" value=$whatformatstyle[1]}
            {if isset($whatformatstyle[2])}{assign var="is_addi" value=$whatformatstyle[2]}{/if}
        {else}
            {assign var="whatformat" value=$whatformatstyle}
            {assign var="whatstyle" value=''}
        {/if}
        {* Add the content *}
        {if $is_addi}
            {addiview_csv field=$whatformat value=$current item=$item}
        {else}
            {if is_object($current) && $current instanceof UsDateTime}
                {$current->us_date_time()|forcsv}
            {elseif is_object($current) && $current instanceof UsDate}
                {$current->us_date()|forcsv}
            {elseif is_object($current) && $current instanceof UsTime}
                {$current->us_stime()|forcsv}
            {elseif $whatformat eq 'float'}
                {isfloat value=$current no_tsd=true}
            {elseif $whatformat eq 'longfloat'}
                {islongfloat value=$current no_tsd=true}
            {elseif $whatformat eq 'turnover_rate'}
                {isturnover_rate value=$current}
            {elseif $whatformat eq 'turnover_rate_%'}
                {isturnover_rate value=$current add_percent_sign=true}
            {elseif $whatformat eq '_empty'}
                {*}empty value{*}
            {else}
                {$current|forcsv}
            {/if}
        {/if}{chr(9)}
    {/foreach}
{/strip}{foreachelse}
{$no_rows_msg}
{/foreach}