{*
 *
 * Creates a table list with editable links.
 *
 * !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 * 
 * ALWAYS RUN trim.php AFTER MODIFICATION
 *
 * ENSURE YOU HAVE A SPACE AT THE END OF AN STILL OPEN TAG
 *
 * !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 *
 * @param   tabname          string     ID of the table and name of the corrosponding javascript action object
 *                                      The javascript object has to provide the following methodes:
 *                                      <ul>
 *                                          <li>open            Opens the edit popup - parameter:
 *                                                              <ul>
 *                                                                 <li>mixed   Either 'add' or the item ID
 *                                                                 <li>bool    Are the values NOT editable
 *                                                                 <li>string  The message to show if it is not editable
 *                                                              </ul>
 *                                          </li>
 *                                          <li>delete_entry    Delete entry - parameter:
 *                                                              <ul>
 *                                                                 <li>int     The item ID to delete
 *                                                              </ul>
 *                                          <li>order_by        Only active if the "format" parameter is an array holding the order by name - parameter:
 *                                                              <ul>
 *                                                                 <li>string  The name of the field in the database to order by
 *                                                              </ul>
 *                                          </li>
 *                                          <li>hover_update    Only active if the "hover_script" parameter is set to TRUE - parameter:
 *                                                              <ul>
 *                                                                 <li>int     The item ID
 *                                                              </ul>
 *                                          </li>
 *                                          <li>hover_stop      Only active if the "hover_script" parameter is set to TRUE.
 *                                                              Stops the hover script
 *                                          </li>
 *                                      </ul>
 * @param   user_can_delete     bool    Optional Show the delete button
 * @param   user_can_add        bool    Optional Show the add button
 * @param   user_can_edit       bool    Optional Overwrite the object_is_editable setting of each item
 * @param   hover_script        bool    Optional Activate the hover over script
 * @param   group_by            mixed   Optional A field name to group by, can also be a special array for multiple groupings
 * @param   header              array   An assoc array holding the header infos.
 *                                      The key holds the name (key) of the translation
 *                                      Each item is one of:
 *                                          Either an array:
 *                                          <ul>
 *                                              <li>width       string  The width of the table column for CSS
 *                                              <li>stylehead   string  Additional CSS properties
 *                                              <li>sort        string  The name of the field in the database to order by (see order_by above)
 *                                          </ul>
 *                                          Or it holds a string with the formating CSS
 * @param   whatview            array   An assoc array holding the row infos.
 *                                      The key holds the field name in the "list" item object
 *                                      Each item is one of:
 *                                          Either an array:
 *                                          <ul>
 *                                              <li>0   string  Format - Same as below at string only
 *                                              <li>1   string  A string with the formating CSS for each cell
 *                                          </ul>
 *                                          Or it holds a string with field formating infos.
 *                                          It can be either "bool", "isnew", "urgent" for boolean values or it holds the truncate value.
 * @param   order_by_def        mixed   Optional The default value for order by
 * @param   order_by_dir_def    mixed   Optional The default value for order by direction
 * @param   list                array   The values array. It holds the following objects:
 *                                      <ul>
 *                                          <li>rowinfo                 object  Optional Formatting infos:
 *                                                                              <ul>
 *                                                                                  <li>title   string  Title for the row
 *                                                                                  <li>class   string  A CSS class for the row
 *                                                                              </ul>
 *                                          </li>
 *                                          <li>object_is_editable      bool    Is the object editable
 *                                          </li>
 *                                          <li>object_is_editable_msg  string  Optional The message to show when the object is not editable
 *                                          </li>
 *                                          <li>id                      mixed   The item ID
 *                                          </li>
 *                                          <li>...                     mixed   Any type of fields (string, object) defined via whatview
 *                                          </li>
 *                                      </ul>
 * @param   no_rows_msg         string  The message to show, if there are no results (One of $i.basic_no_result1, $i.basic_no_result2)

 * @param   tableconfig_section         string  Table config section
 * @param   tableconfig_xmlfile         string  Table config xmlfile
 * @param   tableconfig_onsaveaction    string  Table config action on save
 *}
 {if !isset($ondblclick_action)}{$ondblclick_action='open'}{/if}
<div class="editablepopdiv">
<table id="edtp-table-{$tabname}" class="editablepop {if isset($tr_single_line) && $tr_single_line}oneline{/if}">
    <colgroup>
        <col class="edt{if isset($user_can_delete) && $user_can_delete}del{/if}{if isset($btn_auto_width) && $btn_auto_width}auto{/if}p-btn noprint"/>
        {foreach key="item" item="format" from=$header}
            <col style="width:{$format.width}; {if !$format.allow_edit_width}max-width:{$format.width};{/if}" />
        {/foreach}
    </colgroup>
    <thead>
        <tr>
            <th class="edt{if isset($user_can_delete) && $user_can_delete}del{/if}p-btn noprint">
                {strip}
                {if isset($user_can_add) && $user_can_add}
                    <span onclick="{$tabname}.open('add', false, '');" 
                          class="{if isset($tableconfig_section) && isset($tableconfig_xmlfile)}dialog_make_btn16{else}dialog_make_btn24{/if} add mobile_big smallbtns ui-corner-all accesskey_add" 
                          title="{$i.basic_add}"></span>
                {/if}
                {if isset($tableconfig_section) && isset($tableconfig_xmlfile)}
                    {include file='r_basic/tableconfig_btn.tpl'}
                {/if}
                {/strip}
            </th>
            {foreach key="item" item="format" from=$header}
                {if {$format.lang}}
                    {assign var="title" value=$i.{$format.lang}}
                {else}
                    {assign var="title" value=$format.title}
                {/if}
                {if {$format.langtitle}}
                    {assign var="hovertitle" value=$i.{$format.langtitle}}
                {else}
                    {assign var="hovertitle" value=$title}
                {/if}
                <th {if isset($format.stylehead)}style="{$format.stylehead}"{/if} title="{$hovertitle}">
                    {if $format.sort}
                        <span onclick="{$tabname}.order_by('{$format.sort}', '{$reportname}'{if $portlet_id}, '{$portlet_id}'{/if});" 
                            class="order_by {if isset($order_by_def) && $order_by_def eq $format.sort}selected{$order_by_dir_def}{/if} {if {$format.langtitle}}hoverhelp{/if}" >
                            {if strstr({$format.width}, 'px') && intval(substr({$format.width}, 0, -2)) < 20}
                                <span title="{$title}" class="dialog_make_btn14 info smallbtns ui-corner-all">{$title}</span>
                            {else}
                               {$title}
                            {/if}
                        </span>
                    {else}
                    <span class="{if {$format.langtitle}}hoverhelp{/if}">
                        {if strstr({$format.width}, 'px') && intval(substr({$format.width}, 0, -2)) < 20}
                            <span title="{$title}" class="dialog_make_btn14 info smallbtns ui-corner-all">{$title}</span>
                        {else}
                           {$title}
                        {/if}                        
                    </span>
                        {/if}
                </th>
            {/foreach}
            <th style="display:none;"></th>{* Fix faulty jQuery UI selectable, which always calles the click of the last cell, if there is no onlick in a cell *}
        </tr>
    </thead>
    <tbody id="edtp-tbody-{$tabname}">
        {foreach item="item" name="index" from=$list}
            {* Get the is_editable value from either global or per item *}
            {assign var="is_editable" value=FALSE}
            {if isset($user_can_edit)}
                {assign var="is_editable" value=$user_can_edit}
            {/if}
            {if isset($item->object_is_editable) && !$item->object_is_editable}
                {assign var="is_editable" value=FALSE}
            {/if}
            
            {assign var="is_deleteable" value=FALSE}
            {if isset($user_can_delete)}
                {assign var="is_deleteable" value=$user_can_delete}
            {/if}
            {if isset($item->object_is_deleteable) && !$item->object_is_deleteable}
                {assign var="is_deleteable" value=FALSE}
            {/if}            
            {if isset($item->rowinfo) && isset($item->rowinfo->title) && !empty($item->rowinfo->title)}
               {assign var="main_title" value=" [`$item->rowinfo->title`]"}
            {else}
               {assign var="main_title" value=''}
            {/if}

            {if isset($hide_edtp_btn)}
                {assign var="hide_edtp_btn" value=$hide_edtp_btn}
            {else}
                {assign var="hide_edtp_btn" value=FALSE}
            {/if}

            {* The data row *}
            <tr class="{if $smarty.foreach.index.iteration is even}even{/if} {if isset($item->rowinfo) && isset($item->rowinfo->class)}{$item->rowinfo->class}{/if}" 
                {if isset($item->rowinfo) && isset($item->rowinfo->row_id)}id="{$item->rowinfo->row_id}" {/if}
                {if isset($item->rowinfo) && isset($item->rowinfo->style)}style="{$item->rowinfo->style}" {/if}>
                <td class="edtp-btn noprint">
                    {if !$hide_edtp_btn}
                        {if $dot3p}
                            <span class="dialog_make_btn24 dot3ps smallbtns" onclick="tableeditpop.open(this, '{$tabname}', '{$item->id}'{if $portlet_id}, '{$portlet_id}'{/if});" title="{$i.basic_edit}"></span>
                            <div class="ui-corner-all ui-widget-header dot3-container">
                                <div class="dot3-caret">
                                    <div class="dot3-caret-outer"></div>
                                    <div class="dot3-caret-inner"></div>
                                </div>
                        {/if}
                        {if $is_editable}
                            {strip}
                            {if isset($item->force_no_editable_link)}
                                <span onclick="{$tabname}.open('{$item->id}', true, this.title, '{$reportname}');" class="dialog_make_btn16 show_only mobile_big smallbtns ui-corner-all" title="{$i.basic_view}">{if $dot3p}{$i.basic_view}{/if}</span>
                            {else}
                                <span onclick="{$tabname}.open('{$item->id}', false, '');" class="dialog_make_btn16 edit mobile_big smallbtns ui-corner-all" title="{$i.basic_edit} ({$item->id})">{if $dot3p}{$i.basic_edit}{/if}</span>
                            {/if}
                            {if isset($user_can_delete) && $user_can_delete}
                                <span onclick="{$tabname}.delete_entry('{$item->id}');" class="dialog_make_btn16 delete mobile_big smallbtns ui-corner-all" title="{$i.basic_delete}">{if $dot3p}{$i.basic_delete}{/if}</span>
                            {/if}
                            {/strip}
                        {else}
                            {strip}
                            {if isset($hover_link) && $hover_link}
                                <a href="{str_replace('<#item_id#>', $item->id, $hover_link)}" onclick="{$tabname}.open('{$item->id}', true, this.title, '{$reportname}', this); return false;" >
                                    <span class="dialog_make_btn16 show_only mobile_big smallbtns ui-corner-all"
                                    {if isset($item->object_is_editable_msg)} title="{$item->object_is_editable_msg} ({$item->id})"{/if}>
                                        {if $dot3p}{$i.basic_view}{/if}
                                    </span>
                                </a>
                            {else}
                                <span onclick="{$tabname}.open('{$item->id}', true, this.title, '{$reportname}', this);" class="dialog_make_btn16 show_only mobile_big smallbtns ui-corner-all"
                                    {if isset($item->object_is_editable_msg)} title="{$item->object_is_editable_msg} ({$item->id})"{/if}>
                                        {if $dot3p}{$i.basic_view}{/if}
                                </span>
                            {/if}
                            {if $is_deleteable}
                                <span onclick="{$tabname}.delete_entry('{$item->id}');" class="dialog_make_btn16 delete smallbtns ui-corner-all" title="{$i.basic_delete}">{if $dot3p}{$i.basic_delete}{/if}</span>
                            {/if}
                            {/strip}
                        {/if}
                        {if isset($additional_buttons) && is_array($additional_buttons)}
                            {foreach item="additional_button" from=$additional_buttons}
                                {additional_button value=$item type=$additional_button dot3p=$dot3p}
                            {/foreach}
                        {/if}                        
                        {if $dot3p}
                                <div id="dot3_btns_{$tabname}{$item->id}"><span class="dialog_make_load16" title="Loading..."></span></div>
                            </div>
                        {/if}
                    {/if}
                    {if isset($hover_file_link) && $hover_file_link}
                        {$hover_file_link_value=$hover_file_link.0}
                        {if isset($hover_file_link.2)}
                            {* We have 2 file links *}
                            {filelink value=$item->$hover_file_link_value type=$hover_file_link.2 row_id=$item->id show_link_depending_on_item=FALSE item=$item}
                            {$hover_file_link_value=$hover_file_link.1}
                            {filelink value=$item->$hover_file_link_value type=$hover_file_link.2 row_id=$item->id show_link_depending_on_item=TRUE item=$item}
                        {else}
                            {* We have 1 file link *}
                            {filelink value=$item->$hover_file_link_value type=$hover_file_link.1 row_id=$item->id show_link_depending_on_item=FALSE item=$item}
                        {/if}
                    {/if}
                    {if isset($open_right_link)}
                        <span onclick="{$tabname}.open_right('{$item->id}')"
                            class="dialog_make_btn16 edit_simple smallbtns ui-corner-all"
                            title="{$i.basic_view_more}"></span>
                    {/if}
                </td>

                {counter start=0 name='edtp-col' print=FALSE}
                {foreach key="whatitem" item="whatformatstyle" from=$whatview}
                    {* The current value *}
                    {assign var="current" value=$item->$whatitem}
                    {* Get data format and CSS style *}
                    {assign var="is_addi" value=FALSE}
                    {*if is_array($whatformatstyle)*}
                        {assign var="whatformat" value=$whatformatstyle[0]}
                        {assign var="whatstyle" value=$whatformatstyle[1]}
                        {if isset($whatformatstyle[2])}{assign var="is_addi" value=$whatformatstyle[2]}{/if}
                    {*else}
                        {assign var="whatformat" value=$whatformatstyle}
                        {assign var="whatstyle" value=''}
                    {/if*}
                    
                    
                    {if isset($per_row_title) && $per_row_title}
                        {assign var="headerformat" value=$header.$whatitem}
                        {if {$headerformat.lang}}
                            {assign var="title" value="`$i.{$headerformat.lang}``$main_title`"}
                        {else}
                            {assign var="title" value="`$headerformat.title``$main_title`"}
                        {/if}
                    {else}
                        {assign var="title" value=$main_title}
                    {/if}

                    {* GROUP BY AFTER FIRST ROW *}
                    {if isset($group_by) && isset($last_group_by) && (!isset($item->rowinfo) || !isset($item->rowinfo->class) || $item->rowinfo->class!='ui-widget-header' )}
                        {if !is_array($group_by) && $whatitem==$group_by && $current==$last_group_by}
                            {$whatformat = '_empty'}
                            {assign var="whatstyle" value='opacity:0; border-top:0 none; border-bottom:0 none;'}

                        {else if is_array($group_by)}
                           {foreach key="field" item="group_by_field" from=$group_by}
                                {* reset last_group_by if needed  *}
                                {if isset($group_by_field.parent)}
                                    {assign var="parent" value=$group_by_field.parent}
                                    {if $whatitem==$field && $item->$group_by.$parent.group_by!=$last_group_by.$parent}
                                        {$last_group_by.$field = null}
                                    {/if}
                                {/if}
                            {/foreach}

                            {foreach key="field" item="group_by_field" from=$group_by}
                                {if $whatitem==$field && $item->$group_by_field.group_by==$last_group_by.$field}
                                    {$whatformat = '_empty'}
                                    {assign var="whatstyle" value='opacity:0; border-top:0 none; border-bottom:0 none;'}
                                {/if}
                            {/foreach}
                        {/if}
                    {/if}


                    {* GROUP BY FIRST ROW *}
                    {assign var="whattdclass" value=''}
                    {if isset($group_by)}
                        {if !is_array($group_by) && $whatitem==$group_by}
                            {assign var="whattdclass" value='group_by'}
                        {/if}
                    {/if}

                    {* Print the heading of the data cell *}
                    {assign var="select_row" value="$('tr.ui-clkselected').removeClass('ui-clkselected');$(this).parent('tr').addClass('ui-clkselected');"}
                    {counter name='edtp-col' assign='edtpcol'}
                    {if $is_editable}
                        <td class="edtp-col edtp-col-{$edtpcol} {$whattdclass}" 
                            {if isset($hover_script) && $hover_script}onclick="{$select_row}{$tabname}.hover_update('{$item->id}');" {/if}
                            ondblclick="{$tabname}.{$ondblclick_action}('{$item->id}', false, '');" 
                            title="{$title}" 
                            style="{$whatstyle}" >
                    {else}
                        <td class="edtp-col edtp-col-{$edtpcol} {$whattdclass}" 
                            {if isset($hover_script) && $hover_script}onclick="{$select_row}{$tabname}.hover_update('{$item->id}');" {/if}
                            title="{$title}{if isset($item->object_is_editable_msg)} [{$item->object_is_editable_msg}]{/if}" 
                            style="{$whatstyle}" >
                    {/if}
                    
                    {if isset($hover_link) && $hover_link}
                        <a href="{str_replace('<#item_id#>', $item->id, $hover_link)}">
                            <div>
                    {/if}

                    {if $is_addi}
                        {addiview field=$whatformat value=$current item=$item nolink=TRUE}
                    {else}
                        {if is_object($current) && $current instanceof UsDateTime}
                            {$current->us_date_time()|truncate:$whatformat}
                        {elseif is_object($current) && $current instanceof UsDate}
                            {$current->us_date()|truncate:$whatformat}
                        {elseif is_object($current) && $current instanceof UsTime}
                            {$current->us_stime()|truncate:$whatformat}
                        {elseif $whatformat eq 'bool'}
                            {boolview value=$current}
                        {elseif $whatformat eq 'boolcall'}
                            {boolview value=$current call=true}
                        {elseif $whatformat eq 'checkbox'}
                            {include file='r/checkbox-only.tpl' name="edtp-table-`$tabname`-`$whatitem`-`$item->id`" default=$current 
                            filter_extra="onclick=\"`$checkbox_action`(this, {$item->id});\""}
                        {elseif $whatformat eq 'boolhideno'}
                            {boolview value=$current hideno=1}
                        {elseif $whatformat eq 'isnew'}
                            {isnew value=$current}
                        {elseif $whatformat eq 'urgent'}
                            {urgent value=$current}
                        {elseif $whatformat eq 'float'}
                            {isfloat value=$current}
                        {elseif $whatformat eq 'longfloat'}
                            {islongfloat value=$current}
                        {elseif $whatformat eq 'turnover_rate'}
                            {isturnover_rate value=$current}
                        {elseif $whatformat eq 'turnover_rate_%'}
                            {isturnover_rate value=$current add_percent_sign=true}
                        {elseif $whatformat eq 'is_add2indirect_costs'}
                            {is_add2indirect_costs value=$current}
                        {elseif $whatformat eq 'isprivate'}
                            {isprivate value=$current}
                        {elseif $whatformat eq 'is_calc_from_expenses'}
                            {is_calc_from_expenses value=$current}
                        {elseif $whatformat eq 'is_included_in_time_sheet'}
                            {is_included_in_time_sheet value=$current}
                        {elseif $whatformat eq 'vacation_type'}
                            {vacation_type value=$current}
                        {elseif $whatformat eq 'is_active_on_start_timelogging'}
                            {is_active_on_start_timelogging value=$current}
                        {elseif $whatformat eq 'project_offers_status'}
                            {project_offers_status value=$current}   
                        {elseif $whatformat eq 'payment_fee_type'}
                            {payment_fee_type value=$current}   
                        {elseif $whatformat eq 'payment_status'}
                            {payment_status value=$current}   
                        {elseif $whatformat eq 'security'}
                            {security value=$current}                             
                        {elseif $whatformat eq 'fees_is_calcexp'}
                            {fees_is_calcexp value=$current}
                        {elseif $whatformat eq 'isfile'}
                            {filelink value=$current}                             
                        {elseif $whatformat eq 'ismsg'}
                            {filelink value=$current type='msg'}
                        {elseif substr($whatformat, 0, 18) eq 'additional_button:'}
                            {additional_button value=$item type=substr($whatformat, 18) portlet_id=$portlet_id}
                        {elseif $whatformat eq '_empty'}
                            {*}empty value{*}
                        {elseif isset($current)}
                            {htmlspecialchars($current)|truncate:$whatformat|nl2br}
                        {/if}
                    {/if}
                    
                    {if isset($hover_link) && $hover_link}
                            </div>
                        </a>
                    {/if}                    
                    
                    </td>
                {/foreach}
                <td style="display:none;"></td>{* Fix faulty jQuery UI selectable, which always calles the click of the last cell, if there is no onlick in a cell *}
            </tr>
            {* Save the current group by - if *}
            {if isset($group_by) && (!isset($item->rowinfo) || !isset($item->rowinfo->class) || $item->rowinfo->class!='ui-widget-header' )}
                {if !is_array($group_by)}
                    {assign var="last_group_by" value=$item->$group_by}
                {else}
                    {foreach key="field" item="group_by_field" from=$group_by}
                        {$last_group_by.$field = $item->$group_by_field.group_by}
                    {/foreach}
                {/if}
            {/if}
        {foreachelse}
            <tr>
                <td colspan="{count array=$whatview add=1}">{$no_rows_msg}</td>
            </tr>
        {/foreach}
    </tbody>
</table>
</div>