<?php
/**
 * CodeIgniter library
 *
 * @category CodeIgniter
 * @license /_licenses/untermStrich/agb_normal.pdf
 * @author Christian Koller
 * @copyright 2009 untermStrich
 * @package libraries
 * @version $Id: My_Lang.php 22685 2021-06-18 06:52:57Z christian $
*/

if (!defined('BASEPATH')) exit('No direct script access allowed');

/**
 * Provides additional methodes to the Language class
 */
class My_Lang extends CI_Lang
{

    /**
     * Fetch a single line of text from the language array and allows to add vars like sprintf
     *
     * @param	string	$key        The language line (key)
     * @param	string	$args...    Args to add
     * @return	string
     * 
     * @access	public
     */
    public function linef($key)
    {
        $line = $this->line($key);
        if ($line) {
	    $args = func_get_args();
	    array_shift($args); //Removes the line (Important for PHP7)
            if ($line===ucfirst($key)) {
                return $line.' - '.implode(' | ',$args);
            } else {
                return vsprintf($line, $args);
            }
        } else {
            return $line;
        }
    }
    
    /**
     * Fetch a single line of text from the language array or return the key instead
     *
     * @param	string  $key        Optional Line key
     * @param   boolean $force_text Optional Return the text if not string found
     * @return  string
     */
    public function line($key='', $force_text=true)
    {
        $str = parent::line($key);
        if ($str || $force_text===false) {
            return $str;
        } else {
            return ucfirst($key);
        }
    }

    /**
     * Fetch a single line of text from the language array or return the key instead 
     * 
     * WITHOUT &shy;
     *
     * @param	string  $key        Optional Line key
     * @param   boolean $force_text Optional Return the text if not string found
     * @return  string
     */
    public function line_ns($key='', $force_text=true)
    {
        $str = parent::line($key);
        if ($str || $force_text===false) {
            return str_ireplace('&shy;', '', $str);
        } else {
            return ucfirst($key);
        }
    }

} // END class My_Language