<?php

/**
 * CodeIgniter library
 *
 * @category CodeIgniter
 * @license /_licenses/untermStrich/agb_normal.pdf
 * @author Christian Koller
 * @copyright 2009 untermStrich
 * @package libraries
 * @version $Id: My_Security.php 22685 2021-06-18 06:52:57Z christian $
 */
if (!defined('BASEPATH'))
    exit('No direct script access allowed');

/**
 * Prevents CI from replacing CDATA fields used by XAjAX
 */
class My_Security extends CI_Security
{
    /**
     * Set Cross Site Request Forgery Protection Cookie
     *
     * @return	string
     */
    protected function _csrf_set_hash()
    {
        if ($this->_csrf_hash == '') {
            // If the cookie exists we will use it's value.  
            // We don't necessarily want to regenerate it with
            // each page load since a page could contain embedded 
            // sub-pages causing this feature to fail
            if (isset($_COOKIE[$this->_csrf_cookie_name]) &&
                    $_COOKIE[$this->_csrf_cookie_name] != '') {
                return $this->_csrf_hash = $_COOKIE[$this->_csrf_cookie_name];
            }

            return $this->_csrf_hash = md5($this->get_token());
        }

        return $this->_csrf_hash;
    }

    /**
     * Get a secure token
     * 
     * @return  string
     */
    public function get_token()
    {
        return bin2hex(random_bytes(20));
    }

    /**
     * @ignore
     */
    protected $_never_allowed_str = array(
        'document.cookie' => '[removed]',
        'document.write' => '[removed]',
        '.parentNode' => '[removed]',
        '.innerHTML' => '[removed]',
        'window.location' => '[removed]',
        '-moz-binding' => '[removed]',
        '<!--' => '&lt;!--',
        '-->' => '--&gt;',
        #'<![CDATA[' => '&lt;![CDATA['
    );
    
    /**
     * Do Never Allowed
     *
     * A utility function for xss_clean()
     *
     * @param   string  $str    String
     * @return  string
     */
    public function do_my_never_allowed($str)
    {
        $str = remove_invisible_characters($str);
        
        $naughty = 'alert|applet|audio|basefont|base|behavior|bgsound|blink|body|embed|expression|form|frameset|frame|head|html|ilayer|iframe|input|isindex|layer|link|meta|object|plaintext|style|script|textarea|title|video|xml|xss';
        $str = preg_replace_callback('#<(/*\s*)('.$naughty.')([^><]*)([><]*)#is', array($this, '_sanitize_naughty_html'), $str);
        
        return $this->_do_never_allowed($str);
    }
    
    /**
     * Do Never Allowed for E-Mail
     *
     * A utility function for xss_clean()
     *
     * @param   string  $str    String
     * @return  string
     */
    public function do_my_never_allowed_email($str)
    {
        $str = remove_invisible_characters($str);
        
        return $this->_do_never_allowed($str);
    }
    
    /**
     * Do Never Allowed for HTML-Documents
     *
     * A utility function for xss_clean()
     *
     * @param   string  $str    String
     * @return  string
     */
    public function do_my_never_allowed_html($str)
    {
        $str = remove_invisible_characters($str);
        
        //Allow html comments
        try {
            unset($this->_never_allowed_str['<!--']);
            unset($this->_never_allowed_str['-->']);
        } catch (Exception $exc) {
            
        }
        
        return $this->_do_never_allowed($str);
    }
    
    /**
     * JS Image Removal
     *
     * Callback function for xss_clean() to sanitize image tags
     * This limits the PCRE backtracks, making it more performance friendly
     * and prevents PREG_BACKTRACK_LIMIT_ERROR from being triggered in
     * PHP 5.2+ on image tag heavy strings
     * 
     * Allow base64 for images (Used in sendmail)
     *
     * @param	array
     * @return	string
     */
    protected function _js_img_removal($match)
    {
        $attributes = $this->_filter_attributes(str_replace(array('<', '>'), '', $match[1]));

        return str_replace($match[1], preg_replace("#src=.*?(alert\(|alert&\#40;|javascript\:|livescript\:|mocha\:|charset\=|window\.|document\.|\.cookie|<script|<xss\s*,)#si", "", $attributes), $match[0]);
    }
    
}// END class My_Input